/*
 * DAOausgebenSchueler.cs
 */
using System;

using Beringer.oopBuch.ModelKlassen;

namespace Beringer.oopBuch.DaoKlassen
{
    /**
    <summary>
    DAO-Klasse für das Aufbereiten von Schülerdaten und für das Schreiben dieser
    Daten auf die Standardausgabe (= Bildschirm).
    <p>
    Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
    </p>
    <p>
    Aufbereiten von Schülerdaten und formatiert ausgeben in 1 Zeile.
    </p>
    <p>
    Methoden 'oeffnen' und 'schliessen' sind nicht notwendig, weil
    'Console.Out' (= Standardausgabe Bildschirm) standardmäßig immer geöffnet 
    ist.
    </p>

    <p>
    Seit: August 2009 <br />
    Änderung: August 2014 (neue Namensbereiche 'ModelKlassen' und 'DaoKlassen')
    </p>

    <seealso cref="Schueler" />
    <seealso cref="DAOlesenSchueler" />
    <seealso cref="DAOerfassenSchueler" />
    <seealso cref="Beringer.oopBuch.EinAusgabe.LesenAusgeben" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    <ul>
      <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
      <li>Der Einfachheit halber sind die auszugebenden Texte fix codiert 
          (d.h. keine Internationalisierung!).</li>
    </ul>
    </remarks>
    */
    public class DAOausgebenSchueler
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /// Maximale Anzahl Zeilen pro Seite
        private static readonly int     ZZMAX;

        /// Simulieren "Vorschub auf neue Seite"
        private static readonly String  NEUESEITE;

        /// Überschrift
        private static readonly String  UEBER;

        /// Format für die Ausgabezeile
        private static readonly String  F_AB;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        /// Datenobjekt für die auszugebenden Daten
        private Schueler  schueler;
        /// Zeilenzähler
        private int       zz;
    
        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenkonstruktor (Klassen-Initialisierungsblock).
        </summary>
        <remarks>
        Die Werte für nicht initialisierte Klassenkonstante könnten von externen
        Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder 
        von einer lokalisierten Resource-Datei).
		<p>
        Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
		</p>
        </remarks>
        */
        static DAOausgebenSchueler()
        {
            ZZMAX     = 5;      // Zum Testen mindestens 4 Datensätze erfassen

            NEUESEITE = "___________________________________________________";
            UEBER     = "NAME                      NOTENDURCHSCHNITT  KLASSE";
            F_AB      = "{0,-25}       {1:f2}         {2}";
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Am Beginn wird das Ende einer Seite simuliert, um auf einfache Art eine 
        Überschrift auf der ersten Seite zu erzwingen.
        </remarks>
        */
        public DAOausgebenSchueler()
        {
            zz = ZZMAX;
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Set-Property für die Daten des auszugebenden Datensatzes.
        </summary>
        */
        public Schueler SchuelerDaten
        {
            set
            {
                schueler = value;
            }
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden für Dateiverarbeitung
        // ---------------------------------------------------------------------

        /**
        <summary>
        Schreibt 1 Zeile mit den gewünschten Schülerdaten.
        <p>
        Die Datenfelder werden formatiert ausgegeben.
        Gegebenenfalls erfolgt ein (simulierter) Seitenvorschub und die Ausgabe 
        einer Überschrift.
        </p>
        </summary>
        */
        public void SchreibenZeile()
        {
            if (zz >= ZZMAX)
            {
                Console.WriteLine(NEUESEITE);
                Console.WriteLine();
                Console.WriteLine(UEBER);
                Console.WriteLine();
                zz = 2;
            }
            Console.WriteLine(F_AB, schueler.Name,
                                    schueler.Notenschnitt,
                                    schueler.Klasse);
            zz = zz + 1;
        }
    }
}
