/*
 * ErfassenSchreiben.cs
 */
using System;

using Beringer.oopBuch.DaoKlassen;

namespace Beringer.oopBuch.EinAusgabe
{
    /**
    <summary>
    Einfache Konsolenanwendung für Erfassen (Lesen) von Daten über die 
    Standardeingabe (= Tastatur) und Schreiben der Daten in eine Datei.
    <p>
    Der Dateiname wird beim Aufruf als Parameter übergeben.
    </p>

    <p>
    Seit: November 2004 <br />
    Version: August 2007 (eigene Programmklasse) <br />
    Änderung: August 2014 (neuer Namensbereich 'DaoKlassen')
    </p>

    <seealso cref="DAOerfassenSchueler" />
    <seealso cref="DAOschreibenSchueler" />
    <seealso cref="LesenAusgeben" />
    <seealso cref="ProgrammErfassenSchreiben" />

    Autor: Beringer Alfred
    </summary>
    */
    public class ErfassenSchreiben
    {
        // ---------------------------------------------------------------------
        // Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Öffentliche Methode zum Starten der Applikation.
        </summary>

        <param name="dateiname">
          Name der Ausgabedatei (als Zeichenkette)
        </param>
        */
        public void Start(String dateiname)
        {
            DAOerfassenSchueler   eingabedaten = new DAOerfassenSchueler();
            DAOschreibenSchueler  ausgabedaten = new DAOschreibenSchueler();
            bool                  ok;

            // Am Beginn der Applikation Datei öffnen
            ok = ausgabedaten.Oeffnen(dateiname);
            if (!ok)
            {
                Console.WriteLine("Fehler beim Öffnen der Datei " + dateiname);
                return;
            }

            ok = eingabedaten.Lesen();       // Lesen Standardeingabe (Tastatur)
            while (!eingabedaten.Eof)
            {
                if (ok)
                {
                    // nur zum Testen:
                    Console.WriteLine(eingabedaten.SchuelerDaten);

                    // Daten übernehmen
                    //    (d.h. Datenobjekt füllen mit den Eingabedaten)
                    ausgabedaten.SchuelerDaten = eingabedaten.SchuelerDaten;

                    ausgabedaten.Schreiben();     // Schreiben eines Datensatzes
// Schreiben eines Datensatzes im C#-Format:
//                    ausgabedaten.SchreibenCSproprietaer();
                }
                else
                {
                    Console.WriteLine("Fehler beim Lesen");
                }
                ok = eingabedaten.Lesen();   // Lesen Standardeingabe (Tastatur)
            }

            // Am Ende der Applikation Datei schließen
            ausgabedaten.Schliessen();
        }
    }
}
