/*
 * LesenAusgeben.cs
 */
using System;

using Beringer.oopBuch.DaoKlassen;

namespace Beringer.oopBuch.EinAusgabe
{
    /**
    <summary>
    Einfache Konsolenanwendung für Lesen von Daten aus einer Datei und 
    Ausgeben (Schreiben) der Daten auf die Standardausgabe (= Bildschirm).
    <p>
    Der Dateiname wird beim Aufruf als Parameter übergeben.
    </p>

    <p>
    Seit: November 2004 <br />
    Version: August 2007 (eigene Programmklasse) <br />
    Änderung: August 2014 (neuer Namensbereich 'DaoKlassen')
    </p>

    <seealso cref="DAOlesenSchueler" />
    <seealso cref="DAOausgebenSchueler" />
    <seealso cref="ErfassenSchreiben" />
    <seealso cref="ProgrammLesenAusgeben" />

    Autor: Beringer Alfred
    </summary>
    */
    public class LesenAusgeben
    {
        // ---------------------------------------------------------------------
        // Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Öffentliche Methode zum Starten der Applikation.
        </summary>

        <param name="dateiname">
          Name der Eingabedatei (als Zeichenkette)
        </param>
        */
        public void Start(String dateiname)
        {
            DAOlesenSchueler     eingabedaten = new DAOlesenSchueler();
            DAOausgebenSchueler  ausgabedaten = new DAOausgebenSchueler();
            bool                 ok;

            // Am Beginn der Applikation Datei öffnen
            ok = eingabedaten.Oeffnen(dateiname);
            if (!ok)
            {
                Console.WriteLine("Fehler beim Öffnen der Datei " + dateiname);
                return;
            }

            ok = eingabedaten.Lesen();                        // Lesen Datensatz
// Lesen eines Datensatzes im C#-Format:
//            ok = eingabedaten.LesenCSproprietaer();
            while (!eingabedaten.Eof)
            {
                if (ok)
                {
                    // Daten übernehmen
                    //    (d.h. Datenobjekt füllen mit den Eingabedaten)
                    ausgabedaten.SchuelerDaten = eingabedaten.SchuelerDaten;
                    ausgabedaten.SchreibenZeile();          // 1 Zeile schreiben
                }
                else
                {
                    Console.WriteLine("Fehler beim Lesen");
                }
                ok = eingabedaten.Lesen();                    // Lesen Datensatz
// Lesen eines Datensatzes im C#-Format:
//                ok = eingabedaten.LesenCSproprietaer();
            }

            // Am Ende der Applikation Datei schließen
            eingabedaten.Schliessen();

#if DEBUG
            Console.ReadLine();
#endif
        }
    }
}
