/*
 * Zaehler.cs
 */

namespace Beringer.oopBuch.EinfacheBsp
{
    /**
    <summary>
    Instanzen dieser Klasse repräsentieren einen einfachen Zähler,
    der (um 1) erhöht und (auf 0) zurückgesetzt werden kann.
      <example>
         Anwendungsbeispiel:
         <pre>
           Zaehler  z;
           z = new Zaehler();
           z.Erhoehen();
           Console.WriteLine(z.Wert);
         </pre>
      </example>

    <p>
    Seit: November 2004
    </p>

    <seealso cref="Beringer.oopBuch.EinfacheBspProgramme.ProgrammZaehler"/>
    <seealso cref="Zaehler2" />
    <seealso cref="Zaehler3" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Zaehler
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private int  wert;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Zähler wird zurückgesetzt.
        </remarks>
        */
        public Zaehler()
        {
            Reset();
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für den aktuellen Wert des Zählers.
        </summary>
        */
        public int Wert
        {
            get
            {
                return  wert;
            }
        }

        // ---------------------------------------------------------------------
        // Öffentliche Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Erhöht den Zähler um 1.
        </summary>

        <returns>
          neuer (inkrementierter) Zählerwert
        </returns>
        */
        public int Erhoehen()
        {
            setWert( Wert + 1 );
            return  Wert;

            /* Einfacher, aber mit weniger Kapselung:
                   wert = wert + 1;
                   return wert;
            */
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Setzt den Zähler zurück auf 0.
        </summary>
        */
        public void Reset()
        {
            setWert(0);

            /*
             Einfacher, aber mit weniger Kapselung:
                   wert = 0;
            */
        }

        // ---------------------------------------------------------------------
        // Nichtöffentliche Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ändert den Wert des Zählers - es erfolgt jedoch keine Prüfung auf 
        Plausibilität!
        </summary>
        <remarks>
        Die Zugriffsberechtigung wurde für die zusätzliche Beispielklasse 
        Zaehler3 auf protected geändert!
        </remarks>

        <param name="value">
          neuer (beliebiger) Zählerwert
        </param>
        */
        virtual protected void setWert(int value)
        {
            this.wert = value;
        }
    }
}
