/*
 * Zaehler2.cs
 */

namespace Beringer.oopBuch.EinfacheBsp
{
    /**
    <summary>
    Instanzen dieser Klasse repräsentieren einen einfachen zweistelligen
    Zähler, der (um 1) erhöht und (auf 0) zurückgesetzt werden kann. <br />
    Der Zähler beginnt nach 99 (= Standardwert für Endwert) wieder bei 0, d.h. 
    bei 100 erfolgt ein Überlauf.

    <p>
    Seit: November 2004
    </p>

    <seealso cref="Beringer.oopBuch.EinfacheBspProgramme.ProgrammZaehler2"/>
    <seealso cref="Zaehler" />
    <seealso cref="Zaehler3" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Zaehler2
    {
        // ---------------------------------------------------------------------
        // Klassenattribute
        // ---------------------------------------------------------------------

        /**
        <summary>
        Der größte Wert, den der Zähler annehmen kann, danach muss der Zähler
        zurückgesetzt werden.
        </summary>
        */
        private static int  endwert = 99;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private int  wert;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Zähler wird zurückgesetzt
        </remarks>
        */
        public Zaehler2()
        {
            Reset();
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für den aktuellen Wert des Zählers.
        </summary>
        */
        public int Wert
        {
            get
            {
                return  wert;
            }
        }

        // ---------------------------------------------------------------------
        // Öffentliche Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Erhöht den Zähler um 1 - bei Überlauf wird Zähler zurückgesetzt.
        </summary>

        <returns>
          neuer (inkrementierter) Zählerwert
        </returns>
        */
        public int Erhoehen()
        {
            setWert(Wert+1);    // Kapselung notwendig, um Überlauf zu erkennen!
            return  Wert;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Setzt den Zähler zurück auf 0.
        </summary>
        */
        public void Reset()
        {
            setWert(0);
        }

        // ---------------------------------------------------------------------
        // Nichtöffentliche Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ändert den Wert des Zählers.
        </summary>

        <param name="value">
          neuer Zählerwert, der zwischen 0 und Endwert liegen muss
        </param>
        */
        private void setWert(int value)
        {
            this.wert = value;
            if ((this.wert > endwert) || (this.wert < 0))
            {
                Reset();
            }
        }

        // ---------------------------------------------------------------------
        // Klassenmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert den aktuellen Endwert für den Zähler.
        </summary>

        <returns>
          aktueller Endwert
        </returns>
        */
        public static int Endwert()
        {
            return  endwert;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Definiert einen neuen Endwert für den Zähler - es erfolgt jedoch keine 
        Prüfung auf Plausibilität!
        </summary>

        <param name="value">
          neuer (beliebiger) Endwert
        </param>
        */
        public static void Endwert(int value)
        {
            endwert = value;
        }
    }
}
