/*
 * DialogModalKindfensterV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

using Beringer.oopBuch.EinfacheBsp;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Modales Dialogfenster. <br />
    Textfeld für Eingabe mit 2 Buttons ('OK' und 'Abbruch').
    <p>
    Model-Klasse: 'AutoM'
                  (dient als Fachobjekt zum Testen dieser Fensterklasse)
    </p>

    <p>
    Seit: August 2005
    </p>

    <seealso cref="DialogModalElternfensterV" />
    <seealso cref="Beringer.EinfacheBsp.AutoM" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Man beachte, dass für die Internationalisierung dieser Klasse für alle 
    Texte bzw. für alle Beschriftungen bereits Klassenkonstante definiert 
    wurden, dass aber diese der Einfachheit halber im Klassenkonstruktor fix 
    vorgegeben sind.
    </remarks>
    */
    public class DialogModalKindfensterV : Form
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /// Text für Titelzeile
        private static readonly String  FENSTERTITEL;

        /// Beschriftung zu Textfeld (Text für Eingabeaufforderung)
        private static readonly String  LABELTEXT;

        /// Text für Schaltfläche1 'OK'
        private static readonly String  BUTTON1;

        /// Text für Schaltfläche2 'Abbruch'
        private static readonly String  BUTTON2;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private Label    label   = new Label();
        private TextBox  text    = new TextBox();
        private Button   button1 = new Button();
        private Button   button2 = new Button();

        private Form     elternfenster;

        /// Inhalt des Textfelds, der ans Elternfenster übergeben werden soll.
        private String   rueckgabe = "";

        /// Fachobjekt (zum Testen)
        private AutoM    auto = new AutoM();

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenkonstruktor. <br />
        Initialisieren der Klassenkonstanten. <br />
        Die Anfangswerte für die Klassenkonstanten werden im Allg. von 
        externen Datenquellen übernommen (z.B. von einer Ressource-Datei).
        </summary>
        <remarks>
        Der Einfachheit halber werden die Anfangswerte fix vorgegeben.
        </remarks>
        */
        static DialogModalKindfensterV()
        {
            FENSTERTITEL = "Dialog modal - Kindfenster";
            LABELTEXT    = "Bitte neue Farbe für PKW eingeben: ";

            BUTTON1      = "OK";
            BUTTON2      = "Abbruch";
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Referenz auf Elternfenster wird übernommen, falls Daten vom 
        Elternfenster benötigt werden.
        </summary>

        <param name="elternfenster">
          Elternfenster
        </param>
        */
        public DialogModalKindfensterV(Form elternfenster)
        {
            this.elternfenster = elternfenster;

            this.Text = FENSTERTITEL;
            this.Size = new Size(500, 150);

            // Standardschaltflächen definieren
            this.AcceptButton = button1;     // Eingabe mit RETURN-Taste möglich
            this.CancelButton = button2;     // Abbruch mit Taste 'ESC' möglich

            // GUI-Komponenten mit Texten initialisieren

            label.Text = LABELTEXT;
            button1.Text = BUTTON1;
            button2.Text = BUTTON2;

            // Schaltflächen

            button1.AutoSize = true;
            button1.Anchor = AnchorStyles.Right;
            button1.Margin = new Padding(20);

            button2.AutoSize = true;
            button2.Anchor = AnchorStyles.Left;
            button2.Margin = new Padding(20);

            // einzeiliges Textfeld

            text.ReadOnly = false;
            text.Width = 200;

            // Beschriftungen (Labels) rechtsbündig platzieren

            label.AutoSize = true;
            label.Anchor = AnchorStyles.None | AnchorStyles.Right;

            // Tabellen-Layout für Fensterinhalt definieren (2 Spalten)

            TableLayoutPanel pmitte = new TableLayoutPanel();
            pmitte.AutoSize = true;
            pmitte.ColumnCount = 2;
            pmitte.Dock = DockStyle.Top;
            pmitte.Padding = new Padding(20);

            // GUI-Komponenten der Windows-Tabellenform hinzufügen
            // (von links nach rechts (zeilenweise - je 2 Spalten)

            pmitte.Controls.Add(label);
            pmitte.Controls.Add(text);
            pmitte.Controls.Add(button1);
            pmitte.Controls.Add(button2);

            // GUI-Komponenten der Windows-Form hinzufügen

            this.Controls.Add(pmitte);

            // Ereignisse für GUI-Komponenten definieren

            button1.Click += new EventHandler(button1event);
            button2.Click += new EventHandler(button2event);
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für den Inhalt des Textfelds, der ans Elternfenster 
        übergeben werden soll.
        </summary>
        */
        public String Rueckgabe
        {
            get
            {
                return rueckgabe;
            }
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Ereignis.
        </summary>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void button1event(Object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            rueckgabe = text.Text;
            auto.Ausgeben(this.DialogResult + " gedrueckt *" + rueckgabe + '*');
            fensterende();
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Delegate-Methode für Ereignis.
        </summary>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void button2event(Object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            rueckgabe = "";
            auto.Ausgeben(this.DialogResult + " gedrueckt");
            fensterende();
        }

       // ---------------------------------------------------------------------
 
       private void fensterende()
        {
            text.Text = "";
            this.Hide();
        }
    }
}
