/*
 * FensterMausbewegungenV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Hauptfenster für Mausbewegungen. <br />
    Die Mauskoordinaten werden in einer mehrzeiligen TextBox gespeichert.

    <p>
    Seit: August 2005
    </p>

    Autor: Beringer Alfred
    </summary>
    */
    public class FensterMausbewegungenV : Form
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        private static readonly String  NeueZeile = Environment.NewLine;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private TextBox  ausgabe = new TextBox();

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        */
        public FensterMausbewegungenV()
        {
            this.Text = "Fenster mit Mausbewegungen";
            this.Size = new Size(400, 260);

            // mehrzeiliges Textfeld
            ausgabe.Multiline = true;
            ausgabe.Text = "Start" + NeueZeile;
            ausgabe.ScrollBars = ScrollBars.Vertical;
            ausgabe.ReadOnly = true;
            ausgabe.Size = new Size(200, 200);

            // GUI-Komponenten in der Windows-Form platzieren 

            ausgabe.Location = new Point(50, 10);

            // GUI-Komponenten der Windows-Form hinzufügen

            this.Controls.Add(ausgabe);

            // Ereignisse für GUI-Komponenten definieren

            this.MouseMove += new MouseEventHandler(mouseMoved);
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Ereignis. <br />
        Mauskoordinaten holen und im Textbereich ablegen
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Maus-Ereignis)
        </param>
        */
        private void mouseMoved(Object sender, MouseEventArgs e)
        {
          int x = e.X;
          int y = e.Y;

          ausgabe.AppendText("x=" + x + "  y=" + y + NeueZeile);
        }
    }
}
