/*
 * FensterMenuV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

using Beringer.oopBuch.EinfacheBsp;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Hauptfenster mit Menüzeile, Menüs und Untermenüs.
    <p>
    Model-Klasse: 'AutoM'
                  (dient als Fachobjekt zum Testen dieser Fensterklasse)
    </p>

    <p>
    Seit: August 2005 <br />
    Version: August 2006 (.NET 2.0)
    </p>

    <seealso cref="AutoM" />
    <seealso cref="StandardFensterV" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Version für .NET 2.0 (Menüsystem mit der Hauptklasse 'MenuStrip'). <br />
    'MenuStrip' ist ein (neues) Steuerelement, während 'MainMenu' ein Teil 
    des Fensters ist! <br />
    Das bedeutet, dass 'MenuStrip' zur Client-Area gehört und theoretisch
    überall platziert werden könnte (es sollte natürlich immer ganz oben 
    stehen). <br />
    'MainMenu' gehört nicht zur Client-Area und steht immer oberhalb dieser.
    </remarks>
    */
    public class FensterMenuV : Form
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private MenuStrip          menuezeile   = new MenuStrip();

        private ToolStripMenuItem  menue1       = new ToolStripMenuItem("Auto");
        private ToolStripMenuItem  menueItem11  = new ToolStripMenuItem("&neues Auto");
        private ToolStripMenuItem  menueItem12  = new ToolStripMenuItem("&raufschalten");
        private ToolStripMenuItem  menueItem13  = new ToolStripMenuItem("r&unterschalten");
        private ToolStripMenuItem  menueItem14  = new ToolStripMenuItem("be&schleunigen");
        private ToolStripMenuItem  menueItem15  = new ToolStripMenuItem("&bremsen");

        private ToolStripMenuItem  menue2       = new ToolStripMenuItem("Menü2");
        private ToolStripMenuItem  menueItem21  = new ToolStripMenuItem("1. Menüeintrag Menü2");
        private ToolStripMenuItem  checkItem22  = new ToolStripMenuItem("2. Check-M2");
        private ToolStripMenuItem  checkItem23  = new ToolStripMenuItem("3. Check-M2");

        private ToolStripMenuItem  menue24      = new ToolStripMenuItem("4. Untermenü Menü2");
        private ToolStripMenuItem  menueItem241 = new ToolStripMenuItem("1. Menüeintrag Menü24");
        private ToolStripMenuItem  menueItem242 = new ToolStripMenuItem("2. Menüeintrag Menü24");

        private ToolStripMenuItem  radioItem25  = new ToolStripMenuItem("5. Radio-M2");
        private ToolStripMenuItem  radioItem26  = new ToolStripMenuItem("6. Radio-M2");
        private ToolStripMenuItem  radioItem27  = new ToolStripMenuItem("7. Radio-M2");

        private ToolStripMenuItem  menue3       = new ToolStripMenuItem("Hilfe");
        private ToolStripMenuItem  menueItem31  = new ToolStripMenuItem("&Hilfethemen");
        private ToolStripMenuItem  menueItem32  = new ToolStripMenuItem("&Info...");

        private Button             button       = new Button();

        /// Fachobjekt (zum Testen)
        private AutoM  auto = new AutoM();

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Der Menü-Aufbau, das Initialisieren der einzelnen GUI-Komponenten 
        und das Anmelden der Ereignisse wird in private Methoden ausgelagert.
        </remarks>
        */
        public FensterMenuV()
        {
            this.Text = "Fenster mit Menü";
            this.Size = new Size(400, 260);

            // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
            this.AcceptButton = button;

            menuBarErstellen();
            initButton();

            // GUI-Komponenten der Windows-Form hinzufügen

            this.Controls.Add(menuezeile);      // Menüzeile in Fenster einfügen
            this.Controls.Add(button);

            initListener();
        }

        // ---------------------------------------------------------------------
        // Private Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Erzeugen der Menüzeile.
        </summary>
        <remarks>
        Die Reihenfolge der Hinzufügungen ('Add') ist wesentlich.
        </remarks>
        */
        private void menuBarErstellen()
        {
            // Menüpunkte in Menü1 einfügen
            menue1.DropDownItems.Add(menueItem11);
            menue1.DropDownItems.Add(menueItem12);
            menue1.DropDownItems.Add(menueItem13);
            menue1.DropDownItems.Add(menueItem14);
            menue1.DropDownItems.Add(menueItem15);

            // Menüpunkte in Menü2 einfügen
            menue2.DropDownItems.Add(menueItem21);
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(checkItem22);
            menue2.DropDownItems.Add(checkItem23);
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(menue24);          // Untermenü24 als Menüpunkt einfügen
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(radioItem25);
            menue2.DropDownItems.Add(radioItem26);
            menue2.DropDownItems.Add(radioItem27);

            // Menüpunkte in Untermenü24 einfügen
            menue24.DropDownItems.Add(menueItem241);
            menue24.DropDownItems.Add(menueItem242);

            // Menüpunkte in Menü3 einfügen
            menue3.DropDownItems.Add(menueItem31);
            menue3.DropDownItems.Add(menueItem32);

            // Menüs in Menüzeile einfügen
            menuezeile.Items.Add(menue1);                 
            menuezeile.Items.Add(menue2);
            menuezeile.Items.Add(menue3);

            /* Vor jedes MenuItem-Objekt kann ein Hakerl gesetzt werden.
               Da es keine eigenen Menü-Checkbox-Objekte gibt, muss die 
               Funktionalität eines solchen Menüeintrags selbst programmiert 
               werden, d.h. 
                 1.) Jeder als Checkbox vorgesehene Menüeintrag muss bei einem 
                     EventHandler angemeldet werden.
                 2.) In der entsprechenden Ereignisbehandlungsmethode muss der 
                     betroffene Menüeintrag explizit gesetzt werden.
             */

            // Initialisierung der true-Checkbox-Menüeinträge.
            // 'checkItem23' muss nicht gesetzt werden, da 'false' der 
            // Standardwert für ein Hakerl ist.
            // 'checkItem23' ist somit ein normales MenuItem-Objekt ohne Hakerl.
            checkItem22.Checked = true;

            /* Vor ein ToolStripMenuItem-Objekt kann nur ein Hakerl aber kein 
               Radiobutton-Punkt gesetzt werden!
               Da es keine eigenen Menü-Radiobutton-Objekte gibt, muss die 
               Funktionalität einer Button-Gruppe selbst programmiert werden, 
               d.h. 
               1.) Jeder als Radiobutton vorgesehene Menüeintrag muss bei einem 
                   EventHandler angemeldet werden.
               2.) In allen entsprechenden Ereignisbehandlungsmethoden muss 
                   jeder als Radiobutton vorgesehene Menüeintrag explizit 
                   gesetzt werden.
                   Bem.: Da 'false' der Standardwert für Hakerl ist, muss 
                         bei der Initialisierung nur der 1. true-Menüeintrag 
                         explizit gesetzt werden.
             */

            // Initialisierung des (einzigen) true-Radiobutton-Menüeintrags
            radioItem26.Checked = true;

            /* Tasten-Hotkeys für einzelne Menüpunkte definieren.
                  Achtung: Tasten-Hotkeys werden nur unterstrichen angezeigt, 
                           wenn die ALT-Taste gedrückt wird!
               Tasten-Hotkeys funktionieren auch ohne CNTRL- bzw. STRG-Taste, 
               wenn das entsprechende Menü aufgeklappt ist.
             */
            menueItem11.ShortcutKeys = Keys.Control | Keys.N;
            menueItem12.ShortcutKeys = Keys.Control | Keys.R;
            menueItem13.ShortcutKeys = Keys.Control | Keys.U;
            menueItem14.ShortcutKeys = Keys.Control | Keys.S;
            menueItem15.ShortcutKeys = Keys.Control | Keys.B;

            menueItem31.ShortcutKeys = Keys.Control | Keys.H;
            menueItem32.ShortcutKeys = Keys.Control | Keys.I;
        }

        /**
        <summary>
        Schaltfläche initialisieren.
        </summary>
        */
        private void initButton()
        {
            button.Text = "neues Auto";
            button.AutoSize = true;
            button.Dock = DockStyle.Bottom;

            button.Font = new Font(FontFamily.GenericMonospace,
                                   32f,
                                   FontStyle.Bold);
            button.ForeColor = Color.White;
            button.BackColor = Color.Magenta;
        }

        /**
        <summary>
        Ereignisse für GUI-Komponenten definieren
        </summary>
        */
        private void initListener()
        {
          button.Click += neuEvent;

          menueItem11.Click += neuEvent;
          menueItem12.Click += mItem12event;
          menueItem13.Click += mItem13event;
          menueItem14.Click += mItem14event;
          menueItem15.Click += mItem15event;

          menueItem21.Click += mItem21event;
          checkItem22.Click += mItem22event;
          checkItem23.Click += mItem23event;      // muss auch angemeldet werden

          // nicht angemeldet: menue24

          radioItem25.Click += mItem25event;
          radioItem26.Click += mItem26event;
          radioItem27.Click += mItem27event;      // muss auch angemeldet werden

          menueItem31.Click += mItem31event;
          menueItem32.Click += mItem32event;
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Ereignis. <br />
        AutoM-Instanz erzeugen (über Menüauswahl oder über Schaltfläche).
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void neuEvent(Object sender, EventArgs e)
        {
            auto = new AutoM();
            auto.ZeigeInhalt();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü1 - Menüeintrag2.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem12event(Object sender, EventArgs e)
        {
            auto.Raufschalten();
            auto.ZeigeInhalt();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü1 - Menüeintrag3.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem13event(Object sender, EventArgs e)
        {
            auto.Runterschalten();
            auto.ZeigeInhalt();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü1 - Menüeintrag4.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem14event(Object sender, EventArgs e)
        {
            auto.Beschleunigen();
            auto.ZeigeInhalt();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü1 - Menüeintrag5.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem15event(Object sender, EventArgs e)
        {
            auto.Bremsen();
            auto.ZeigeInhalt();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem21event(Object sender, EventArgs e)
        {
            auto.Ausgeben("1. Menueeintrag (Menue2) ausgewaehlt");
            checkM23();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag2. <br />
        Der als Checkbox vorgesehene Menüeintrag muss explizit gesetzt 
        werden!
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem22event(Object sender, EventArgs e)
        {
            checkItem22.Checked = !checkItem22.Checked;
            auto.Ausgeben("2. CheckMenueeintrag (Menue2) ausgewaehlt");
            checkM23();
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag3. <br />
        Der als Checkbox vorgesehene Menüeintrag muss explizit gesetzt 
        werden!
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem23event(Object sender, EventArgs e)
        {
            checkItem23.Checked = !checkItem23.Checked;
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag5. <br />
        Alle als Radiobutton vorgesehenen Menüeinträge müssen explizit 
        gesetzt werden!
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem25event(Object sender, EventArgs e)
        {
            radioItem25.Checked = true;
            radioItem26.Checked = false;
            radioItem27.Checked = false;
            auto.Ausgeben("5. RadioMenueeintrag (Menue2) ausgewaehlt");
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag6. <br />
        Alle als Radiobutton vorgesehenen Menüeinträge müssen explizit 
        gesetzt werden!
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem26event(Object sender, EventArgs e)
        {
            radioItem25.Checked = false;
            radioItem26.Checked = true;
            radioItem27.Checked = false;
            auto.Ausgeben("6. RadioMenueeintrag (Menue2) ausgewaehlt");
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü2 - Menüeintrag7. <br />
        Alle als Radiobutton vorgesehenen Menüeinträge müssen explizit 
        gesetzt werden!
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem27event(Object sender, EventArgs e)
        {
            radioItem25.Checked = false;
            radioItem26.Checked = false;
            radioItem27.Checked = true;
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü3 - Menüeintrag1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem31event(Object sender, EventArgs e)
        {
            MessageBox.Show("Hilfethemen noch nicht installiert!",
                            "Info für MenuFenster",
                            MessageBoxButtons.YesNo,
                            MessageBoxIcon.Warning);
        }

        // ****************************************************************
        /**
        <summary>
        Methode für Ereigniswächter. <br />
        Methode für Menü3 - Menüeintrag2.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void mItem32event(Object sender, EventArgs e)
        {
            MessageBox.Show("Copyright 2005" + Environment.NewLine 
                                             + "Beringer Alfred",
                            "Info für MenuFenster",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Information);
        }

        // ---------------------------------------------------------------------
        // Hilfsmethoden zu den Ereignisbehandlungsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Prüfen der Option Menü2-Menüeintrag3.
        </summary>
        */
        private void checkM23()
        {
            if (!checkItem23.Checked)
            {
                auto.Ausgeben("   3. CheckMenueEintrag (Menue2) hat kein Hakerl");
            }
            else
            {
                auto.Ausgeben("   3. CheckMenueEintrag (Menue2) hat Hakerl");
            }
        }
    }
}
