/*
 * Hauptfenster1.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Einfaches Hauptfenster mit Ereignis für Doppel-Klick zum Schließen des 
    Fensters.

    <p>
    Seit: August 2005
    </p>

    <seealso cref="Hauptfenster2" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Hauptfenster1 : Form
    {
        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        */
        public Hauptfenster1()
        {
            this.Text = "Hauptfenster1 - Doppel-Klick";
            this.Size = new Size(400, 260);

            // Ereignisse für GUI-Komponenten definieren

            this.DoubleClick += new EventHandler(schliessenFenster);
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für "Fenster schließen".
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void schliessenFenster(Object sender, EventArgs e)
        {
            /*
               'sender' ist die Ereignisquelle (= jene Komponente, die das
                        Ereignis "Doppel-Klick" ausgelöst hat) und wird als
                        'Object'-Instanz übergeben
            */
            Form fenster = (Form)sender;

            // ...                 Aktionen vor Schließen des Fensters

            MessageBox.Show(fenster,
                            "Fenster wird geschlossen.",
                            "Info für Hauptfenster1",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Exclamation);
            fenster.Hide();
            fenster.Dispose();
            Environment.Exit(0);                             // Programm beenden
        }
    }
}
