/*
 * LabelButtonFensterV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

using Beringer.oopBuch.EinfacheBsp;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Hauptfenster mit Label und 2 Buttons.
    <p>
    Model-Klasse: 'Auto'
                  (dient als Fachobjekt zum Testen dieser Fensterklasse)
    </p>

    <p>
    Seit: August 2005 <br />
    Version: Mai 2006 (.NET 2.0)
    </p>

    <seealso cref="Beringer.EinfacheBsp.Auto" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Version für .NET 2.0 (mit 'FlowLayoutPanel'). <br />
    Je 1 EventHandler für jedes Ereignis - allerdings immer mit der gleichen 
    Ereignisbehandlungsmethode.
    </remarks>
    */
    public class LabelButtonFensterV : Form
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private Label    label          = new Label();
        private Button   schaltflaeche1 = new Button();
        private Button   schaltflaeche2 = new Button();

        private ToolTip  tips           = new ToolTip();

        /// Fachobjekt (zum Testen)
        private Auto  auto;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        */
        public LabelButtonFensterV()
        {
            this.Text = "Fenster mit Label und Buttons";
            this.Size = new Size(400, 260);

            FlowLayoutPanel  cp = new FlowLayoutPanel();
            cp.Dock = DockStyle.Fill;
            cp.Padding = new Padding(20);

            label.Text = "Auto";
            label.AutoSize = true;
            label.Anchor = AnchorStyles.None;
            label.Margin = new Padding(10);

            schaltflaeche1.Text = "&raufschalten";       // Tasten-Hotkey := 'R'
            schaltflaeche1.AutoSize = true;
            schaltflaeche1.Anchor = AnchorStyles.None;
            schaltflaeche1.Margin = new Padding(10);

            schaltflaeche2.Text = "&bremsen";            // Tasten-Hotkey := 'B'
            schaltflaeche2.AutoSize = true;
            schaltflaeche2.Anchor = AnchorStyles.None;
            schaltflaeche2.Margin = new Padding(10);

            /*
               Button-Property "UseMnemonic" ist standardmäßig true.
               Achtung: Tasten-Hotkey wird nur unterstrichen angezeigt, wenn 
                        die ALT-Taste gedrückt wird!
                        Tasten-Hotkey funktioniert auch ohne ALT-Taste.
            */

            // ToolTip (Quick-Help):
            tips.SetToolTip(schaltflaeche1, "Gang um 1 erhöhen");
            tips.SetToolTip(schaltflaeche2, "Geschwindigkeit um 10% vermindern");

            /*
               GUI-Komponenten der Windows-Form hinzufügen
                  (dafür gibt es 2 gleichwertige Möglichkeiten).
               Die Reihenfolge des Hinzufügens ist wesentlich!
            */

            cp.Controls.Add(label);               // label.Parent = cp;
            cp.Controls.Add(schaltflaeche1);      // schaltflaeche1.Parent = cp;
            cp.Controls.Add(schaltflaeche2);      // schaltflaeche2.Parent = cp;

            this.Controls.Add(cp);                // cp.Parent = this;

            /* 
               Ereignisse für GUI-Komponenten definieren.
               Je GUI-Komponente 1 Ereignis definieren und Komponente anmelden.
               Argument ist die Methode, die durchgeführt werden soll, wenn 
               das Ereignis eintritt (in diesem Beispiel das Ereignis 
               "Schaltfläche geklickt").
               Bem.:
                  In Java ist dies die entsprechende API-Methode für dieses 
                  Ereignis.
            */
            schaltflaeche1.Click += new EventHandler(aktionen);
            schaltflaeche2.Click += new EventHandler(aktionen);

            /*
               Ab .NET 2.0 ist auch folgende einfache Syntax möglich:

                 schaltflaeche1.Click += aktionen;
                 schaltflaeche2.Click += aktionen;
            */

            initFachobjekt();
        }

        /**
        <summary>
        Initialisieren des Fachobjekts.
        </summary>
        */
        private void initFachobjekt()
        {
            auto = new Auto();
            auto.Besitzer = new Person(1980, Person.MAENNLICH);
            auto.Besitzer.AendernVorname("Hugo");
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Ereignisse.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void aktionen(Object sender, EventArgs e)
        {
          if (sender == schaltflaeche1)
          {
              auto.Raufschalten();
          }
          else if (sender == schaltflaeche2)
          {
              auto.Bremsen();
          }
          auto.ZeigeInhalt();
        }
    }
}
