/*
 * MusterfensterV.cs
 */
using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;

using Beringer.oopBuch.EinfacheBsp;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Hauptfenster mit allen Standard-GUI-Komponenten.
    <p>
    Model-Klasse: 'AutoM'
                  (dient als Fachobjekt zum Testen dieser Fensterklasse)
    </p>

    <p>
    Seit: August 2005 <br />
    Version: August 2006 (.NET 2.0)
    </p>

    <seealso cref="AutoM" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Version für .NET 2.0 (mit 'FlowLayoutPanel' und 'TableLayoutPanel').<br />
    Jedes Ereignis hat seine eigene Ereignisbehandlungsmethode.
    </remarks>
    */
    public class MusterfensterV : Form
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        private static readonly String  NeueZeile = Environment.NewLine;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private Label       label        = new Label();
        private Button      button1      = new Button();
        private Button      button2      = new Button();

        private CheckBox    check1       = new CheckBox();
        private CheckBox    check2       = new CheckBox();
        private CheckBox    check3       = new CheckBox();

        private GroupBox    buttongruppe = new GroupBox();
        private RadioButton radio1       = new RadioButton();
        private RadioButton radio2       = new RadioButton();
        private RadioButton radio3       = new RadioButton();

        private Label       labTxfeld    = new Label();
        private Label       labTxbereich = new Label();

        private TextBox     textfeld     = new TextBox();
        private TextBox     textbereich  = new TextBox();

        private Label       labListe1    = new Label();
        private ListBox     liste1       = new ListBox();

        private Label       labListe2    = new Label();
        private ListBox     liste2       = new ListBox();

        private Label       labCombo     = new Label();
        private ComboBox    combo        = new ComboBox();

        private int    zaehler = 0;

        /// Fachobjekt (zum Testen)
        private AutoM  auto    = new  AutoM();

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Das Erzeugen der einzelnen Teilfenster (Panels) und das Anmelden der 
        Ereignisse wird in private Methoden ausgelagert.
        </remarks>
        */
        public MusterfensterV()
        {
            this.Text = "Musterfenster mit vordefinierten GUI-Komponenten";
            this.Size = new Size(500, 600);

            initKomponenten();
            initListener();
        }

        // ---------------------------------------------------------------------
        // Private Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Initialisieren der GUI-Komponenten
        </summary>
        */
        private void initKomponenten()
        {
            // GUI-Komponenten mit Texten initialisieren

            label.Text   = "Label";
            button1.Text = "Schaltfläche für Check";
            button2.Text = "Schaltfläche für Liste";

            check1.Text = "Check1";
            check2.Text = "Check2";
            check3.Text = "Check3";

            buttongruppe.Text = "PKW-Aktionen:";
            radio1.Text = "raufschalten";
            radio2.Text = "bremsen     ";
            radio3.Text = "check3 aus! ";

            labTxfeld.Text    = "\"return\" ... wird ignoriert! ";
            labTxbereich.Text = "\"return\" ... nur Zeilenvorschub: ";
            labListe1.Text    = "nur 1 einziges Element auswählbar: ";
            labListe2.Text    = "mehrere Elemente auswählbar: ";
            labCombo.Text     = "Das ist eine Auswahl-Komponente: ";

            textfeld.Text    = "Textfeld";
            textbereich.Text = "Textbereich-Zeile1" + NeueZeile + "Zeile2";

            liste1.Items.AddRange(new String[] {
                                                  "Liste1-Element1 ",
                                                  "Liste1-Element2 ",
                                                  "Liste1-Element3",
                                                  "Liste1-Element4",
                                                  "Liste1-Element5 ",
                                                  "Liste1-Element6 ",
                                                  "Liste1-Element7 ",
                                                  "Liste1-Element8 "
                                                } );
            liste2.Items.AddRange( new String[] {
                                                  "Liste2-Element1 ",
                                                  "Liste2-Element2 ",
                                                  "Liste2-Element3",
                                                  "Liste2-Element4"
                                                } );

            combo.Items.AddRange( new String[] {
                                                 "Combo-Element1",
                                                 "Combo-Element2",
                                                 "Combo-Element3"
                                               } );

            // Label mit Schaltflächen

            label.AutoSize = true;
            button1.AutoSize = true;
            button2.AutoSize = true;

            label.Anchor = AnchorStyles.None;
            button1.Anchor = AnchorStyles.None;
            button2.Anchor = AnchorStyles.None;

            label.Margin   = new Padding(10,0,10,0);
            button1.Margin = new Padding(10,0,10,0);
            button2.Margin = new Padding(10,0,10,0);

            // Kontrollkästchen

            check1.Checked = false;
            check2.Checked = true;
            check3.Checked = true;

            check1.AutoSize = true;
            check2.AutoSize = true;
            check3.AutoSize = true;

            check1.Anchor = AnchorStyles.None;
            check2.Anchor = AnchorStyles.None;
            check3.Anchor = AnchorStyles.None;

            check1.Margin = new Padding(10,0,10,0);
            check2.Margin = new Padding(10,0,10,0);
            check3.Margin = new Padding(10,0,10,0);

            // Optionsfelder und GroupBox
            
            radio1.Checked = false;
            radio2.Checked = false;
            radio3.Checked = true;

            radio1.AutoSize = true;
            radio2.AutoSize = true;
            radio3.AutoSize = true;

            // Optionsfelder der GroupBox hinzufügen 
            buttongruppe.Controls.Add(radio1);
            buttongruppe.Controls.Add(radio2);
            buttongruppe.Controls.Add(radio3);

            // Optionsfelder in der GroupBox platzieren
            // (dynamisches Layout (noch) nicht möglich)
            radio1.Location = new Point( 70, 20);
            radio2.Location = new Point(170, 20);
            radio3.Location = new Point(270, 20);

            buttongruppe.AutoSize = true;
            buttongruppe.Dock = DockStyle.Top;
            buttongruppe.Padding = new Padding(10);

            // einzeiliges Textfeld

            textfeld.ReadOnly = false;
            textfeld.Width = 200;

            // mehrzeiliges Textfeld

            textbereich.ReadOnly = false;
            textbereich.Multiline = true;
            textbereich.WordWrap = false;
            textbereich.AcceptsReturn = false;
            textbereich.ScrollBars = ScrollBars.Both;
            textbereich.Size = new Size(200, 80);

            // Listen

            liste1.ScrollAlwaysVisible = true;
            liste1.SelectionMode = SelectionMode.One;
            liste1.AutoSize = true;

            liste2.SelectionMode = SelectionMode.MultiExtended;
            liste2.AutoSize = true;

            combo.SelectedIndex = 0;
            combo.DropDownStyle = ComboBoxStyle.DropDownList; // nicht editierbar
            combo.AutoSize = true;

            // Beschriftungen (Labels) rechtsbündig platzieren

            labTxfeld.AutoSize    = true;
            labTxbereich.AutoSize = true;
            labListe1.AutoSize    = true;
            labListe2.AutoSize    = true;
            labCombo.AutoSize     = true;

            labTxfeld.Anchor = AnchorStyles.None | AnchorStyles.Right;
            labTxbereich.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            labListe1.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            labListe2.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            labCombo.Anchor = AnchorStyles.None | AnchorStyles.Right;

            // FlowLayout für oberen Fensterinhalt definieren 

            FlowLayoutPanel poben1 = new FlowLayoutPanel();
            FlowLayoutPanel poben2 = new FlowLayoutPanel();

            poben1.AutoSize = true;
            poben2.AutoSize = true;

            poben1.Dock = DockStyle.Top;
            poben2.Dock = DockStyle.Top;

            poben1.Padding = new Padding(20,20,20,10);
            poben2.Padding = new Padding(20,10,20,20);

            poben1.Controls.Add(label);
            poben1.Controls.Add(button1);
            poben1.Controls.Add(button2);

            poben2.Controls.Add(check1);
            poben2.Controls.Add(check2);
            poben2.Controls.Add(check3);

            // Obere GUI-Komponenten der Windows-Form hinzufügen
            // (Dock = Top: von innen nach außen,
            //              d.h. zuletzt die oberste Komponente!)

            this.Controls.Add(buttongruppe);
            this.Controls.Add(poben2);
            this.Controls.Add(poben1);

            // Tabellen-Layout für unteren Fensterinhalt definieren (2 Spalten) 

            TableLayoutPanel  pmitte = new TableLayoutPanel();

            pmitte.SuspendLayout();

            pmitte.AutoSize = true;
            pmitte.ColumnCount = 2;
            pmitte.Dock = DockStyle.Bottom;
            pmitte.Padding = new Padding(40);

            // Untere GUI-Komponenten der Windows-Tabellenform hinzufügen
            // (von links nach rechts (zeilenweise - je 2 Spalten)

            pmitte.Controls.Add(labTxfeld);
            pmitte.Controls.Add(textfeld);
            pmitte.Controls.Add(labTxbereich);
            pmitte.Controls.Add(textbereich);
            pmitte.Controls.Add(labListe1);
            pmitte.Controls.Add(liste1);
            pmitte.Controls.Add(labListe2);
            pmitte.Controls.Add(liste2);
            pmitte.Controls.Add(labCombo);
            pmitte.Controls.Add(combo);

            pmitte.ResumeLayout();

            // Untere GUI-Komponente der Windows-Form hinzufügen

            this.Controls.Add(pmitte);
        }

        /**
        <summary>
        Ereignisse für GUI-Komponenten definieren. <br />
        Jede Komponente hat ihre eigene Ereignisbehandlungsmethode.
        </summary>
        <remarks>
        Einfachere Syntax seit .NET 2.0 <br />
        Früher:  komponente.ereignis += new EventHandler(delegatemethode);
        </remarks>
        */
        private void initListener()
        {
          button1.Click              += button1event;
          button2.Click              += button2event;
          check1.Click               += check1event;
          radio1.Click               += radio1event;
          radio2.Click               += radio2event;
          radio3.Click               += radio3event;
          textfeld.Leave             += textfeldevent;
          liste1.MouseClick          += liste1event;
          combo.SelectedIndexChanged += comboevent;
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden (Delegate-Methoden)
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Schaltfläche1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void button1event(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Ausgeben("Schaltflaeche1 gedrueckt");

            if (check1.Checked)
                auto.Ausgeben("     check1 hat Hakerl");
            else
                auto.Ausgeben("     check1 leer");

            if (check2.Checked)
                auto.Ausgeben("     check2 hat Hakerl");
            else
                auto.Ausgeben("     check2 leer");

            if (check3.Checked)
                auto.Ausgeben("     check3 hat Hakerl");
            else
                auto.Ausgeben("     check3 leer");
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Schaltfläche2.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void button2event(Object sender, EventArgs e)
        {
            ListBox.SelectedObjectCollection  inhalte = liste2.SelectedItems;
            ListBox.SelectedIndexCollection   indices = liste2.SelectedIndices;

            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");

            auto.Ausgeben("Schaltflaeche2 gedrueckt, ausgewaehlte Zeile von Liste1: ");
            auto.Ausgeben("            " +  liste1.SelectedValue
                                  + " (" + (liste1.SelectedIndex+1) + ". Zeile)");
            auto.Ausgeben("                          ausgewaehlte Zeilen von Liste2: ");
            for (int i = 0; i < inhalte.Count; i++)
                auto.Ausgeben("            " +  inhalte[i]
                                      + " (" + (indices[i]+1) + ". Zeile)");
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Kontrollkästchen1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void check1event(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Ausgeben("check1 gedrueckt");
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Optionsfeld1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void radio1event(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Raufschalten();
            auto.ZeigeInhalt();
            auto.Ausgeben("radio1 gedrueckt (raufschalten)");
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Optionsfeld2.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void radio2event(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Bremsen();
            auto.ZeigeInhalt();
            auto.Ausgeben("radio2 gedrueckt (bremsen)");
            textbereich.AppendText(NeueZeile + auto);
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Optionsfeld3.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void radio3event(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!"
                                   + NeueZeile + auto);
            auto.Ausgeben("radio3 gedrueckt, check3 wird leer");
            check3.Checked = false;
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Textfeld.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void textfeldevent(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Ausgeben("Textfeld geaendert: " + textfeld.Text);
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Listenfeld1.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void liste1event(Object sender, MouseEventArgs e)
        {
            auto.Ausgeben("Mauswaechter: Liste1, ausgewaehlte Zeile von Liste1: ");
            auto.Ausgeben("            " +  liste1.SelectedValue
                                  + " (" + (liste1.SelectedIndex+1) + ". Zeile)");
            auto.Ausgeben("    Mausklick in "
                          + (liste1.IndexFromPoint(e.Location)+1) + ". Zeile");
        }

        // ****************************************************************
        /**
        <summary>
        Delegate-Methode für Auswahlfeld.
        </summary>
        <remarks>
        Entspricht in Java einer API-Methode.
        </remarks>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void comboevent(Object sender, EventArgs e)
        {
            textbereich.AppendText(NeueZeile + (++zaehler) 
                                             + ". Action-Ereignis!");
            auto.Ausgeben("Auswahl Combo: " + combo.SelectedItem
                                     + " (" + (combo.SelectedIndex+1) + ". Zeile)");
        }
    }
}
