/*
 * StandardfensterV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Einfaches Hauptfenster mit Menüzeile (ohne Ereignisse).

    <p>
    Seit: August 2006
    </p>

    <seealso cref="Standardfenster11" />
    <seealso cref="FensterMenuV" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Version für .NET 2.0 (Menüsystem mit der Hauptklasse 'MenuStrip'). <br />
    'MenuStrip' ist ein (neues) Steuerelement, während 'MainMenu' ein Teil 
    des Fensters ist! <br />
    Das bedeutet, dass 'MenuStrip' zur Client-Area gehört und theoretisch
    überall platziert werden könnte (es sollte natürlich immer ganz oben 
    stehen). <br />
    'MainMenu' gehört nicht zur Client-Area und steht immer oberhalb dieser.
    </remarks>
    */
    public class StandardfensterV : Form
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private MenuStrip          menuezeile   = new MenuStrip();

        private ToolStripMenuItem  menue1       = new ToolStripMenuItem("Menü1");
        private ToolStripMenuItem  menueItem11  = new ToolStripMenuItem("1. Menüeintrag Menü1");
        private ToolStripMenuItem  menueItem12  = new ToolStripMenuItem("2. Menüeintrag Menü1");
        private ToolStripMenuItem  menueItem13  = new ToolStripMenuItem("...");
        
        private ToolStripMenuItem  menue2       = new ToolStripMenuItem("Menü2");
        private ToolStripMenuItem  menueItem21  = new ToolStripMenuItem("1. Menüeintrag Menü2");
        private ToolStripMenuItem  checkItem22  = new ToolStripMenuItem("2. Check-Menüeintrag Menü2");
        private ToolStripMenuItem  checkItem23  = new ToolStripMenuItem("3. Check-Menüeintrag Menü2");
        private ToolStripMenuItem  menue24      = new ToolStripMenuItem("4. Untermenü Menü2");

        private ToolStripMenuItem  menueItem241 = new ToolStripMenuItem("1. Menüeintrag Untermenü24");
        private ToolStripMenuItem  menueItem242 = new ToolStripMenuItem("2. Menüeintrag Untermenü24");

        private ToolStripMenuItem  radioItem25  = new ToolStripMenuItem("5. Radio-Menüeintrag Menü2");
        private ToolStripMenuItem  radioItem26  = new ToolStripMenuItem("6. Radio-Menüeintrag Menü2");
        private ToolStripMenuItem  radioItem27  = new ToolStripMenuItem("7. Radio-Menüeintrag Menü2");
        private ToolStripMenuItem  menueItem28  = new ToolStripMenuItem("...");

        private ToolStripMenuItem  menue3       = new ToolStripMenuItem("...");
        private ToolStripMenuItem  menueItem31  = new ToolStripMenuItem("...");

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor
        </summary>
        <remarks>
        Das Erzeugen der Menüzeile wird in eine private Methode ausgelagert.
        </remarks>
        */
        public StandardfensterV()
        {
            this.Text = "Standardfenster mit Menüzeile";
            this.Size = new Size(400, 260);

            menuezeileErstellen();

            // GUI-Komponente der Windows-Form hinzufügen

            this.Controls.Add(menuezeile);
        }

        // ---------------------------------------------------------------------
        // private Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Erzeugen der Menüzeile.
        </summary>
        */
        private void menuezeileErstellen()
        {
            // Menüpunkte in Menüs einfügen

            menue1.DropDownItems.Add(menueItem11);                
            menue1.DropDownItems.Add(menueItem12);
            menue1.DropDownItems.Add(menueItem13);

            menue2.DropDownItems.Add(menueItem21);
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(checkItem22);
            menue2.DropDownItems.Add(checkItem23);
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(menue24);   // Untermenü24 als Menüpunkt einfügen
            menue2.DropDownItems.Add(new ToolStripSeparator());
            menue2.DropDownItems.Add(radioItem25);
            menue2.DropDownItems.Add(radioItem26);
            menue2.DropDownItems.Add(radioItem27);
            menue2.DropDownItems.Add(menueItem28);

            menue3.DropDownItems.Add(menueItem31);

            // Menüpunkte in Untermenüs einfügen

            menue24.DropDownItems.Add(menueItem241);              
            menue24.DropDownItems.Add(menueItem242);

            // Menüs in Menüzeile einfügen

            menuezeile.Items.Add(menue1);
            menuezeile.Items.Add(menue2);
            menuezeile.Items.Add(menue3);

            // Initialisierung der true-Checkbox-Menüeinträge

            checkItem22.Checked = true;

            // Initialisierung des (einzigen) true-Radiobutton-Menüeintrags

            radioItem26.Checked = true;
        }
    }
}
