/*
 * OrganisationTesten.cs
 */
using System;

namespace Beringer.oopBuch.Organisation
{
    /**
    <summary>
    Applikation zum Testen des Beispiels 
    "Personen - Mitarbeiter - Organisationen" (Buch Kap.4.4).
    <p>
    Seit: November 2004 <br />
    Version: August 2007 (eigene Programmklasse)
    </p>

    <seealso cref="Person" />
    <seealso cref="Mitarbeiter" />
    <seealso cref="Manager" />
    <seealso cref="Organisation" />
    <seealso cref="Abteilung" />
    <seealso cref="Projekt" />
    <seealso cref="ProgrammOrganisationTesten" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Die Testdaten werden der Einfachheit halber statisch vordefiniert.
    Es gibt keine Eingabedaten!
    </remarks>
    */
    public class OrganisationTesten
    {
        /** --------------------------------------------------------------------
        <summary>
        Start-Methode zum Testen der Applikation.
        </summary>
        */
        public void Start()
        {
            Mitarbeiter  mita11, mita12, mita2, mitp1, mitp21, mitp22;
            Manager      mana1,  mana2,  manp1, manp2;
            Abteilung    abt1,   abt2;
            Projekt      proj1,  proj2;

            abt1 = new Abteilung();
            abt1.Name = "Abt1";
            abt1.Einnahmen = 200000.0f;
            abt1.Sachkosten = 7000.0f;
    
            abt2 = new Abteilung();
            abt2.Name = "Abt2";
            abt2.Einnahmen = 350000.0f;
    
            proj1 = new Projekt();
            proj1.Name = "Projekt1";

            proj2 = new Projekt();
            proj2.Name = "Projekt2";
            proj2.Sachkosten = 5500.0f;
            proj2.PlanZeit = 2;
            proj2.PlanJahresgewinn = 600000.0f;
            proj2.PlanLebensdauer = 6;

            //    mita11 = new Mitarbeiter();       ***error*** geht nicht"
            mita11 = new Mitarbeiter(new DateTime(1962,7,17), 'M');
            mita11.Name = "Huber Hans";
            mita11.SetGrundgehalt(2100.0f);
            mita11.Orga = abt1;

            mita12 = new Mitarbeiter(new DateTime(1977,1,13), 'W');
            mita12.Name = "Irene Gross";
            mita12.SetGrundgehalt(2500.0f);
            mita12.Orga = abt1;

            mita2  = new Mitarbeiter("Karli", DateTime.Today, 'M');
            mita2.Orga = abt2;

            mitp1  = new Mitarbeiter("Doris", DateTime.Today, 'W');
            mitp1.Orga = proj1;

            mitp21 = new Mitarbeiter("Hansi", DateTime.Today, 'M');
            mitp21.Orga = proj2;

            mitp22 = new Mitarbeiter("Helga", DateTime.Today, 'W');
            mitp22.Orga = proj2;

            mana1 = new Manager("Klammer Franz", DateTime.Today, 'M');
            mana1.SetGrundgehalt(5500.0f);
            mana1.Orga = abt1;
            mana1.GeleiteteOrganisation = abt1;

            mana2 = new Manager("Muster Tom", DateTime.Today, 'M');
            mana2.Orga = abt2;
            mana2.GeleiteteOrganisation = abt2;

            manp1 = new Manager("Toni", DateTime.Today, 'M');
            manp1.Orga = proj1;
            manp1.GeleiteteOrganisation = proj1;

            manp2 = new Manager("Annemarie", DateTime.Today, 'W');
            manp2.SetGrundgehalt(4700.0f);
            manp2.Orga = proj2;
            manp2.GeleiteteOrganisation = proj2;

            Console.WriteLine( "Personalnummern: "
                + mita11.PersNr + " " + mita12.PersNr + " "
                + mita2.PersNr  + " " + mitp1.PersNr  + " "
                + mitp21.PersNr + " " + mitp22.PersNr + " "
                + mana1.PersNr  + " " + mana2.PersNr  + " "
                + manp1.PersNr  + " " + manp2.PersNr );

            Console.WriteLine(mita12.Anrede());
            Console.WriteLine(mana1.Anrede());

            Console.WriteLine(mita11.Alter());
            Console.WriteLine(mita12.Alter());

            Console.WriteLine(mita11.Verdient());
            Console.WriteLine(mana1.Verdient());
            Console.WriteLine(manp2.Verdient());

            abt1.DruckeAlleMitarbeiter();    Console.WriteLine();
            abt2.DruckeAlleMitarbeiter();    Console.WriteLine();
            proj1.DruckeAlleMitarbeiter();   Console.WriteLine();
            proj2.DruckeAlleMitarbeiter();   Console.WriteLine();

            abt1.DruckeKostenUebersicht();    Console.WriteLine();
            proj2.DruckeKostenUebersicht();   Console.WriteLine();

#if DEBUG
            Console.ReadLine();
#endif
        }
    }
}
