#include <string.h>
#include <iostream>

using namespace std;

#define MAXANZAHL 100

// ******************************************************************

class Organisation;

// ******************************************************************

class Datum
{
  /* Eine einfache Datumsklasse. */

  public:
    Datum();
    Datum(int tag, int monat, int jahr);
    Datum(Datum &datum);
    int   getTT();
    int   getMM();
    int   getJJ();
    char *monatsname();
    int   lfdTagDesJahres();
    bool  istSchaltjahr();

    static bool  istSchaltjahr(int jahr);
    static Datum heute();

  private:
    static char Monatsnamen [12][9+1];
    static int  TageProMonat [12];
    static int  LfdTagMonatsanfang [12];

    int tt,
        mm,
        jj;
};

// ******************************************************************

class Person
{
  /* Jede Instanz dieser Klasse reprsentiert eine natrliche Person. */

  public:
    Person();
    Person(char einName[], Datum &einDatum, char kennzeichen, char eineAdresse[]);
    Person(char einName[], Datum &einDatum, char kennzeichen);
    Person(Datum &einDatum, char kennzeichen);

    char  *getName();
    void   setName(char neuerName[]);
    Datum& getGebDatum();
    void   setGebDatum(Datum einDatum);
    char   getGeschlecht();
    bool   setGeschlecht(char kennzeichen);
    char  *getAdresse();
    void   setAdresse(char neueAdresse[]);

    char  *anrede();
    int    alter();

  private:
    char  name [30];
    Datum gebDatum;
    char  geschlecht;
    char  adresse [30];
};

// ******************************************************************

class Kunde : public Person
{
  /* Instanzen dieser Klasse reprsentieren Kunden. */

  public:
    Kunde();      //      ... Standardkonstruktor nicht erlaubt
    Kunde(char einName[], Datum &einDatum, char kennzeichen, char eineAdresse[]);
    Kunde(char einName[], Datum &einDatum, char kennzeichen);
    Kunde(Datum &einDatum, char kennzeichen);

    float getJahresumsatz();
    void  setJahresumsatz(float eineZahl);
    float getKreditlimit();
    void  setKreditlimit(float eineZahl);

  private:
    float jahresumsatz;
    float kreditlimit;
};

// ******************************************************************

class Mitarbeiter : public Person
{
  /* Instanzen dieser Klasse reprsentieren Mitarbeiter. */

  public:
    // Mitarbeiter();   ... Standardkonstruktor nicht erlaubt
    Mitarbeiter(char einName[], Datum &einDatum, char kennzeichen, char eineAdresse[]);
    Mitarbeiter(char einName[], Datum &einDatum, char kennzeichen);
    Mitarbeiter(Datum &einDatum, char kennzeichen);

    int           getPersNr();
    char         *getStellung();
    void          setStellung(char neueStellung[]);
    float         getGrundgehalt();
    bool          setGrundgehalt(float eineZahl);
    Organisation& getOrganisation();
    void          setOrganisation(Organisation &eineOrg);
    char         *anrede();

    virtual float verdient();

    static int    getLetztePersNr();
    static void   setLetztePersNr(int eineZahl);
    static float  getMaxGehalt();
    static void   setMaxGehalt(float eineZahl);

  private:
    static int    LetztePersNr;
    static float  MaxGehalt;

    int           persNr;
    char          stellung [20];
    float         grundgehalt;
    Organisation *org;

    bool persNrNeu();
};

// ******************************************************************

class Manager : public Mitarbeiter
{
  /* Instanzen dieser Klasse reprsentieren Manager. */

  public:
    // Manager();   ... Standardkonstruktor nicht erlaubt
    Manager(char einName[], Datum &einDatum, char kennzeichen, char eineAdresse[]);
    Manager(char einName[], Datum &einDatum, char kennzeichen);
    Manager(Datum &einDatum, char kennzeichen);

    Organisation& getGeleiteteOrganisation();
    void          setGeleiteteOrganisation(Organisation &eineOrganisation);

    virtual float verdient();

    static float  getGewinnanteil();
    static void   setGewinnanteil(float eineZahl);

  private:
    static float  Gewinnanteil;
    static float  MaxGehalt;

    Organisation *geleiteteOrganisation;
};

// ******************************************************************

class Organisation
{
  /* Abstrakte Klasse fr organisatorische Einheiten in einem Unternehmen.
     Solche organisatorischen Einheiten knnen Abteilungen, Projekte usw. sein. */

  public:
    Organisation();

    char         *getName();
    void          setName(char neuerName[]);
    Manager&      getLeiter();
    void          setLeiter(Manager &neuerLeiter);
    Mitarbeiter  *getMitarbeiterliste();
    int           getAnzahlMitarb();
    float         getSachkosten();
    void          setSachkosten(float eineZahl);
    void          addMitarbeiter(Mitarbeiter &einMitarbeiter);
    void          removeMitarbeiter(Mitarbeiter &einMitarbeiter);
    void          druckeAlleMitarbeiter();
    void          druckeKostenUebersicht();
    float         personalkostenOhnePraemie();
    float         gesamtkostenOhnePraemie();
    float         praemie();
    float         personalkosten();
    float         gesamtkosten();
    float         gewinn();

    virtual float gewinnOhnePraemie() = 0;

  private:
    char          name [30];
    int           anzahlMitarb;
    Manager      *leiter;
    Mitarbeiter  *mitarbeiterliste [MAXANZAHL];
    float         sachkosten;

    void          setMitarbeiterliste(Mitarbeiter neueMitarbeiterliste[]);
};

// ******************************************************************

class Abteilung : public Organisation
{
  /* Instanzen reprsentieren Abteilungen. */

  public:
    Abteilung();

    float         getEinnahmen();
    void          setEinnahmen(float eineZahl);
    void          addiereZuEinnahmen(float eineZahl);

    virtual float gewinnOhnePraemie();

  private:
    float einnahmen;
};

// ******************************************************************

class Projekt : public Organisation
{
  /* Instanzen reprsentieren Projekte. */

  public:
    Projekt();

    int           getPlanZeit();
    void          setPlanZeit(int eineZahl);
    float         getPlanJahresgewinn();
    void          setPlanJahresgewinn(float eineZahl);
    int           getPlanLebensdauer();
    void          setPlanLebensdauer(int eineZahl);
    float         gewinn();

    virtual float gewinnOhnePraemie();

  private:
    int   planZeit;
    float planJahresgewinn;
    int   planLebensdauer;
};
