/*
 * DAOausgebenSchueler.java
 */
package at.beringer.oopBuch.daoKlassen;

import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Aufbereiten von Schülerdaten und für das Schreiben dieser 
Daten auf die Standardausgabe (= Bildschirm).
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
</p>
<p>
Aufbereiten von Schülerdaten und formatiert ausgeben in 1 Zeile.
</p>
<p>
Methoden 'oeffnen' und 'schliessen' sind nicht notwendig, weil
'System.out' (= Standardausgabe Bildschirm) standardmäßig immer geöffnet ist.
</p>
<p>
Bemerkungen:
<ul>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Der Einfachheit halber sind die auszugebenden Texte fix codiert 
      (d.h. keine Internationalisierung!).</li>
</ul>
</p>

<hr />

@since August 2009
@version Juli 2014 (Verschieben in Paket 'daoKlassen')

@see Schueler

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben
@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@see DAOausgebenSchuelerString
@see DAOschreibenSchueler

@author Beringer Alfred
*/
public class DAOausgebenSchueler
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /** Maximale Anzahl Zeilen pro Seite                                      */
    private static final int     ZZMAX;

    /** Simulieren "Vorschub auf neue Seite"                                  */
    private static final String  NEUESEITE;

    /** Überschrift                                                           */
    private static final String  UEBER;

    /** Format für die Ausgabezeile                                           */
    private static final String  F_AB;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die auszugebenden Daten
    */
    private Schueler  schueler;
    /**
    Zeilenzähler
    */
    private int       zz;
    
    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenkonstante könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    */
    static
    {
        ZZMAX     = 5;          // Zum Testen mindestens 4 Datensätze erfassen

        NEUESEITE = "%n___________________________________________________%n%n";
        UEBER     = "NAME                      NOTENDURCHSCHNITT  KLASSE %n%n";
        F_AB      = "%-25s       %5.2f        %s %n";
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Am Beginn wird das Ende einer Seite simuliert, um auf einfache Art eine 
    Überschrift auf der ersten Seite zu erzwingen.
    </p>
    */
    public DAOausgebenSchueler()
    {
        zz = ZZMAX;
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Setzt die Daten für den auszugebenden Datensatz.

    @param  schueler  Datenobjekt mit den Daten des auszugebenden Datensatzes
    */
    public void setSchueler(Schueler schueler)
    {
        this.schueler = schueler;
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für Dateiverarbeitung
    // -------------------------------------------------------------------------

    /**
    Schreibt 1 Zeile mit den gewünschten Schülerdaten.
    <p>
    Die Datenfelder werden formatiert ausgegeben.
    Gegebenenfalls erfolgt ein (simulierter) Seitenvorschub und die Ausgabe 
    einer Überschrift.
    </p>
    */
    public void schreibenZeile()
    {
        if (zz >= ZZMAX)
        {
            System.out.printf(NEUESEITE);
            System.out.printf(UEBER);
            zz = 2;
        }
        System.out.printf(F_AB, schueler.getName(),
                                schueler.getNotenschnitt(),
                                schueler.getKlasse());
        zz = zz + 1;
    }
}
