/*
 * DAOausgebenSchuelerString.java
 */
package at.beringer.oopBuch.daoKlassen;

import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Aufbereiten von Schülerdaten und für das Ausgeben in eine 
Zeichenkette.
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
</p>
<p>
Aufbereiten von Schülerdaten und formatiert ausgeben in 1 Zeichenkette.
</p>
<p>
Es gibt keine Methoden 'oeffnen' und 'schliessen', weil die Ausgabe in eine 
Zeichenkette erfolgt.
</p>
<p>
Bemerkungen:
<ul>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Der Einfachheit halber sind die auszugebenden Texte fix codiert 
      (d.h. keine Internationalisierung!).</li>
</ul>
</p>

<hr />

@since November 2013
@version Juli 2014 (Verschieben in Paket 'daoKlassen')

@see Schueler

@see at.beringer.oopBuch.einAusgabeSwing.LesenAusgebenStringV

@see DAOausgebenSchueler

@author Beringer Alfred
*/
public class DAOausgebenSchuelerString
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /** Überschrift                                                           */
    private static final String  UEBER;

    /** Format für die Ausgabezeichenkette                                    */
    private static final String  F_AB;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die auszugebenden Daten
    */
    private Schueler  schueler;
    
    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenkonstante könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    */
    static
    {
        UEBER     = "NAME                      NOTENDURCHSCHNITT  KLASSE %n%n";
        F_AB      = "%-25s       %6.2f       %s %n";
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Setzt die Daten für den auszugebenden Datensatz.

    @param  schueler  Datenobjekt mit den Daten des auszugebenden Datensatzes
    */
    public void setSchueler(Schueler schueler)
    {
        this.schueler = schueler;
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für die Ausgabe
    // -------------------------------------------------------------------------

    /**
    Schreibt formatiert in 1 Zeichenkette die gewünschten Schülerdaten.
    <p>
    Die Datenfelder werden formatiert ausgegeben.
    </p>
    @return  aufbereitete Daten
    */
    public String schreibenZeile()
    {
        return String.format(F_AB, schueler.getName(),
                                   schueler.getNotenschnitt(),
                                   schueler.getKlasse());
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert eine formatierte Überschrift.

    @return  aufbereitete Überschrift
    */
    public static String getUeberschrift()
    {
        return String.format(UEBER);
    }
}
