/*
 * LesenAusgebenStringV.java
 */
package at.beringer.oopBuch.einAusgabeSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.daoKlassen.DAOlesenSchueler;
import at.beringer.oopBuch.daoKlassen.DAOausgebenSchuelerString;

/**
Sehr Einfache Swing-Anwendung für Lesen von Daten aus einer Datei und 
Ausgeben (Schreiben) der Daten in eine Liste (JList).<br />
Am Beginn werden alle Datensätze gelesen und in eine Liste geschrieben.
<p>
Der Dateiname wird beim Aufruf als Parameter übergeben.
</p>
<p>
Die Schülerdaten werden in einer Zeichenkette aufbereitet, diese Zeichenkette 
wird in einer Liste ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der 
      Applikationsklasse 'LesenAusgeben'.</li>
  <li>Definition eines 'Default-Buttons'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since November 2013

@see DAOlesenSchueler
@see DAOausgebenSchuelerString

@see ProgrammLesenAusgebenString

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben

@author Beringer Alfred
*/
public class LesenAusgebenStringV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DefaultListModel<String>  listeDaten = new DefaultListModel<>();
    private JList<String>             liste      = new JList<>(listeDaten);
    private JScrollPane               scrListe   = new JScrollPane(liste);

    private JButton  butEnde = new JButton("  Ende  ");

    // ----------

    private DAOlesenSchueler  eingabedaten;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.

    @param  dateiname  Name der Ausgabedatei (als Zeichenkette)
    */
    public LesenAusgebenStringV(String dateiname)
    {
        setSize(600, 500);
        setTitle("Fenster für Datenausgabe in einer JList (Datei)");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butEnde);

        initListe();
        initFenster();
        initListener();

        lesenDatei(dateiname);
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Öffnen der Datei, lesen und ausgeben aller Datensätze.

    @param  dateiname  Name der Eingabedatei (als Zeichenkette)
    */
    private void lesenDatei(String dateiname)
    {
        DAOausgebenSchuelerString  ausgabedaten;
        boolean                    ok;

        eingabedaten = new DAOlesenSchueler();

        // Am Beginn der Applikation Datei öffnen
        if (!eingabedaten.oeffnen(dateiname))
        {
            listeDaten.addElement("Fehler beim Öffnen der Datei " 
                                  + dateiname
                                  + "   --->   Ende");
            return;
        }

        // ---------------------------------------------------------------------
        // Alle Datensätze lesen und in die Liste schreiben

        ausgabedaten = new DAOausgebenSchuelerString();

        // Überschrift mit nachfolgender Leerzeile
        listeDaten.addElement(DAOausgebenSchuelerString.getUeberschrift());
        listeDaten.addElement(" ");
        
        ok = eingabedaten.lesen();                            // Lesen Datensatz
        while (!eingabedaten.getEof())
        {
            if (ok)
            {
                // Daten übernehmen (Datenobjekt füllen mit den Eingabedaten)
                ausgabedaten.setSchueler(eingabedaten.getSchueler());
                listeDaten.addElement(ausgabedaten.schreibenZeile());
            }
            else
            {
                listeDaten.addElement("   ***** Fehler beim Lesen! *****");
            }
            ok = eingabedaten.lesen();                        // Lesen Datensatz
        }
    }

    // -------------------------------------------------------------------------

    /**
    JList initialisieren.
    */
    private void initListe()
    {
        // Nichtproportionale Schrift festlegen (für spaltenweise Aufbereitung)
        liste.setFont( new Font(Font.MONOSPACED,
                                liste.getFont().getStyle(),
                                liste.getFont().getSize()) );
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        // Mitte --------------------------------
        this.add(scrListe);

        // unten --------------------------------
        JPanel unten = new JPanel();

        unten.setLayout(new FlowLayout());
        unten.add(butEnde);

        this.add(unten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });

        butEnde.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     fensterSchliessen(evt);
                 }
             });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Beenden der Applikation - Datei schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    private void fensterSchliessen(AWTEvent evt)
    {
        if (eingabedaten != null)
        {
            eingabedaten.schliessen();
        }
        System.exit(0);
    }
}
