/*
 * ErfassenSchreibenV.java
 */
package at.beringer.oopBuch.einAusgabeSwingView;

import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.viewKlassen.ErfassenV;
import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOschreibenSchueler;

/**
Einfache Swing-Anwendung für Erfassen von Daten und 
Schreiben der Daten in eine Datei.
<p>
Der Dateiname wird beim Aufruf als Parameter übergeben.
</p>
<p>
Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt) <br />
View-Klasse:    'ErfassenV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Es werden der Reihe nach Daten eines 'Schueler'-Objekts erfasst und in 
      eine Datei geschrieben, d.h. diese Fensterklasse entspricht der 
      DAO-Klasse 'DAOerfassenSchueler' und der Applikationsklasse 
      'ErfassenSchreiben'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see ErfassenV
@see Schueler
@see DAOschreibenSchueler

@see ProgrammErfassenSchreiben

@see at.beringer.oopBuch.daoKlassen.DAOerfassenSchueler
@see at.beringer.oopBuch.einAusgabe.ErfassenSchreiben

@author Beringer Alfred
*/
public class ErfassenSchreibenV extends ErfassenV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu erfassenden Daten
    */
    private Schueler  schueler;

    private DAOschreibenSchueler  ausgabedaten = new DAOschreibenSchueler();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.

    @param  dateiname  Name der Ausgabedatei (als Zeichenkette)
    */
    public ErfassenSchreibenV(String dateiname)
    {
        setTitle("Fenster für Datenerfassung (Datei)");

        oeffnenDatei(dateiname);
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Öffnen der Datei.

    @param  dateiname  Name der Ausgabedatei (als Zeichenkette)
    */
    private void oeffnenDatei(String dateiname)
    {
        // Am Beginn der Applikation Datei öffnen
        if (!ausgabedaten.oeffnen(dateiname))
        {
            listeDaten.addElement("Fehler beim Öffnen der Datei " 
                                  + dateiname 
                                  + "   --->   Ende");

            // GUI-Komponenten für die Datenerfassung deaktivieren:
            txfName.setEnabled(false);
            txfNote.setEnabled(false);
            txfKlasse.setEnabled(false);
            butErfassen.setEnabled(false);
        }
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Daten erfassen, eingegebene Daten kontrollieren, korrekte Daten schreiben

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    @Override
    protected void butErfassenEreignis(ActionEvent evt)
    {
        String  name;
        String  notenschnittStr;
        float   notenschnitt;
        String  klasse;

        // --------------------
        // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)

        name = txfName.getText();

        // --------------------
        // 2. Datenfeld (Notendurchschnitt - Zahl)
        
        notenschnittStr = txfNote.getText();
        try
        {
            notenschnitt = Float.parseFloat(notenschnittStr);
        }
        catch (NumberFormatException fehler)
        {
            listeDaten.addElement("   ***** Falscher Notendurchschnitt: " 
                                  + notenschnittStr + " *****");
            return;
        }

        // --------------------
        // 3. Datenfeld (Klassenbezeichnung - Zeichenkette mit fixer Länge)

        klasse = txfKlasse.getText();

        // --------------------
        // Erfasste Daten sind korrekt -> Objekt für Datensatz instanzieren

        schueler = new Schueler();
        schueler.setName(name);
        schueler.setNotenschnitt(notenschnitt);
        schueler.setKlasse(klasse);

        // Daten übernehmen und Datensatz schreiben
        ausgabedaten.setSchueler(schueler);
        ausgabedaten.schreiben();

        // Erfasste Daten zur Kontrolle in die Liste übertragen
        listeDaten.addElement(name + " "
                              + notenschnitt + " "
                              + klasse);

        // Textfelder leeren
        txfName.setText("");
        txfNote.setText("");
        txfKlasse.setText("");
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Datei schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        if (ausgabedaten != null)
        {
            ausgabedaten.schliessen();
        }
        System.exit(0);
    }
}
