/*
 * LesenAusgebenJTableV.java
 */
package at.beringer.oopBuch.einAusgabeSwingView;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.util.*;

import at.beringer.oopBuch.viewKlassen.AusgebenJTableV;
import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenSchueler;

/**
Einfache Swing-Anwendung für Lesen von Daten aus einer Datei und 
Ausgeben (Schreiben) der Daten in eine Tabelle (JTable).<br />
Die Datensätze werden einzeln der Reihe nach in eine Tabelle geschrieben.
<p>
Der Dateiname wird beim Aufruf als Parameter übergeben.
</p>
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts. Sie werden 
aufbereitet und formatiert in einer Tabelle ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt) <br />
View-Klasse:    'AusgebenJTableV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der DAO-Klasse 'DAOausgebenSchueler' und 
      der Applikationsklasse 'LesenAusgeben'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see AusgebenJTableV
@see Schueler
@see DAOlesenSchueler

@see ProgrammLesenAusgebenJTable

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben

@author Beringer Alfred
*/
public class LesenAusgebenJTableV extends AusgebenJTableV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DAOlesenSchueler  eingabedaten;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.

    @param  dateiname  Name der Ausgabedatei (als Zeichenkette)
    */
    public LesenAusgebenJTableV(String dateiname)
    {
        setTitle("Fenster für Datenausgabe in einer JTable (Datei)");

        oeffnenDatei(dateiname);
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Öffnen der Datei.

    @param  dateiname  Name der Eingabedatei (als Zeichenkette)
    */
    private void oeffnenDatei(String dateiname)
    {
        eingabedaten = new DAOlesenSchueler();

        // Am Beginn der Applikation Datei öffnen
        if (!eingabedaten.oeffnen(dateiname))
        {
            JOptionPane.showMessageDialog(this,
                                          "Fehler beim Öffnen der Datei " 
                                          + dateiname 
                                          + "   --->   Ende");
            butNext.setEnabled(false);
            return;
        }
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Lesen eines Datensatzes (aus der Datei).

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    @Override
    protected void lesen(ActionEvent evt)
    {
        boolean   ok;

        ok = eingabedaten.lesen();
        if (!eingabedaten.getEof())
        {
            if (ok)
            {
                // nächsten Datensatz übernehmen
                addTabellenzeile(eingabedaten.getSchueler());
            }
            else
            {
                JOptionPane.showMessageDialog(this,
                                              "***** Fehler beim Lesen! *****");
            }
        }
        else
        {
            JOptionPane.showMessageDialog(this, "EOF");
        }
    }

    // -------------------------------------------------------------------------

    private void addTabellenzeile(Schueler daten)
    {
        Vector<String>  zeile = new Vector<String>();

        zeile.add(daten.getName());
        zeile.add("" + daten.getNotenschnitt());
        zeile.add(daten.getKlasse());
        
        alleZeilen.add(zeile);
        tabelleDTM.setDataVector(alleZeilen, tabellenueberschrift);
//        tabelleDTM.addRow(zeile);
      
        testdruck();   // Nur zum Testen
    }

    // -------------------------------------------------------------------------

    /**
    Diese Methode dient NUR zum Testen der Ausgabe.
    <p>
    Achtung:<br />
    Nach jedem Lesen werden immer wieder ALLE bisher gelesenen Datensätze 
    ausgedruckt!
    </p>
    */
    private void testdruck()
    {
        Vector<String>  zeile;
        
        for (int i = 0; i < alleZeilen.size(); i++)
        {
            zeile = alleZeilen.get(i);
            System.out.println(zeile.get(0) + " " +
                               zeile.get(1) + " " +
                               zeile.get(2));
        }
        System.out.println();
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Datei schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        if (eingabedaten != null)
        {
            eingabedaten.schliessen();
        }
        System.exit(0);
    }
}
