/*
 * LesenAusgebenStringV.java
 */
package at.beringer.oopBuch.einAusgabeSwingView;

import java.awt.*;

import at.beringer.oopBuch.viewKlassen.AusgebenStringV;
import at.beringer.oopBuch.daoKlassen.DAOlesenSchueler;
import at.beringer.oopBuch.daoKlassen.DAOausgebenSchuelerString;

/**
Sehr Einfache Swing-Anwendung für Lesen von Daten aus einer Datei und 
Ausgeben (Schreiben) der Daten in eine Liste (JList).<br />
Am Beginn werden alle Datensätze gelesen und in eine Liste geschrieben.
<p>
Der Dateiname wird beim Aufruf als Parameter übergeben.
</p>
<p>
Die Schülerdaten werden in einer Zeichenkette aufbereitet, diese Zeichenkette 
wird in einer Liste ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
View-Klasse:    'AusgebenStringV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der 
      Applikationsklasse 'LesenAusgeben'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see AusgebenStringV
@see DAOlesenSchueler
@see DAOausgebenSchuelerString

@see ProgrammLesenAusgebenString

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben

@author Beringer Alfred
*/
public class LesenAusgebenStringV extends AusgebenStringV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DAOlesenSchueler  eingabedaten;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.

    @param  dateiname  Name der Ausgabedatei (als Zeichenkette)
    */
    public LesenAusgebenStringV(String dateiname)
    {
        setTitle("Fenster für Datenausgabe in einer JList (Datei)");

        lesenDatei(dateiname);
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Öffnen der Datei, lesen und ausgeben aller Datensätze.

    @param  dateiname  Name der Eingabedatei (als Zeichenkette)
    */
    private void lesenDatei(String dateiname)
    {
        DAOausgebenSchuelerString  ausgabedaten;
        boolean                    ok;

        eingabedaten = new DAOlesenSchueler();

        // Am Beginn der Applikation Datei öffnen
        if (!eingabedaten.oeffnen(dateiname))
        {
            listeDaten.addElement("Fehler beim Öffnen der Datei " 
                                  + dateiname
                                  + "   --->   Ende");
            return;
        }

        // ---------------------------------------------------------------------
        // Alle Datensätze lesen und in die Liste schreiben

        ausgabedaten = new DAOausgebenSchuelerString();

        // Überschrift mit nachfolgender Leerzeile
        listeDaten.addElement(DAOausgebenSchuelerString.getUeberschrift());
        listeDaten.addElement(" ");
        
        ok = eingabedaten.lesen();                            // Lesen Datensatz
        while (!eingabedaten.getEof())
        {
            if (ok)
            {
                // Daten übernehmen (Datenobjekt füllen mit den Eingabedaten)
                ausgabedaten.setSchueler(eingabedaten.getSchueler());
                listeDaten.addElement(ausgabedaten.schreibenZeile());
            }
            else
            {
                listeDaten.addElement("   ***** Fehler beim Lesen! *****");
            }
            ok = eingabedaten.lesen();                        // Lesen Datensatz
        }
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Beenden der Applikation - Datei schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        if (eingabedaten != null)
        {
            eingabedaten.schliessen();
        }
        System.exit(0);
    }
}
