/*
 * Auto.java
 */
package at.beringer.oopBuch.einfacheBsp;

/**
Eine sehr einfache Auto-Klasse, jede Instanz dieser Klasse repräsentiert ein 
Auto.
<ul>
  <li>Jede Instanz wird mit einer Anfangsgeschwindigkeit von 10 km/h 
      initialisiert.</li>
  <li>Rauf- und runterschalten bewirkt keine Geschwindigkeitsänderung.</li>
  <li>Bremsen und beschleunigen bewirkt immer eine Geschwindigkeitsänderung 
      von 10%, jedoch keinen Gangwechsel.</li>
</ul>
<p>
Es wurde zusätzlich noch (neben der Methode 'zeigeInhalt') 
die Methode 'toString' überschrieben.
</p>
<p>
Diese Klasse dient auch als Fachobjekt ("model") für die GUI-Beispiele.
</p>

<hr />

@since Jänner 2002
@version Juli 2012 (PKW-Klasse umbenennen auf 'Auto'.
                    Keine Initialisierung des Besitzers im Konstruktor.)

@see Person
@see AutoTesten
@see ProgrammAutoTesten
@see AutoM

@author Beringer Alfred
*/
public class Auto
{
    // -------------------------------------------------------------------------
    // Klassenattribute
    // -------------------------------------------------------------------------

    /** Mehrwertsteuersatz                                                    */
    private static int  MwstSatz;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String  farbe;
    private int     anzahlGaenge;
    private float   geschwindigkeit;
    private int     gang;
    private Person  besitzer;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Alle Attribute (ausgenommen Besitzer) werden mit fixen Werten initialisiert.
    Es wird eine Anfangsgeschwindigkeit von 10 km/h angenommen.
    </p>
    */
    public Auto()
    {
        setFarbe("rot");
        setAnzahlGaenge(5);
        setGeschwindigkeit(10.0f);
        setGang(1);
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die Farbe des Autos.

    @return  Farbe
    */
    public String getFarbe()
    {
      return  farbe;
    }

    // ----------

    /**
    Ändert die Farbe des Autos.

    @param  farbe  neue Farbe
    */
    public void setFarbe(String  farbe)
    {
        this.farbe = farbe;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Maximalanzahl der Gänge des Autos.

    @return  Anzahl Gänge
    */
    public int getAnzahlGaenge()
    {
        return  anzahlGaenge;
    }

    // ----------

    /**
    Ändert die Maximalanzahl der Gänge des Autos - dies entspricht einem Umbau 
    des Autos.
    <ul>
      <li>Die Maximalanzahl der Gänge muss zumindest 1 sein.</li>
      <li>Der eingelegte Gang wird auf 1 zurückgesetzt.</li>
    </ul>

    @param  anzahlGaenge  neue Anzahl Gänge
    */
    public void setAnzahlGaenge(int  anzahlGaenge)
    {
        if (anzahlGaenge >= 1)
        {
            this.anzahlGaenge = anzahlGaenge;
        }
        setGang(1);
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die aktuelle Geschwindigkeit.

    @return  Geschwindigkeit
    */
    public float getGeschwindigkeit()
    {
        return  geschwindigkeit;
    }

    // ----------

    /**
    Ändert die aktuelle Geschwindigkeit.
    <p>
    Es erfolgt aber keine Prüfung auf Plausibilität!
    </p>

    @param  geschwindigkeit  neue Geschwindigkeit
    */
    public void setGeschwindigkeit(float  geschwindigkeit)
    {
        this.geschwindigkeit = geschwindigkeit;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den gerade eingelegten Gang.

    @return  Gang
    */
    public int getGang()
    {
        return  gang;
    }

    // ----------

    /**
    Legt anderen Gang ein.
    <p>
    Der eingelegte Gang kann nicht kleiner sein als 1 und nicht höher als 
    die Anzahl der Gänge. <br />
    Bei einem Fehler wird der Gang nicht geändert.
    </p>

    @param  gang  neuer Gang
    */
    public void setGang(int gang)
    {
        if ( (gang >= 1) && (gang <= anzahlGaenge) )
        {
            this.gang = gang;
        }
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den Besitzer des Autos.

    @return  Besitzer
    */
    public Person getBesitzer()
    {
        return  besitzer;
    }

    // ----------

    /**
    Ändert den Besitzer des Autos.

    @param  besitzer  neuer Besitzer
    */
    public void setBesitzer(Person  besitzer)
    {
        this.besitzer = besitzer;
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Gang um 1 erhöhen.
    */
    public void raufschalten()
    {
        if (gang < anzahlGaenge)
        {
            gang = gang + 1;
        }
    }

    // ----------

    /**
    Gang um 1 vermindern.
    */
    public void runterschalten()
    {
        if (gang > 1)
        {
            gang = gang - 1;
        }
    }

    // ----------

    /**
    Geschwindigkeit um 10% erhöhen.
    */
    public void beschleunigen()
    { 
        geschwindigkeit = geschwindigkeit * 1.1f;
    }

    // ----------

    /**
    Geschwindigkeit um 10% vermindern.
    */
    public void bremsen()
    {
        geschwindigkeit = geschwindigkeit * 0.9f;
    }

    // -------------------------------------------------------------------------

    /**
    Schreibt alle Attribute in die Standardausgabe.
    <p>
    Achtung: Besitzer sollte bereits initialisiert sein!
    </p>
    <p>
    Anstelle dieser Methode sollte besser die Methode 'Object.toString()'
    überschrieben werden.
    </p>
    */
    public void zeigeInhalt()
    {
        // direkter Zugriff auf Attribute (keine Datenkapselung)
        // 'getName()' liefert Pfad + Name der Klasse
        System.out.println(getClass().getName() 
                + "-Daten: [ Mehrwertsteuersatz = " + MwstSatz + "%" );
        System.out.println("   Besitzer = "         + besitzer.heisst()
                + ", Farbe = "                      + farbe
                + ", Anzahl Gänge = "               + anzahlGaenge );
        System.out.println("   eingelegter Gang = " + gang
                + " / aktuelle Geschwindigkeit = "  + geschwindigkeit + " ]" );
    }

    // -------------------------------------------------------------------------
    // Methode 'toString'
    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'String Object.toString()'.
    <p>
    Achtung: Besitzer sollte bereits initialisiert sein!
    </p>
    <p>
    Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
    <pre>
      Auto-Daten: [ Mehrwertsteuersatz = ...%
           Besitzer = ..., Farbe = ..., Anzahl Gänge = ...
           eingelegter Gang = ... / aktuelle Geschwindigkeit = ... ]
    </pre>
    </p>
    <p>
    Hinweis: Statt der Klasse 'String' sollte die Klasse 'StringBuilder' 
             verwendet werden!
    </p>

    @see java.lang.Object#toString()

    @return  Zeichenkette, die eine Instanz beschreibt
    */
    @Override
    public String toString()
    {
        String  neueZeile = System.getProperty("line.separator");

        // Zugriff auf Attribute nur über Zugriffsmethoden (Datenkapselung)
        // 'getSimpleName()' liefert nur den Namen der Klasse
        return  getClass().getSimpleName() 
           + "-Daten: [ Mehrwertsteuersatz = " + getMwstSatz() + "%" + neueZeile
           + "     Besitzer = "                + getBesitzer().heisst()
           + ", Farbe = "                      + getFarbe()
           + ", Anzahl Gänge = "               + getAnzahlGaenge()   + neueZeile
           + "     eingelegter Gang = "        + getGang()
           + " / aktuelle Geschwindigkeit = "  + getGeschwindigkeit() + " ]" ;
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert den Mehrwertsteuersatz.

    @return  Mehrwertsteuersatz
    */
    public static int getMwstSatz()
    {
        return  MwstSatz;
    }

    // ----------

    /**
    Ändert den Mehrwertsteuersatz.

    @param  mwstSatz  neuer Mehrwertsteuersatz
    */
    public static void setMwstSatz(int mwstSatz)
    {
        MwstSatz = mwstSatz;
    }
}
