/*
 * Person7.java
 */
package at.beringer.oopBuch.einfacheBsp;

import java.util.GregorianCalendar;

/**
Eine einfache Personen-Klasse (alte Version bis JDK 7).

<p>
Diese Klasse verwendet die JDK-Klasse 'GregorianCalendar' und wurde nicht mit 
Java 8 aktualisiert. Abgesehen davon ist diese Klasse daher identisch mit der 
Klasse 'Person'.
</p>

<p>
Jede Instanz dieser Klasse repräsentiert eine natürliche Person.
</p>
<ul>
  <li>Klassenkonstante für gültige Geschlechtskennzeichen.</li>
  <li>Jede Person hat immer ein Geburtsjahr und ein Geschlecht.</li>
  <li>Geburtsjahr und Geschlecht können nicht geändert werden.</li>
  <li>Geburtsjahr und Geschlecht werden geprüft: <br />
      Geburtsjahr darf nicht größer sein als das laufende Jahr,
      das Geschlecht darf nur 'M' oder 'W' sein.</li>
</ul>
<p>
Hinweis: Die Namen der Standardzugriffsmethoden entsprechen NICHT den üblichen 
         Java- bzw. UML-Konventionen ('getAttribut' bzw. 'setAttribut'). 
</p>
 
<hr />

@since August 1999
@version Juli 2003 (Klassenkonstante für männlich und weiblich)
@version <br />Juni 2006 (Geburtsjahr)

@see Person

@author Beringer Alfred
*/
public class Person7
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /**
    Kennzeichen für männliches Geschlecht.
    <p>
    Der Wert könnte von externen Datenquellen geholt werden, wird jedoch der 
    Einfachheit halber fix vorgegeben.
    </p>
    */
    public static final char  MAENNLICH = 'M';

    /**
    Kennzeichen für weibliches Geschlecht.
    <p>
    Der Wert könnte von externen Datenquellen geholt werden, wird jedoch der 
    Einfachheit halber fix vorgegeben.
    </p>
    */
    public static final char  WEIBLICH = 'W';

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String  vorname;
    private int     geburtsjahr;
    private char    geschlecht;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Jede Person muss ein Geburtsjahr und ein Geschlecht haben.
    <p>
    Standardkonstruktor ist nicht erlaubt.
    </p>
    <p>
    Der Einfachheit halber wird bei falschen Werten keine Ausnahme geworfen.
    </p>

    @param  geburtsjahr  Geburtsjahr
    @param  geschlecht   Geschlecht
    */
    public Person7(int geburtsjahr, char geschlecht)
    {
        aendernGeburtsjahr(geburtsjahr);
        aendernGeschlecht(geschlecht);
    }

    // -------------------------------------------------------------------------
    // Öffentliche Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den Vornamen der Person.

    @return  Vorname
    */
    public String heisst()
    {
        return  vorname;
    }

    // ----------

    /** 
    Ändert den Vornamen der Person.

    @param  neuerVorname  neuer Vorname
    */
    public void aendernVorname(String neuerVorname)
    {
        vorname = neuerVorname;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geburtsjahr der Person.

    @return  Geburtsjahr
    */
    public int geboren()
    {
        return  geburtsjahr;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geschlecht der Person.

    @return  Geschlecht
    */
    public char hatGeschlecht()
    {
        return  geschlecht;
    }

    // -------------------------------------------------------------------------
    // Sonstige öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert die Anrede für die Person (abhängig vom Geschlecht).
    <p>
    Die Anrede ist fix codiert und nicht variabel!
    </p>

    @return  Anrede entsprechend dem Geschlecht
    */
    public String anrede()
    {
        if (geschlecht == MAENNLICH)
        {
            return  "Sehr geehrter Herr";
        }

        return  "Sehr geehrte Frau";

        /*
          kürzer mit Bedingungsoperator:

        return  (geschlecht == MAENNLICH) ? "Sehr geehrter Herr"
                                          : "Sehr geehrte Frau";
        */
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet das Alter der Person.

    @return  Alter in Jahren
    */
    public int alterBerechnen()
    {
        GregorianCalendar  heute = new GregorianCalendar();
    
        return  heute.get(GregorianCalendar.YEAR) - geburtsjahr;
    }

    // -------------------------------------------------------------------------
    // Private Zugriffsmethoden
    // -------------------------------------------------------------------------

    /**
    Setzt das Geburtsjahr der Person.
    <p>
    Das Geburtsjahr wird auf Gültigkeit geprüft:
        (Geburtsjahr &lt;= laufendes Jahr). <br />
    Bei einem Fehler wird das Geburtsjahr nicht gesetzt!
    </p>

    @param  einJahr  neues Geburtsjahr

    @return  true, wenn 'einJahr' ein gültiges Geburtsjahr ist
    */
    private boolean aendernGeburtsjahr(int einJahr)
    {
        int lfdJahr = (new GregorianCalendar()).get(GregorianCalendar.YEAR);

        if (einJahr <= lfdJahr)
        {
            geburtsjahr = einJahr;
            return  true;
        }

        // ungültiges Geburtsjahr
        return  false;
    }

    // -------------------------------------------------------------------------

    /**
    Setzt das Geschlecht der Person.
    <p>
    Das Geschlecht wird auf Gültigkeit geprüft. <br />
    Bei einem Fehler wird das Geschlecht nicht gesetzt!
    </p>

    @param  kennzeichen  Kennzeichen für Geschlecht (männlich oder weiblich)

    @return  true, wenn 'kennzeichen' ein gültiges Geschlechtskennzeichen ist
    */
    private boolean aendernGeschlecht(char kennzeichen)
    {
        if ( (kennzeichen == MAENNLICH) || (kennzeichen == WEIBLICH) )
        {
            geschlecht = kennzeichen;
            return  true;
        }

        // ungültiges Kennzeichen für Geschlecht
        return  false;
    }
}
