/*
 * Zaehler.java
 */
package at.beringer.oopBuch.einfacheBsp;

/**
Instanzen dieser Klasse repräsentieren einen einfachen Zähler,
der (um 1) erhöht und (auf 0) zurückgesetzt werden kann.
<p>
   Anwendungsbeispiel:
   <pre>
       Zaehler  z;
       z = new Zaehler();
       z.erhoehen();
       System.out.println(z.getWert());
   </pre>
</p>

<hr />

@since August 1999
@version April 2002 (protected setWert)

@see ProgrammZaehler
@see Zaehler2
@see Zaehler3

@author Beringer Alfred
*/
public class Zaehler
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private int  wert;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Zähler wird zurückgesetzt.
    </p>
    */
    public Zaehler()
    {
        reset();
    }

    // -------------------------------------------------------------------------
    // Öffentliche Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den aktuellen Wert des Zählers.

    @return  aktueller Zählerwert
    */
    public int getWert()
    {
        return  wert;
    }

    // -------------------------------------------------------------------------
    // Sonstige Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Erhöht den Zähler um 1.

    @return  neuer (inkrementierter) Zählerwert
    */
    public int erhoehen()
    {
        setWert( getWert() + 1 );
        return  getWert();

        /*
         Einfacher, aber mit weniger Kapselung:
               wert = wert + 1;
               return  wert;
        */
    }

    // -------------------------------------------------------------------------

    /**
    Setzt den Zähler zurück auf 0.
    */
    public void reset()
    {
        setWert(0);

        /*
         Einfacher, aber mit weniger Kapselung:
               wert = 0;
        */
    }

    // -------------------------------------------------------------------------
    // Nichtöffentliche Zugriffsmethoden
    // -------------------------------------------------------------------------

    /**
    Ändert den Wert des Zählers - es erfolgt jedoch keine Prüfung auf 
    Plausibilität!
    <p>
    Bem.: Die Zugriffsberechtigung wurde für die zusätzliche Beispielklasse 
          'Zaehler3' auf 'protected' geändert!
    </p>

    @param  wert  neuer (beliebiger) Zählerwert
    */
    protected void setWert(int wert)
    {
        this.wert = wert;
    }
}
