/*
 * Zaehler2.java
 */
package at.beringer.oopBuch.einfacheBsp;

/**
Instanzen dieser Klasse repräsentieren einen einfachen zweistelligen
Zähler, der (um 1) erhöht und (auf 0) zurückgesetzt werden kann. <br />
Der Zähler beginnt nach 99 (= Standardwert für Endwert) wieder bei 0, d.h.
bei 100 erfolgt ein Überlauf.

<hr />

@since August 1999

@see ProgrammZaehler2
@see Zaehler
@see Zaehler3

@author Beringer Alfred
*/
public class Zaehler2
{
    // -------------------------------------------------------------------------
    // Klassenattribute
    // -------------------------------------------------------------------------

    /**
    Der größte Wert, den der Zähler annehmen kann, danach muss der Zähler
    zurückgesetzt werden.
    */
    private static int  Endwert = 99;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private int  wert;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Zähler wird zurückgesetzt.
    </p>
    */
    public Zaehler2()
    {
        reset();
    }

    // -------------------------------------------------------------------------
    // Öffentliche Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den aktuellen Wert des Zählers.

    @return  aktueller Zählerwert
    */
    public int getWert()
    {
        return  wert;
    }

    // -------------------------------------------------------------------------
    // Sonstige öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Erhöht den Zähler um 1 - bei Überlauf wird Zähler zurückgesetzt.

    @return  neuer (inkrementierter) Zählerwert
    */
    public int erhoehen()
    {
        setWert(getWert()+1);   // Kapselung notwendig, um Überlauf zu erkennen!
        return  getWert();
    }

    // -------------------------------------------------------------------------

    /**
    Setzt den Zähler zurück auf 0.
    */
    public void reset()
    {
        setWert(0);
    }

    // -------------------------------------------------------------------------
    // Nichtöffentliche Zugriffsmethoden
    // -------------------------------------------------------------------------

    /**
    Ändert den Wert des Zählers.

    @param  wert  neuer Zählerwert, der zwischen 0 und Endwert liegen muss
    */
    private void setWert(int wert)
    {
        this.wert = wert;
        if ((this.wert > Endwert) || (this.wert < 0))
        {
            reset();
        }
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert den aktuellen Endwert für den Zähler.

    @return  aktueller Endwert
    */
    public static int getEndwert()
    {
        return  Endwert;
    }

    // ----------

    /**
    Definiert einen neuen Endwert für den Zähler - es erfolgt jedoch keine 
    Prüfung auf Plausibilität!

    @param  endwert  neuer (beliebiger) Endwert
    */
    public static void setEndwert(int endwert)
    {
        Endwert = endwert;
    }
}
