/*
 * LabelButtonFenster2V.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.geometry.*;

import javafx.event.*;

import at.beringer.oopBuch.einfacheBsp.Person;
import at.beringer.oopBuch.einfacheBsp.Auto;

/**
Hauptfenster mit Text ('Label') und 2 Schaltfächen.
<p>
Anonyme Klasse: von 'EventHandler-ActionEvent' <br />
Model-Klasse:  'Auto' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Diese Fensterklasse entspricht der Klasse 'LabelButtonFensterV', wobei jedoch 
jedem Ereignis ein eigener Ereigniswächter ('EventHandler') 
zugeordnet wird.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

<hr />

@since Juni 2013

@see Auto
@see Person
@see LabelButtonFensterV

@author Beringer Alfred
*/
public class LabelButtonFenster2V extends Application
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private Button  schaltflaeche1 = new Button("_raufschalten");
    private Button  schaltflaeche2 = new Button("  _bremsen  ");

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private Auto  auto;

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode
    */
    @Override
    public void start(Stage primaryStage)
    {
        FlowPane  root  = new FlowPane();

        // initialisieren Stage
        primaryStage.setTitle("JavaFX-Fenster2 mit Label und Buttons");
        primaryStage.setWidth(480);
        primaryStage.setHeight(260);

        // ToolTip (Quick-Help)
        schaltflaeche1.setTooltip(new Tooltip("Gang um 1 erhöhen"));
        schaltflaeche2.setTooltip(new Tooltip("Geschwindigkeit um 10% vermindern"));

        // Tasten-Hotkey, wird unterstrichen angezeigt, 
        //    funktioniert nur mit ALT-Taste.
        schaltflaeche1.setMnemonicParsing(true);  // 'R'
        schaltflaeche2.setMnemonicParsing(true);  // 'B'

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        root.setAlignment(Pos.TOP_CENTER);
        root.setHgap(15);

        root.getChildren().add(new Label("Auto  "));
        root.getChildren().add(schaltflaeche1);
        root.getChildren().add(schaltflaeche2);

        initListener();
        initAuto();

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // -------------------------------------------------------------------------

    /**
    <p>
    Initialisieren des Fachobjekts.
    </p>
    <p>
    Zum Testen wurden der Einfachheit halber willkürliche Werte fix codiert.
    </p>
    */
    private void initAuto()
    {
        auto = new Auto();
        auto.setBesitzer(new Person(1980, Person.MAENNLICH));
        auto.getBesitzer().aendernVorname("Hugo");
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        schaltflaeche1.setOnAction(new EventHandler<ActionEvent>()
            {
                @Override
                public void handle(ActionEvent evt)
                {
                    schaltflaeche1ereignis(evt);
                }
            });

        schaltflaeche2.setOnAction(new EventHandler<ActionEvent>()
            {
                @Override
                public void handle(ActionEvent evt)
                {
                    schaltflaeche2ereignis(evt);
                }
            });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Methode für Schaltfläche1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void schaltflaeche1ereignis(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************
    
    /**
    Methode für Schaltfläche2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void schaltflaeche2ereignis(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }
}
