/*
 * LabelButtonFensterV.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.geometry.*;

import javafx.event.*;

import at.beringer.oopBuch.einfacheBsp.Person;
import at.beringer.oopBuch.einfacheBsp.Auto;

/**
Hauptfenster mit Text ('Label') und 2 Schaltfächen (sehr einfache 
Ereignisbehandlung!).
<p>
Nur 1 Ereigniswächter für alle Ereignisse. <br />
ACHTUNG: Dies entspricht keinem guten Programmierstil - es soll in diesem ersten
Beispiel mit Ereignisbehandlung nur anschaulich zeigen, was bei einem Ereignis 
passiert und welche Möglichkeiten es für die Ereignisbehandlung gibt.
</p>
<p>
Innere Klasse: 'LabelButtonC' (Ereigniswächter) <br />
Model-Klasse:  'Auto' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Dieses JavaFX-Fenster entspricht dem gleichnamigen Swing-Fenster aus dem 
Paket 'at.beringer.oopBuch.guiSwing'.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

<hr />

@since Juni 2013

@see Auto
@see Person
@see at.beringer.oopBuch.guiSwing.LabelButtonFensterV
@see LabelButtonFenster2V

@author Beringer Alfred
*/
public class LabelButtonFensterV extends Application
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private Button        schaltflaeche1 = new Button("_raufschalten");
    private Button        schaltflaeche2 = new Button("  _bremsen  ");

    private LabelButtonC  waechter       = new LabelButtonC();

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode
    */
    @Override
    public void start(Stage primaryStage)
    {
        FlowPane  root  = new FlowPane();

        // initialisieren Stage
        primaryStage.setTitle("JavaFX-Fenster mit Label und Buttons");
        primaryStage.setWidth(480);
        primaryStage.setHeight(260);

        // ToolTip (Quick-Help)
        schaltflaeche1.setTooltip(new Tooltip("Gang um 1 erhöhen"));
        schaltflaeche2.setTooltip(new Tooltip("Geschwindigkeit um 10% vermindern"));

        // Tasten-Hotkey, wird unterstrichen angezeigt, 
        //    funktioniert nur mit ALT-Taste.
        schaltflaeche1.setMnemonicParsing(true);  // 'R'
        schaltflaeche2.setMnemonicParsing(true);  // 'B'

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        root.setAlignment(Pos.TOP_CENTER);
        root.setHgap(15);

        root.getChildren().add(new Label("Auto  "));
        root.getChildren().add(schaltflaeche1);
        root.getChildren().add(schaltflaeche2);

        // GUI-Komponenten beim Ereigniswächter anmelden

        schaltflaeche1.setOnAction(waechter);
        schaltflaeche2.setOnAction(waechter);

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // *************************************************************************
    // Lokale (innere) Klasse
    // *************************************************************************

    /**
    Ereigniswächter (innere Klasse).
    */
    class LabelButtonC implements EventHandler<ActionEvent>
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------
              
        /** Model-Klasse: Fachobjekt (zum Testen)                             */
        private Auto  auto;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        Standardkonstruktor.
        <p>
        Initialisieren des Fachobjekts.
        </p>
        <p>
        Zum Testen wurden der Einfachheit halber willkürliche Werte fix codiert.
        </p>
        */
        public LabelButtonC()
        {
            auto = new Auto();
            auto.setBesitzer(new Person(1980, Person.MAENNLICH));
            auto.getBesitzer().aendernVorname("Hugo");
        }

        // ---------------------------------------------------------------------
        // API-Methoden überschreiben
        // ---------------------------------------------------------------------

        @Override
        public void handle(ActionEvent evt)
        {
            Object  evtquelle = evt.getSource();

            if (evtquelle == schaltflaeche1)
            {
                auto.raufschalten();
            }
            else if (evtquelle == schaltflaeche2)
            {
                auto.bremsen();
            }
            auto.zeigeInhalt();
        }
    }
    // ********************************************* Ende lokale (innere) Klasse
}
