/*
 * MausbewegungenV.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.control.ScrollPane.ScrollBarPolicy;
import javafx.scene.input.*;
import javafx.scene.layout.*;
import javafx.geometry.Pos;

import javafx.event.*;

/**
Hauptfenster für Mausbewegungen. <br />
Die Mauskoordinaten werden in einem Textbereich gespeichert.
<p>
Anonyme Klasse: von 'EventHandler-MouseEvent'
</p>
<p>
Dieses JavaFX-Fenster entspricht dem gleichnamigen Swing-Fenster aus dem 
Paket 'at.beringer.oopBuch.guiSwing'.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

<hr />

@since Juni 2013

@see at.beringer.oopBuch.guiSwing.MausbewegungenV

@author Beringer Alfred
*/
public class MausbewegungenV extends Application
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    private static final String  NEUEZEILE = 
                                     System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private TextArea    ausgabe       = new TextArea("Start");
    private ScrollPane  ausgabeScroll = new ScrollPane();

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode
    */
    @Override
    public void start(Stage primaryStage)
    {
        FlowPane  root  = new FlowPane();

        // initialisieren Stage
        primaryStage.setTitle("JavaFX-Fenster mit Mausbewegungen");
        primaryStage.setWidth(480);
        primaryStage.setHeight(260);

        // initialisieren Textbereich
        ausgabe.setEditable(false);
        
        ausgabeScroll.setHbarPolicy(ScrollBarPolicy.NEVER);
        ausgabeScroll.setVbarPolicy(ScrollBarPolicy.AS_NEEDED);

        ausgabeScroll.setPrefViewportWidth(200);
        ausgabeScroll.setPrefViewportHeight(80);

        ausgabeScroll.setContent(ausgabe);

        // Komponenten dem Fenster hinzufügen 

        root.setAlignment(Pos.TOP_CENTER);
        root.getChildren().add(ausgabeScroll);

        // GUI-Komponenten bei Ereigniswächtern anmelden

        root.setOnMouseMoved(new EventHandler<MouseEvent>()
              // ***************************************************************
              // anonyme (innere) Klasse
              // ***************************************************************
              {
                  // -----------------------------------------------------------
                  // API-Methoden überschreiben
                  // -----------------------------------------------------------

                  @Override
                  public void handle(MouseEvent evt)
                  {
                      mausEreignis(evt);
                      /* Ereignisobjekt mitgeben, 
                            falls Daten vom Ereignis benötigt werden   */
                  }
              }
              // ********************************** Ende anonyme (innere) Klasse
                            );

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden (private Methoden des Hauptfensters)
    // *************************************************************************
    
    /**
    Methode für Mausbewegung.

    @param  evt  MouseEvent (von API-Methode 'handle' übernommen)
    */
    private void mausEreignis(MouseEvent evt)
    {
        double  x = evt.getX();
        double  y = evt.getY();

        ausgabe.appendText(NEUEZEILE + "x=" + x + "  y=" + y);
    }
}
