/*
 * Menue2V.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.*;
import javafx.stage.*;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.text.*;
import javafx.geometry.*;

import javafx.event.*;

import org.controlsfx.dialog.Dialogs;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit Menüzeile, Menüs (auch "Datei"-Menü) und Untermenüs.
<p>
Diese Fensterklasse entspricht der Fensterklasse 'MenueV', jedoch gibt es ein 
zusätzliches Menü "Datei" mit dem Menüpunkt "Beenden".
</p>
<p>
Anonyme Klasse: von 'EventHandler-ActionEvent' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Dieses JavaFX-Fenster entspricht dem gleichnamigen Swing-Fenster aus dem 
Paket 'at.beringer.oopBuch.guiSwing'.
</p>
<p>
Da es in JavaFX (noch) keine Standard-Dialogfenster gibt (entsprechend der 
Swing-Klasse 'JOptionPane'), wurde die 'Dialogs'-Klasse aus der 
'ControlsFX'-Bibliothek verwendet.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

@since August 2014

@see AutoM
@see at.beringer.oopBuch.guiSwing.Menue2V

@author Beringer Alfred
*/
public class Menue2V extends Application
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private MenuBar       menuezeile   = new MenuBar();

    private Menu          menue1       = new Menu("Datei");
    private MenuItem      menueItem11  = new MenuItem("_neues Auto");
    private MenuItem      menueItem12  = new MenuItem("_Beenden");

    private Menu          menue2       = new Menu("Auto");
    private MenuItem      menueItem21  = new MenuItem("_raufschalten");
    private MenuItem      menueItem22  = new MenuItem("r_unterschalten");
    private MenuItem      menueItem23  = new MenuItem("be_schleunigen");
    private MenuItem      menueItem24  = new MenuItem("_bremsen");

    private Menu          menue3       = new Menu("Menü3");
    private MenuItem      menueItem31  = new MenuItem("1. Menüeintrag Menü3");
    private CheckMenuItem checkItem32  = new CheckMenuItem("2. Check-M3");
    private CheckMenuItem checkItem33  = new CheckMenuItem("3. Check-M3");

    private Menu          menue34      = new Menu("4. Untermenü Menü3");
    private MenuItem      menueItem341 = new MenuItem("1. Menüeintrag Menü34");
    private MenuItem      menueItem342 = new MenuItem("2. Menüeintrag Menü34");

    private ToggleGroup   radiogruppe  = new ToggleGroup();
    private RadioMenuItem radioItem35  = new RadioMenuItem("5. Radio-M3");
    private RadioMenuItem radioItem36  = new RadioMenuItem("6. Radio-M3");
    private RadioMenuItem radioItem37  = new RadioMenuItem("7. Radio-M3");

    private Menu          menue4       = new Menu("Hilfe");
    private MenuItem      menueItem41  = new MenuItem("_Hilfethemen");
    private MenuItem      menueItem42  = new MenuItem("_Info...");

    private Button        button       = new Button("neues Auto");

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode.
    <p>
    Das Erzeugen der Menüzeile, das Initialisieren der Schaltfläche sowie
    das Anmelden bei den Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    @Override
    public void start(Stage primaryStage)
    {
        BorderPane  root  = new BorderPane();
        
        FlowPane    unten = new FlowPane();

        // initialisieren Stage
        primaryStage.setTitle("JavaFX-Fenster mit Menü2");
        primaryStage.setWidth(480);
        primaryStage.setHeight(260);

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        button.setDefaultButton(true);

        initMenueKomponenten();
        initButton();

        menuBarErstellen();

        // Menüzeile in Fenster einfügen
        root.setTop(menuezeile);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        unten.setAlignment(Pos.BOTTOM_CENTER);
        unten.getChildren().add(button);
        root.setBottom(unten);

        initListener();

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Initialisieren der Menü-Komponenten
    */
    private void initMenueKomponenten()
    {
        menueItem11.setMnemonicParsing(true);  // 'N'
        menueItem12.setMnemonicParsing(true);  // 'B'

        menueItem21.setMnemonicParsing(true);  // 'R'
        menueItem22.setMnemonicParsing(true);  // 'U'
        menueItem23.setMnemonicParsing(true);  // 'S'
        menueItem24.setMnemonicParsing(true);  // 'B'

        menueItem41.setMnemonicParsing(true);  // 'H'
        menueItem42.setMnemonicParsing(true);  // 'I'
        
        checkItem32.setSelected(true);
        checkItem33.setSelected(false);
        
        radioItem35.setSelected(false);
        radioItem36.setSelected(true);
        radioItem37.setSelected(false);
    }

    // -------------------------------------------------------------------------

    /**
    Erzeugen der Menüzeile. <br />
    Die Reihenfolge der Hinzufügungen ('add') ist wesentlich.
    */
    private void menuBarErstellen()
    {
        // Menüpunkte in Menü1 einfügen
        menue1.getItems().add(menueItem11);
        menue1.getItems().add(menueItem12);

        // Menüpunkte in Menü2 einfügen

        menue2.getItems().add(menueItem21);
        menue2.getItems().add(menueItem22);
        menue2.getItems().add(menueItem23);
        menue2.getItems().add(menueItem24);

        // Menüpunkte in Menü3 einfügen

        menue3.getItems().add(menueItem31);
        menue3.getItems().add(new SeparatorMenuItem());
        menue3.getItems().add(checkItem32);
        menue3.getItems().add(checkItem33);
        menue3.getItems().add(new SeparatorMenuItem());

        // Menüpunkte in Untermenü34 einfügen
        menue34.getItems().add(menueItem341);
        menue34.getItems().add(menueItem342);
        menue3.getItems().add(menue34);      // Untermenü als Menüpunkt einfügen
        menue3.getItems().add(new SeparatorMenuItem());

        // zusammengehörende Optionsfelder gruppieren
        radioItem35.setToggleGroup(radiogruppe);
        radioItem36.setToggleGroup(radiogruppe);
        radioItem37.setToggleGroup(radiogruppe);

        menue3.getItems().add(radioItem35);
        menue3.getItems().add(radioItem36);
        menue3.getItems().add(radioItem37);

        // Menüpunkte in Menü4 einfügen

        menue4.getItems().add(menueItem41);
        menue4.getItems().add(menueItem42);

        // Menüs in Menüzeile einfügen

        menuezeile.getMenus().addAll(menue1, menue2, menue3, menue4);
    }

    // -------------------------------------------------------------------------

    /**
    Schaltfläche initialisieren.
    */
    private void initButton()
    {
        button.setFont(Font.font("Monospaced", FontWeight.BOLD, 32));
        button.setStyle("-fx-foreground-color: white; " 
                      + "-fx-background-color: magenta;");
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Elemente bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert. <br />
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      neuesAuto(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });
        menueItem11.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      neuesAuto(evt);
                  }
              });
        menueItem12.setOnAction(new EventHandler<ActionEvent>()
                {
                    @Override
                    public void handle(ActionEvent evt)
                    {
                        schliessenFenster(evt);
                    }
                });
        menueItem21.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem21event(evt);
                  }
              });
        menueItem22.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem22event(evt);
                  }
              });
        menueItem23.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem23event(evt);
                  }
              });
        menueItem24.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem24event(evt);
                  }
              });
        menueItem31.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem31event(evt);
                  }
              });
        checkItem32.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem32event(evt);
                  }
              });

        // nicht angemeldet: checkItem33

        // nicht angemeldet: menue34

        radioItem35.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem35event(evt);
                  }
              });
        radioItem36.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem36event(evt);
                  }
              });

        // nicht angemeldet: radioItem37

        menueItem41.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem41event(evt);
                  }
              });
        menueItem42.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem42event(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Methode für "Fenster schließen".

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void schliessenFenster(ActionEvent evt)
    {
        System.exit(0);                                      // Programm beenden
    }

    // *************************************************************************
    
    /**
    Auto-Instanz erzeugen (über Menüauswahl oder über Schaltfläche).

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void neuesAuto(ActionEvent evt)
    {
        auto = new AutoM();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem21event(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem22event(ActionEvent evt)
    {
        auto.runterschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag3.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem23event(ActionEvent evt)
    {
        auto.beschleunigen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag4.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem24event(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem31event(ActionEvent evt)
    {
        auto.ausgeben("1. Menüeintrag (Menü3) ausgewählt");
        checkM33();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem32event(ActionEvent evt)
    {
        auto.ausgeben("2. Menüeintrag (Menü3) ausgewählt");
        checkM33();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem35event(ActionEvent evt)
    {
        auto.ausgeben("5. Menüeintrag (Menü3) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag6.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem36event(ActionEvent evt)
    {
        auto.ausgeben("6. Menüeintrag (Menü3) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü4 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem41event(ActionEvent evt)
    {
        // Es gibt noch kein Standard-Dialogfenster in JavaFX!

        Dialogs.create().title("Info für Menue2V")
                        .masthead("Achtung")
                        .message("Hilfethemen noch nicht installiert!")
                        .showWarning();
    }

    // *************************************************************************

    /**
    Methode für Menü4 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem42event(ActionEvent evt)
    {
        // Es gibt noch kein Standard-Dialogfenster in JavaFX!

        Dialogs.create().title("Info für Menue2V")
                        .masthead(null)
                        .message("Copyright 2014"
                                 + System.getProperty("line.separator")
                                 + "Beringer Alfred")
                        .showInformation();
    }

    // -------------------------------------------------------------------------
    // Hilfsmethoden zu den Ereignisbehandlungsmethoden
    // -------------------------------------------------------------------------

    /**
    Prüfen der Option Menü3-Menüeintrag3.
    */
    private void checkM33()
    {
        if (checkItem33.isSelected())
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü3) hat Hakerl");
        }
        else
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü3) hat kein Hakerl");
        }
    }

    // -------------------------------------------------------------------------

    /**
    'main'-Methode zum Starten des Applikationsfensters.
    Sollte in Java 8 nicht mehr notwendig sein.
    <p>
    'launch' ist eine Klassenmethode von 'Application'
    </p>

    @param  args  Kommandozeilenparameter
    */
    public static void main(String[] args)
    {
        launch(args);
    }
}
