/*
 * MenueV.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.*;
import javafx.stage.*;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.text.*;
import javafx.geometry.*;

import javafx.event.*;

import org.controlsfx.dialog.Dialogs;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit Menüzeile, Menüs und Untermenüs.
<p>
Anonyme Klasse: von 'EventHandler-ActionEvent' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Dieses JavaFX-Fenster entspricht dem gleichnamigen Swing-Fenster aus dem 
Paket 'at.beringer.oopBuch.guiSwing'.
</p>
<p>
Da es in JavaFX (noch) keine Standard-Dialogfenster gibt (entsprechend der 
Swing-Klasse 'JOptionPane'), wurde die 'Dialogs'-Klasse aus der 
'ControlsFX'-Bibliothek verwendet.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

@since Juni 2013
@version August 2014 ('Dialogs'-Klasse aus der 'ControlsFX'-Bibliothek)

@see AutoM
@see at.beringer.oopBuch.guiSwing.MenueV

@author Beringer Alfred
*/
public class MenueV extends Application
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private MenuBar       menuezeile   = new MenuBar();

    private Menu          menue1       = new Menu("Auto");
    private MenuItem      menueItem11  = new MenuItem("_neues Auto");
    private MenuItem      menueItem12  = new MenuItem("_raufschalten");
    private MenuItem      menueItem13  = new MenuItem("r_unterschalten");
    private MenuItem      menueItem14  = new MenuItem("be_schleunigen");
    private MenuItem      menueItem15  = new MenuItem("_bremsen");

    private Menu          menue2       = new Menu("Menü2");
    private MenuItem      menueItem21  = new MenuItem("1. Menüeintrag Menü2");
    private CheckMenuItem checkItem22  = new CheckMenuItem("2. Check-M2");
    private CheckMenuItem checkItem23  = new CheckMenuItem("3. Check-M2");

    private Menu          menue24      = new Menu("4. Untermenü Menü2");
    private MenuItem      menueItem241 = new MenuItem("1. Menüeintrag Menü24");
    private MenuItem      menueItem242 = new MenuItem("2. Menüeintrag Menü24");

    private ToggleGroup   radiogruppe  = new ToggleGroup();
    private RadioMenuItem radioItem25  = new RadioMenuItem("5. Radio-M2");
    private RadioMenuItem radioItem26  = new RadioMenuItem("6. Radio-M2");
    private RadioMenuItem radioItem27  = new RadioMenuItem("7. Radio-M2");

    private Menu          menue3       = new Menu("Hilfe");
    private MenuItem      menueItem31  = new MenuItem("_Hilfethemen");
    private MenuItem      menueItem32  = new MenuItem("_Info...");

    private Button        button       = new Button("neues Auto");

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // �ffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode.
    <p>
    Das Erzeugen der Menüzeile, das Initialisieren der Schaltfläche sowie
    das Anmelden bei den Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    @Override
    public void start(Stage primaryStage)
    {
        BorderPane  root  = new BorderPane();
        
        FlowPane    unten = new FlowPane();

        // initialisieren Stage
        primaryStage.setTitle("JavaFX-Fenster mit Menü");
        primaryStage.setWidth(480);
        primaryStage.setHeight(260);

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        button.setDefaultButton(true);

        initMenueKomponenten();
        initButton();

        menuBarErstellen();

        // Menüzeile in Fenster einfügen
        root.setTop(menuezeile);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        unten.setAlignment(Pos.BOTTOM_CENTER);
        unten.getChildren().add(button);
        root.setBottom(unten);

        initListener();

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Initialisieren der Menü-Komponenten
    */
    private void initMenueKomponenten()
    {
        menueItem11.setMnemonicParsing(true);  // 'N'
        menueItem12.setMnemonicParsing(true);  // 'R'
        menueItem13.setMnemonicParsing(true);  // 'U'
        menueItem14.setMnemonicParsing(true);  // 'S'
        menueItem15.setMnemonicParsing(true);  // 'B'

        menueItem31.setMnemonicParsing(true);  // 'H'
        menueItem32.setMnemonicParsing(true);  // 'I'
        
        checkItem22.setSelected(true);
        checkItem23.setSelected(false);
        
        radioItem25.setSelected(false);
        radioItem26.setSelected(true);
        radioItem27.setSelected(false);
    }

    // -------------------------------------------------------------------------

    /**
    Erzeugen der Menüzeile. <br />
    Die Reihenfolge der Hinzufügungen ('add') ist wesentlich.
    */
    private void menuBarErstellen()
    {
        // Menüpunkte in Menü1 einfügen

        menue1.getItems().add(menueItem11);
        menue1.getItems().add(menueItem12);
        menue1.getItems().add(menueItem13);
        menue1.getItems().add(menueItem14);
        menue1.getItems().add(menueItem15);

        // Menüpunkte in Menü2 einfügen

        menue2.getItems().add(menueItem21);
        menue2.getItems().add(new SeparatorMenuItem());
        menue2.getItems().add(checkItem22);
        menue2.getItems().add(checkItem23);
        menue2.getItems().add(new SeparatorMenuItem());

        // Menüpunkte in Untermenü24 einfügen
        menue24.getItems().add(menueItem241);
        menue24.getItems().add(menueItem242);
        menue2.getItems().add(menue24);      // Untermenü als Menüpunkt einfügen
        menue2.getItems().add(new SeparatorMenuItem());

        // zusammengehörende Optionsfelder gruppieren
        radioItem25.setToggleGroup(radiogruppe);
        radioItem26.setToggleGroup(radiogruppe);
        radioItem27.setToggleGroup(radiogruppe);

        menue2.getItems().add(radioItem25);
        menue2.getItems().add(radioItem26);
        menue2.getItems().add(radioItem27);

        // Menüpunkte in Menü3 einfügen

        menue3.getItems().add(menueItem31);
        menue3.getItems().add(menueItem32);

        // Menüs in Menüzeile einfügen

        menuezeile.getMenus().addAll(menue1, menue2, menue3);
    }

    // -------------------------------------------------------------------------

    /**
    Schaltfläche initialisieren.
    */
    private void initButton()
    {
        button.setFont(Font.font("Monospaced", FontWeight.BOLD, 32));
        button.setStyle("-fx-foreground-color: white; " 
                      + "-fx-background-color: magenta;");
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Elemente bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert. <br />
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      neuEvent(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });
        menueItem11.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      neuEvent(evt);
                  }
              });
        menueItem12.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem12event(evt);
                  }
              });
        menueItem13.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem13event(evt);
                  }
              });
        menueItem14.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem14event(evt);
                  }
              });
        menueItem15.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem15event(evt);
                  }
              });
        menueItem21.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem21event(evt);
                  }
              });
        checkItem22.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem22event(evt);
                  }
              });

        // nicht angemeldet: checkItem23

        // nicht angemeldet: menue24

        radioItem25.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem25event(evt);
                  }
              });
        radioItem26.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem26event(evt);
                  }
              });

        // nicht angemeldet: radioItem27

        menueItem31.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem31event(evt);
                  }
              });
        menueItem32.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      mItem32event(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Auto-Instanz erzeugen (über Menüauswahl oder über Schaltfläche).

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void neuEvent(ActionEvent evt)
    {
        auto = new AutoM();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem12event(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag3.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem13event(ActionEvent evt)
    {
        auto.runterschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag4.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem14event(ActionEvent evt)
    {
        auto.beschleunigen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem15event(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem21event(ActionEvent evt)
    {
        auto.ausgeben("1. Menüeintrag (Menü2) ausgewählt");
        checkM23();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem22event(ActionEvent evt)
    {
        auto.ausgeben("2. Menüeintrag (Menü2) ausgewählt");
        checkM23();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem25event(ActionEvent evt)
    {
        auto.ausgeben("5. Menüeintrag (Menü2) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag6.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem26event(ActionEvent evt)
    {
        auto.ausgeben("6. Menüeintrag (Menü2) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem31event(ActionEvent evt)
    {
        // Es gibt noch kein Standard-Dialogfenster in JavaFX!

        Dialogs.create().title("Info für MenueV")
                        .masthead("Achtung")
                        .message("Hilfethemen noch nicht installiert!")
                        .showWarning();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void mItem32event(ActionEvent evt)
    {
        // Es gibt noch kein Standard-Dialogfenster in JavaFX!

        Dialogs.create().title("Info für MenueV")
                        .masthead(null)
                        .message("Copyright 2013"
                                 + System.getProperty("line.separator")
                                 + "Beringer Alfred")
                        .showInformation();
    }

    // -------------------------------------------------------------------------
    // Hilfsmethoden zu den Ereignisbehandlungsmethoden
    // -------------------------------------------------------------------------

    /**
    Prüfen der Option Menü2-Menüeintrag3.
    */
    private void checkM23()
    {
        if (checkItem23.isSelected())
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü2) hat Hakerl");
        }
        else
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü2) hat kein Hakerl");
        }
    }

    // -------------------------------------------------------------------------

    /**
    'main'-Methode zum Starten des Applikationsfensters.
    Sollte in Java 8 nicht mehr notwendig sein.
    <p>
    'launch' ist eine Klassenmethode von 'Application'
    </p>

    @param  args  Kommandozeilenparameter
    */
    public static void main(String[] args)
    {
        launch(args);
    }
}
