/*
 * MusterfensterV.java
 */
package at.beringer.oopBuch.guiJavaFX;

import javafx.application.Application;
import javafx.stage.Stage;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.control.ScrollPane.ScrollBarPolicy;
import javafx.scene.input.*;
import javafx.scene.layout.*;
import javafx.geometry.*;
import javafx.collections.*;

import javafx.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit einfachen Standard-GUI-Komponenten.
<p>
Anonyme Klassen: von 'EventHandler-ActionEvent' <br />
                 und von 'EventHandler-MouseEvent' <br />
Model-Klasse:    'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Dieses JavaFX-Fenster entspricht dem gleichnamigen Swing-Fenster aus dem 
Paket 'at.beringer.oopBuch.guiSwing'.
</p>

<dl>
<dt>Bem.:
  <dd>Eine JavaFX-Applikation wird mit der Klassenmethode 'launch' der Klasse 
      'Application' gestartet.
  <dd>Seit Java 8.0 kann eine JavaFX-Applikation direkt gestartet werden, d.h. 
      eine 'main'-Methode (mit 'Application.launch(args)') ist nicht mehr 
      notwendig.
</dt>
</dl>

<hr />

@since Juni 2013

@see AutoM
@see at.beringer.oopBuch.guiSwing.MusterfensterV

@author Beringer Alfred
*/
public class MusterfensterV extends Application
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    private static final String  NEUEZEILE = 
                                     System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private Label        label        = new Label("Label");
    private Button       button1      = new Button("Schaltfläche für Check");
    private Button       button2      = new Button("Schaltfläche für Liste");

    private CheckBox     check1       = new CheckBox("Check1");
    private CheckBox     check2       = new CheckBox("Check2");
    private CheckBox     check3       = new CheckBox("Check3");

    private ToggleGroup  buttongruppe = new ToggleGroup();
    private RadioButton  radio1       = new RadioButton("raufschalten");
    private RadioButton  radio2       = new RadioButton("bremsen     ");
    private RadioButton  radio3       = new RadioButton("check3 aus! ");

    private Label        labTxfeld    = new Label(" \"return\" ... beendet die Eingabe: ");
    private TextField    textfeld     = new TextField("Textfeld");

    private Label        labTxbereich = new Label(" \"return\" ... nur Zeilenvorschub: ");
    private TextArea     textbereich  = new TextArea("Textbereich-Zeile1" + NEUEZEILE
                                                     + "Zeile2");
    private ScrollPane   textscroll   = new ScrollPane();

    private Label        labListe1    = new Label("nur 1 einziges Element auswählbar: ");
    private ListView<String> liste1   = new ListView<>();
    private ScrollPane   liste1scroll = new ScrollPane();

    // Der Einfachheit halber fix codierte Daten f�r Listeninhalt
    private String[]     liste1Daten  = {
                                          "Liste1-Element1",
                                          "Liste1-Element2",
                                          "Liste1-Element3",
                                          "Liste1-Element4",
                                          "Liste1-Element5",
                                          "Liste1-Element6",
                                          "Liste1-Element7",
                                          "Liste1-Element8"
                                        };

    private Label        labListe2    = new Label("mehrere Elemente auswählbar: ");
    private ListView<String> liste2   = new ListView<>();

    // Der Einfachheit halber fix codierte Daten für Listeninhalt
    private ObservableList<String> liste2Daten = 
            FXCollections.observableArrayList(
                                               "Liste2-Element1",
                                               "Liste2-Element2",
                                               "Liste2-Element3",
                                               "Liste2-Element4"
                                             );

    private Label        labCombo     = new Label("Das ist eine Auswahl-Komponente: ");

    // Der Einfachheit halber fix initialisierte Daten für Listeninhalt
    private ComboBox<String> combo    = new ComboBox<>();
    private String[]     comboDaten   = {
                                          "Combo-Element1",
                                          "Combo-Element2",
                                          "Combo-Element3"
                                        };

    /** Aktionszähler                                                         */
    private int  zaehler = 0;

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'start'-Methode.
    <p>
    Das Erzeugen der einzelnen Teilfenster (Panes) und das Anmelden bei den
    Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    @Override
    public void start(Stage primaryStage)
    {
        BorderPane root  = new BorderPane();

        // initialisieren Stage
        primaryStage.setTitle("Musterfenster mit vordefinierten JavaFX-Komponenten");
        primaryStage.setWidth(700);
        primaryStage.setHeight(900);

        initKomponenten();
        initFenster(root);
        initListener();

        primaryStage.setScene(new Scene(root));
        primaryStage.show();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Eigenschaften der GUI-Komponenten definieren.
    */
    private void initKomponenten()
    {
        // Kontrollkästchen
        check1.setSelected(false);
        check2.setSelected(true);
        check3.setSelected(true);

        // Optionsfelder
        radio1.setToggleGroup(buttongruppe);
        radio2.setToggleGroup(buttongruppe);
        radio3.setToggleGroup(buttongruppe);

        radio3.setSelected(true);

        // Textfeld (einzeilig)
        textfeld.setMaxWidth(200.0);
        textfeld.setEditable(true);

        // Textbereich (mehrzeilig)
        textscroll.setHbarPolicy(ScrollBarPolicy.AS_NEEDED);
        textscroll.setVbarPolicy(ScrollBarPolicy.AS_NEEDED);

        textscroll.setPrefViewportWidth(300);
        textscroll.setPrefViewportHeight(100);

        textscroll.setContent(textbereich);
        textbereich.setEditable(true);

        // Listen

        liste1scroll.setContent(liste1);
        liste1scroll.setPrefViewportWidth(300);
        liste1scroll.setFitToHeight(true);

        liste1.getSelectionModel().setSelectionMode(SelectionMode.SINGLE);

        // Liste1 füllen (Zeilen einzeln hinzufügen)
        for (String zeile : liste1Daten)
        {
            liste1.getItems().add(zeile);
        }

        liste2.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
                              // nicht notwendig, denn MULTIPLE ist Standardwert
        liste2.setPrefHeight(150.0);

        // Liste2 füllen (automatisch mit einer JavaFX-Collection)
        liste2.setItems(liste2Daten);

        // ComboBox füllen (automatisch mit einem Array)
        combo.getItems().addAll(comboDaten);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    
    @param  root  Basis-Panel
    */
    private void initFenster(BorderPane root)
    {
        FlowPane  poben1 = new FlowPane();
        FlowPane  poben2 = new FlowPane();
        FlowPane  poben3 = new FlowPane();
        GridPane  poben  = new GridPane();

        GridPane  pmitte4 = new GridPane();
        GridPane  pmitte5 = new GridPane();
        GridPane  pmitte6 = new GridPane();
        GridPane  pmitte7 = new GridPane();
        GridPane  pmitte8 = new GridPane();
        GridPane  pmitte  = new GridPane();

        // Initialisieren 'poben1' für Schaltflächen
        poben1.setHgap(10.0);
        poben1.getChildren().add(label);
        poben1.getChildren().add(button1);
        poben1.getChildren().add(button2);

        // Initialisieren 'poben2' für Kontrollkästchen
        poben2.setHgap(10.0);
        poben2.getChildren().add(check1);
        poben2.getChildren().add(check2);
        poben2.getChildren().add(check3);

        // Initialisieren 'poben3' für Optionsfelder
        initPoben3(poben3);

        // Initialisieren 'poben'
        poben.setVgap(10.0);
        poben.setPadding(new Insets(0, 0, 5, 0));
        poben.setAlignment(Pos.TOP_CENTER);

        poben.add(poben1, 1, 1);    // zuerst Spalte, dann Zeile
        poben.add(poben2, 1, 2);
        poben.add(poben3, 1, 3);

        // Initialisieren Panels für 'pmitte'
        initPanelMitBeschriftung(pmitte4, labTxfeld,    textfeld);
        initPanelMitBeschriftung(pmitte5, labTxbereich, textscroll);
        initPanelMitBeschriftung(pmitte6, labListe1,    liste1scroll);
        initPanelMitBeschriftung(pmitte7, labListe2,    liste2);
        initPanelMitBeschriftung(pmitte8, labCombo,     combo);

        // Initialisieren 'pmitte'
        pmitte.setVgap(10.0);
        pmitte.setPadding(new Insets(0, 0, 5, 0));
        pmitte.setAlignment(Pos.CENTER);

        pmitte.add(pmitte4, 1, 1);    // zuerst Spalte, dann Zeile
        pmitte.add(pmitte5, 1, 2);
        pmitte.add(pmitte6, 1, 3);
        pmitte.add(pmitte7, 1, 4);
        pmitte.add(pmitte8, 1, 5);

        // Komponenten dem Fenster hinzufügen

        root.setTop(poben);
        root.setCenter(pmitte);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren 'poben3' für Optionsfelder.

    @param  p  'poben3' für Optionsfelder
    */
    private void initPoben3(FlowPane p)
    {
        p.setHgap(10.0);
        p.getChildren().add(radio1);
        p.getChildren().add(radio2);
        p.getChildren().add(radio3);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren Panelzeile für eine GUI-Komponente mit Beschriftung. <br />
    Die Beschriftung steht links - die Komponente rechts daneben.

    @param  p             Panel für GUI-Komponente mit Beschriftung
    @param  beschriftung  Beschriftung (Label)
    @param  komponente    GUI-Komponente
    */
    private void initPanelMitBeschriftung(GridPane p,
                                          Label    beschriftung,
                                          Control  komponente)
    {
        FlowPane  plinks  = new FlowPane();
        FlowPane  prechts = new FlowPane();

        plinks.setAlignment(Pos.CENTER_RIGHT);
        prechts.setAlignment(Pos.CENTER_LEFT);

        plinks.getChildren().add(beschriftung);
        prechts.getChildren().add(komponente);
        p.add(plinks,  1, 1);
        p.add(prechts, 2, 1);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button1.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      button1event(evt);
                  }
              });    // button1 ist zuständig für Kontrollkästchen
        button2.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      button2event(evt);
                  }
              });    // button2 ist zuständig für Listen
        check1.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      check1event(evt);
                  }
              });     // nicht notwendig, zuständig ist button1!
    
        // check2 und check3 nicht angemeldet (zuständig ist button1)
    
        radio1.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      radio1event(evt);
                  }
              });
        radio2.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      radio2event(evt);
                  }
              });
        radio3.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      radio3event(evt);
                  }
              });
        textfeld.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      textfeldevent(evt);
                  }
              });
        liste1.setOnMouseClicked(new EventHandler<MouseEvent>()
              {
                  @Override
                  public void handle(MouseEvent evt)
                  {
                      liste1event(evt);
                  }
              });
        combo.setOnAction(new EventHandler<ActionEvent>()
              {
                  @Override
                  public void handle(ActionEvent evt)
                  {
                      comboevent(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Methode für Schaltfläche1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void button1event(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Schaltfl�che1 gedr�ckt");

        if (check1.isSelected())
        {
            auto.ausgeben("     check1 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check1 leer");
        }

        if (check2.isSelected())
        {
            auto.ausgeben("     check2 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check2 leer");
        }

        if (check3.isSelected())
        {
            auto.ausgeben("     check3 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check3 leer");
        }
    }

    // *************************************************************************

    /**
    Methode für Schaltfläche2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void button2event(ActionEvent evt)
    {
        java.util.List<String>  inhalte = liste2.getSelectionModel().getSelectedItems();
        java.util.List<Integer> indices = liste2.getSelectionModel().getSelectedIndices();
        Integer[] intIndices            = new Integer[indices.size()];

        indices.toArray(intIndices);

        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");

        auto.ausgeben("Schaltfläche2 gedrückt, ausgewählte Zeile von Liste1: ");
        auto.ausgeben("            " +  liste1.getSelectionModel().getSelectedItem()
                              + " (" + (liste1.getSelectionModel().getSelectedIndex()+1)
                              + ". Zeile)");

        auto.ausgeben("                          ausgewählte Zeilen von Liste2: ");
        for (int i = 0; i < inhalte.size(); i++)
        {
            auto.ausgeben("            " + inhalte.get(i)
                                  + " (" + (intIndices[i]+1) 
                                  + ". Zeile)");
        }
    }

    // *************************************************************************

    /**
    Methode für Kontrollkästchen1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void check1event(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("check1 gedrückt");
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld1.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void radio1event(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("radio1 gedrückt (raufschalten)");
        auto.raufschalten();
        auto.zeigeInhalt();
        textbereich.appendText(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld2.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void radio2event(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("radio2 gedrückt (bremsen)");
        auto.bremsen();
        auto.zeigeInhalt();
        textbereich.appendText(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld3.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void radio3event(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!"
                               + NEUEZEILE + auto);
        auto.ausgeben("radio3 gedrückt, check3 wird leer");
        check3.setSelected(false);
    }

    // *************************************************************************

    /**
    Methode für Textfeld.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void textfeldevent(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Textfeld geändert: " + textfeld.getText());
    }

    // *************************************************************************

    /**
    Methode für Listenfeld1.

    @param  evt  MouseEvent (von API-Methode 'handle' übernommen)
    */
    private void liste1event(MouseEvent evt)
    {
        auto.ausgeben("Mauswächter: Liste1, ausgewählte Zeile von Liste1: ");
        auto.ausgeben("            " +  liste1.getSelectionModel().getSelectedItem()
                              + " (" + (liste1.getSelectionModel().getSelectedIndex() + 1) 
                              + ". Zeile)");
        auto.ausgeben("    Mausklick in "
                      + (liste1.getSelectionModel().getSelectedIndex() + 1)
                      + ". Zeile");
    }

    // *************************************************************************

    /**
    Methode für Auswahlfeld.

    @param  evt  ActionEvent (von API-Methode 'handle' übernommen)
    */
    private void comboevent(ActionEvent evt)
    {
        textbereich.appendText(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Auswahl Combo: " +  combo.getSelectionModel().getSelectedItem()
                                 + " (" + (combo.getSelectionModel().getSelectedIndex() + 1) 
                                 + ". Zeile)");
    }
}
