/*
 * LabelButtonFensterV.java
 */
package at.beringer.oopBuch.guiSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.Auto;
import at.beringer.oopBuch.einfacheBsp.Person;

/**
Hauptfenster mit Text ('Label') und 2 Schaltfächen (sehr einfache 
Ereignisbehandlung!).
<p>
Nur 1 Ereigniswächter für alle Ereignisse. <br />
ACHTUNG: Dies entspricht keinem guten Programmierstil - es soll in diesem ersten
Beispiel mit Ereignisbehandlung nur anschaulich zeigen, was bei einem Ereignis 
passiert und welche Möglichkeiten es für die Ereignisbehandlung gibt.
</p>
<p>
Innere Klasse: 'LabelButtonC' (Ereigniswächter) <br />
Model-Klasse:  'Auto' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Im Paket 'at.beringer.oopBuch.guiSwingI18N' befindet sich eine vollständig 
internationalisierte - und parametrisierte (Fenstergröße) - Version dieser 
Fensterklasse (mit individuellen Ereigniswächtern).
</p>

<hr />

@since Mai 2000
@version Jänner 2002 (Swing ab JDK 1.3, Auto)
@version <br />Juni 2012 (ohne ContentPane, ab JDK 6 möglich)

@see Auto
@see Person
@see ProgrammLabelButtonFenster
@see ProgrammLabelButtonFensterUI
@see at.beringer.oopBuch.guiSwingI18N.LabelButtonFensterV
@see at.beringer.oopBuch.guiJavaFX.LabelButtonFensterV

@author Beringer Alfred
*/
public class LabelButtonFensterV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private JButton       schaltflaeche1 = new JButton("raufschalten");
    private JButton       schaltflaeche2 = new JButton("  bremsen  ");

    private LabelButtonC  waechter       = new LabelButtonC();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public LabelButtonFensterV()
    {
        this.setTitle("Swing-Fenster mit Label und Buttons");
        this.setSize(480, 260);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // ToolTip (Quick-Help)
        schaltflaeche1.setToolTipText("Gang um 1 erhöhen");
        schaltflaeche2.setToolTipText("Geschwindigkeit um 10% vermindern");

        // Tasten-Hotkey, wird unterstrichen angezeigt, 
        //    funktioniert nur mit ALT-Taste.
        schaltflaeche1.setMnemonic('R');
        schaltflaeche2.setMnemonic('B');

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        /*
           Die Content-Pane des Hauptfensters beinhaltet die GUI-Komponenten des
           Fensters und verwaltet diese.
           Bis Java 5 muss die Content-Pane direkt angesprochen werden - u.a. 
           wird das Layout des Fensters in der Content-Pane definiert und die 
           GUI-Komponenten werden der Content-Pane hinzugefügt:

        Container  cp = this.getContentPane();
        cp.setLayout(new FlowLayout());

        cp.add(... GUI-Komponente ...);
        ...
         
           Ab JDK 6 erfolgt die Kommunikation mit der Content-Pane über das 
           Fenster selbst, die Content-Pane wird daher im Allg. nicht mehr 
           benötigt.
        */

        this.setLayout(new FlowLayout());

        this.add(new JLabel("Auto  "));
        this.add(schaltflaeche1);
        this.add(schaltflaeche2);

        // GUI-Komponenten beim Ereigniswächter anmelden

        schaltflaeche1.addActionListener(waechter);  
        schaltflaeche2.addActionListener(waechter);
    }

    // *************************************************************************
    // Lokale (innere) Klasse
    // *************************************************************************

    /**
    Ereigniswächter (innere Klasse).
    */
    class LabelButtonC implements ActionListener
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------
              
        /** Model-Klasse: Fachobjekt (zum Testen)                             */
        private Auto  auto;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        Standardkonstruktor.
        <p>
        Initialisieren des Fachobjekts.
        </p>
        <p>
        Zum Testen wurden der Einfachheit halber willkürliche Werte fix codiert.
        </p>
        */
        public LabelButtonC()
        {
            auto = new Auto();
            auto.setBesitzer(new Person(1980, Person.MAENNLICH));
            auto.getBesitzer().aendernVorname("Hugo");
        }

        // ---------------------------------------------------------------------
        // API-Methoden überschreiben
        // ---------------------------------------------------------------------

        @Override
        public void actionPerformed(ActionEvent evt)
        {
            Object  evtquelle = evt.getSource();

            if (evtquelle == schaltflaeche1)
            {
                auto.raufschalten();
            }
            else if (evtquelle == schaltflaeche2)
            {
                auto.bremsen();
            }
            auto.zeigeInhalt();
        }
    }
    // ********************************************* Ende lokale (innere) Klasse

}
