/*
 * MausbewegungenV.java
 */
package at.beringer.oopBuch.guiSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
Hauptfenster für Mausbewegungen. <br />
Die Mauskoordinaten werden in einem Textbereich gespeichert.
<p>
Anonyme Klasse: von 'MouseMotionAdapter'
</p>
<p>
Im Paket 'at.beringer.oopBuch.guiSwingI18N' befindet sich eine vollständig 
internationalisierte - und parametrisierte (Fenstergröße) - Version dieser 
Fensterklasse.
</p>

<hr />

@since Mai 2000
@version Jänner 2002 (Swing ab JDK 1.3)
@version <br />Juni 2012 (ohne ContentPane, ab JDK 6 möglich)

@see ProgrammMausbewegungen
@see at.beringer.oopBuch.guiSwingI18N.MausbewegungenV
@see at.beringer.oopBuch.guiJavaFX.MausbewegungenV

@author Beringer Alfred
*/
public class MausbewegungenV extends JFrame
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    private static final String  NEUEZEILE = 
                                     System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private JTextArea    ausgabe       = new JTextArea("Start", 8, 20);
    private JScrollPane  ausgabeScroll = new JScrollPane(ausgabe);

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public MausbewegungenV()
    {
        this.setTitle("Swing-Fenster mit Mausbewegungen");
        this.setSize(480, 260);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // initialisieren Textbereich
        ausgabe.setEditable(false);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new FlowLayout());

        this.add(ausgabeScroll);

        // GUI-Komponenten bei Ereigniswächtern anmelden

        this.addMouseMotionListener(new MouseMotionAdapter()
              // ***************************************************************
              // anonyme (innere) Klasse
              // ***************************************************************
              {
                  // -----------------------------------------------------------
                  // API-Methoden überschreiben
                  // -----------------------------------------------------------

                  @Override
                  public void mouseMoved(MouseEvent evt)
                  {
                      mausEreignis(evt);
                      /* Ereignisobjekt mitgeben, 
                            falls Daten vom Ereignis benötigt werden   */
                  }
              }
              // ********************************** Ende anonyme (innere) Klasse
                                   );
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden (private Methoden des Hauptfensters)
    // *************************************************************************
    
    /**
    Methode für Mausbewegung.

    @param  evt  MouseEvent (von API-Methode 'mouseMoved' übernommen)
    */
    private void mausEreignis(MouseEvent evt)
    {
        int  x = evt.getX();
        int  y = evt.getY();

        ausgabe.append(NEUEZEILE + "x=" + x + "  y=" + y);
    }
}
