/*
 * Menue2V.java
 */
package at.beringer.oopBuch.guiSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit Menüzeile, Menüs (auch "Datei"-Menü) und Untermenüs.
<p>
Diese Fensterklasse entspricht der Fensterklasse 'MenueV', jedoch gibt es ein 
zusätzliches Menü "Datei" mit dem Menüpunkt "Beenden".
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>

<hr />

@since August 2014

@see AutoM
@see ProgrammMenue2
@see MenueV
@see StandardfensterV

@author Beringer Alfred
*/
public class Menue2V extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private JMenuBar             menuezeile   = new JMenuBar();

    private JMenu                menue1       = new JMenu("Datei");
    private JMenuItem            menueItem11  = new JMenuItem("neues Auto", 'N');
    private JMenuItem            menueItem12  = new JMenuItem("Beenden",    'B');

    private JMenu                menue2       = new JMenu("Auto");
    private JMenuItem            menueItem21  = new JMenuItem("raufschalten",   'R');
    private JMenuItem            menueItem22  = new JMenuItem("runterschalten", 'U');
    private JMenuItem            menueItem23  = new JMenuItem("beschleunigen",  'S');
    private JMenuItem            menueItem24  = new JMenuItem("bremsen",        'B');

    private JMenu                menue3       = new JMenu("Menü3");
    private JMenuItem            menueItem31  = new JMenuItem("1. Menüeintrag Menü3");
    private JCheckBoxMenuItem    checkItem32  = new JCheckBoxMenuItem("2. Check-M3", true);
    private JCheckBoxMenuItem    checkItem33  = new JCheckBoxMenuItem("3. Check-M3", false);

    private JMenu                menue34      = new JMenu("4. Untermenü Menü3");
    private JMenuItem            menueItem341 = new JMenuItem("1. Menüeintrag Menü34");
    private JMenuItem            menueItem342 = new JMenuItem("2. Menüeintrag Menü34");

    private ButtonGroup          radiogruppe  = new ButtonGroup();
    private JRadioButtonMenuItem radioItem35  = new JRadioButtonMenuItem("5. Radio-M3", false);
    private JRadioButtonMenuItem radioItem36  = new JRadioButtonMenuItem("6. Radio-M3", true);
    private JRadioButtonMenuItem radioItem37  = new JRadioButtonMenuItem("7. Radio-M3", false);

    private JMenu                menue4       = new JMenu("Hilfe");
    private JMenuItem            menueItem41  = new JMenuItem("Hilfethemen", 'H');
    private JMenuItem            menueItem42  = new JMenuItem("Info...",     'I');

    private JButton              button       = new JButton("neues Auto");

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Das Erzeugen der Menüzeile, das Initialisieren der Schaltfläche sowie
    das Anmelden bei den Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    public Menue2V()
    {
        this.setTitle("Swing-Fenster mit Menü2");
        this.setSize(480, 260);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(button);

        initButton();

        menuBarErstellen();

        // Menüzeile in Fenster einfügen
        this.setJMenuBar(menuezeile);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new BorderLayout());
        this.add(button,BorderLayout.SOUTH);

        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Erzeugen der Menüzeile. <br />
    Die Reihenfolge der Hinzufügungen ('add') ist wesentlich.
    */
    private void menuBarErstellen()
    {
        // Menüpunkte in Menü1 einfügen

        menue1.add(menueItem11);
        menue1.add(menueItem12);

        // Menüpunkte in Menü2 einfügen

        menue2.add(menueItem21);
        menue2.add(menueItem22);
        menue2.add(menueItem23);
        menue2.add(menueItem24);

        // Menüpunkte in Menü3 einfügen

        menue3.add(menueItem31);
        menue3.addSeparator();
        menue3.add(checkItem32);
        menue3.add(checkItem33);
        menue3.addSeparator();

        // Menüpunkte in Untermenü34 einfügen
        menue34.add(menueItem341);
        menue34.add(menueItem342);
        menue3.add(menue34);                 // Untermenü als Menüpunkt einfügen
        menue3.addSeparator();

        // zusammengehörende Optionsfelder gruppieren
        radiogruppe.add(radioItem35);
        radiogruppe.add(radioItem36);
        radiogruppe.add(radioItem37);

        menue3.add(radioItem35);
        menue3.add(radioItem36);
        menue3.add(radioItem37);

        // Menüpunkte in Menü4 einfügen

        menue4.add(menueItem41);
        menue4.add(menueItem42);

        // Menüs in Menüzeile einfügen

        menuezeile.add(menue1);
        menuezeile.add(menue2);
        menuezeile.add(menue3);
        menuezeile.add(menue4);
    }

    // -------------------------------------------------------------------------

    /**
    Schaltfläche initialisieren.
    */
    private void initButton()
    {
        button.setFont(new Font("Monospaced", Font.BOLD, 32));
        button.setForeground(Color.WHITE);
        button.setBackground(Color.MAGENTA);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Elemente bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert. <br />
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private 
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese 
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis 
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      neuesAuto(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });
        menueItem11.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      neuesAuto(evt);
                  }
              });
        menueItem12.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      schliessenFenster(evt);
                  }
              });
        menueItem21.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem21event(evt);
                  }
              });
        menueItem22.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem22event(evt);
                  }
              });
        menueItem23.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem23event(evt);
                  }
              });
        menueItem24.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem24event(evt);
                  }
              });
        menueItem31.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem31event(evt);
                  }
              });
        checkItem32.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem32event(evt);
                  }
              });

        // nicht angemeldet: checkItem33

        // nicht angemeldet: menue34

        radioItem35.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem35event(evt);
                  }
              });
        radioItem36.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem36event(evt);
                  }
              });

        // nicht angemeldet: radioItem37

        menueItem41.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem41event(evt);
                  }
              });
        menueItem42.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem42event(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Methode für "Fenster schließen".

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void schliessenFenster(ActionEvent evt)
    {
        this.setVisible(false);
        this.dispose();
        System.exit(0);                                      // Programm beenden
    }

    // *************************************************************************

    /**
    Auto-Instanz erzeugen (über Menüauswahl oder über Schaltfläche).

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void neuesAuto(ActionEvent evt)
    {
        auto = new AutoM();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem21event(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem22event(ActionEvent evt)
    {
        auto.runterschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag3.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem23event(ActionEvent evt)
    {
        auto.beschleunigen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag4.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem24event(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem31event(ActionEvent evt)
    {
        auto.ausgeben("1. Menüeintrag (Menü3) ausgewählt");
        checkM33();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem32event(ActionEvent evt)
    {
        auto.ausgeben("2. Menüeintrag (Menü3) ausgewählt");
        checkM33();
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem35event(ActionEvent evt)
    {
        auto.ausgeben("5. Menüeintrag (Menü3) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag6.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem36event(ActionEvent evt)
    {
        auto.ausgeben("6. Menüeintrag (Menü3) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü4 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem41event(ActionEvent evt)
    {
        JOptionPane.showMessageDialog(this,
                                      "Achtung"
                                          + System.getProperty("line.separator")
                                        + "Hilfethemen noch nicht installiert!",
                                      "Info für Menue2V",
                                      JOptionPane.WARNING_MESSAGE);
    }

    // *************************************************************************

    /**
    Methode für Menü4 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem42event(ActionEvent evt)
    {
        JOptionPane.showMessageDialog(this,
                                      "Copyright 2014"
                                          + System.getProperty("line.separator")
                                          + "Beringer Alfred",
                                      "Info für Menue2V",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    // -------------------------------------------------------------------------
    // Hilfsmethoden zu den Ereignisbehandlungsmethoden
    // -------------------------------------------------------------------------

    /**
    Prüfen der Option Menü3-Menüeintrag3.
    */
    private void checkM33()
    {
        if (checkItem33.getState())
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü3) hat Hakerl");
        }
        else
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü3) hat kein Hakerl");
        }
    }
}
