/*
 * MenueV.java
 */
package at.beringer.oopBuch.guiSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit Menüzeile, Menüs und Untermenüs.
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>

<hr />

@since Mai 2000
@version Jänner 2002 (Swing ab JDK 1.3)
@version <br />Juni 2012 (ohne ContentPane, ab JDK 6 möglich)

@see AutoM
@see ProgrammMenue
@see Menue2V
@see StandardfensterV

@author Beringer Alfred
*/
public class MenueV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
        
    private JMenuBar             menuezeile   = new JMenuBar();

    private JMenu                menue1       = new JMenu("Auto");
    private JMenuItem            menueItem11  = new JMenuItem("neues Auto",     'N');
    private JMenuItem            menueItem12  = new JMenuItem("raufschalten",   'R');
    private JMenuItem            menueItem13  = new JMenuItem("runterschalten", 'U');
    private JMenuItem            menueItem14  = new JMenuItem("beschleunigen",  'S');
    private JMenuItem            menueItem15  = new JMenuItem("bremsen",        'B');

    private JMenu                menue2       = new JMenu("Menü2");
    private JMenuItem            menueItem21  = new JMenuItem("1. Menüeintrag Menü2");
    private JCheckBoxMenuItem    checkItem22  = new JCheckBoxMenuItem("2. Check-M2", true);
    private JCheckBoxMenuItem    checkItem23  = new JCheckBoxMenuItem("3. Check-M2", false);

    private JMenu                menue24      = new JMenu("4. Untermenü Menü2");
    private JMenuItem            menueItem241 = new JMenuItem("1. Menüeintrag Menü24");
    private JMenuItem            menueItem242 = new JMenuItem("2. Menüeintrag Menü24");

    private ButtonGroup          radiogruppe  = new ButtonGroup();
    private JRadioButtonMenuItem radioItem25  = new JRadioButtonMenuItem("5. Radio-M2", false);
    private JRadioButtonMenuItem radioItem26  = new JRadioButtonMenuItem("6. Radio-M2", true);
    private JRadioButtonMenuItem radioItem27  = new JRadioButtonMenuItem("7. Radio-M2", false);

    private JMenu                menue3       = new JMenu("Hilfe");
    private JMenuItem            menueItem31  = new JMenuItem("Hilfethemen", 'H');
    private JMenuItem            menueItem32  = new JMenuItem("Info...",     'I');

    private JButton              button       = new JButton("neues Auto");

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Das Erzeugen der Menüzeile, das Initialisieren der Schaltfläche sowie
    das Anmelden bei den Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    public MenueV()
    {
        this.setTitle("Swing-Fenster mit Menü");
        this.setSize(480, 260);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(button);

        initButton();

        menuBarErstellen();

        // Menüzeile in Fenster einfügen
        this.setJMenuBar(menuezeile);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new BorderLayout());
        this.add(button,BorderLayout.SOUTH);

        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Erzeugen der Menüzeile. <br />
    Die Reihenfolge der Hinzufügungen ('add') ist wesentlich.
    */
    private void menuBarErstellen()
    {
        // Menüpunkte in Menü1 einfügen

        menue1.add(menueItem11);
        menue1.add(menueItem12);
        menue1.add(menueItem13);
        menue1.add(menueItem14);
        menue1.add(menueItem15);

        // Menüpunkte in Menü2 einfügen

        menue2.add(menueItem21);
        menue2.addSeparator();
        menue2.add(checkItem22);
        menue2.add(checkItem23);
        menue2.addSeparator();

        // Menüpunkte in Untermenü24 einfügen
        menue24.add(menueItem241);
        menue24.add(menueItem242);
        menue2.add(menue24);                 // Untermenü als Menüpunkt einfügen
        menue2.addSeparator();

        // zusammengehörende Optionsfelder gruppieren
        radiogruppe.add(radioItem25);
        radiogruppe.add(radioItem26);
        radiogruppe.add(radioItem27);

        menue2.add(radioItem25);
        menue2.add(radioItem26);
        menue2.add(radioItem27);

        // Menüpunkte in Menü3 einfügen

        menue3.add(menueItem31);
        menue3.add(menueItem32);

        // Menüs in Menüzeile einfügen

        menuezeile.add(menue1);
        menuezeile.add(menue2);
        menuezeile.add(menue3);
    }

    // -------------------------------------------------------------------------

    /**
    Schaltfläche initialisieren.
    */
    private void initButton()
    {
        button.setFont(new Font("Monospaced", Font.BOLD, 32));
        button.setForeground(Color.WHITE);
        button.setBackground(Color.MAGENTA);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Elemente bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert. <br />
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private 
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese 
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis 
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      neuesAuto(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });
        menueItem11.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      neuesAuto(evt);
                  }
              });
        menueItem12.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem12event(evt);
                  }
              });
        menueItem13.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem13event(evt);
                  }
              });
        menueItem14.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem14event(evt);
                  }
              });
        menueItem15.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem15event(evt);
                  }
              });
        menueItem21.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem21event(evt);
                  }
              });
        checkItem22.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem22event(evt);
                  }
              });

        // nicht angemeldet: checkItem23

        // nicht angemeldet: menue24

        radioItem25.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem25event(evt);
                  }
              });
        radioItem26.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem26event(evt);
                  }
              });

        // nicht angemeldet: radioItem27

        menueItem31.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem31event(evt);
                  }
              });
        menueItem32.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      mItem32event(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Auto-Instanz erzeugen (über Menüauswahl oder über Schaltfläche).

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void neuesAuto(ActionEvent evt)
    {
        auto = new AutoM();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem12event(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag3.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem13event(ActionEvent evt)
    {
        auto.runterschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag4.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem14event(ActionEvent evt)
    {
        auto.beschleunigen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü1 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem15event(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem21event(ActionEvent evt)
    {
        auto.ausgeben("1. Menüeintrag (Menü2) ausgewählt");
        checkM23();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem22event(ActionEvent evt)
    {
        auto.ausgeben("2. Menüeintrag (Menü2) ausgewählt");
        checkM23();
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag5.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem25event(ActionEvent evt)
    {
        auto.ausgeben("5. Menüeintrag (Menü2) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü2 - Menüeintrag6.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem26event(ActionEvent evt)
    {
        auto.ausgeben("6. Menüeintrag (Menü2) ausgewählt");
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem31event(ActionEvent evt)
    {
        JOptionPane.showMessageDialog(this,
                                      "Achtung"
                                          + System.getProperty("line.separator")
                                        + "Hilfethemen noch nicht installiert!",
                                      "Info für MenueV",
                                      JOptionPane.WARNING_MESSAGE);
    }

    // *************************************************************************

    /**
    Methode für Menü3 - Menüeintrag2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void mItem32event(ActionEvent evt)
    {
        JOptionPane.showMessageDialog(this,
                                      "Copyright 2001"
                                          + System.getProperty("line.separator")
                                          + "Beringer Alfred",
                                      "Info für MenueV",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    // -------------------------------------------------------------------------
    // Hilfsmethoden zu den Ereignisbehandlungsmethoden
    // -------------------------------------------------------------------------

    /**
    Prüfen der Option Menü2-Menüeintrag3.
    */
    private void checkM23()
    {
        if (checkItem23.getState())
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü2) hat Hakerl");
        }
        else
        {
            auto.ausgeben("     3. CheckMenüeintrag (Menü2) hat kein Hakerl");
        }
    }
}
