/*
 * MusterfensterV.java
 */
package at.beringer.oopBuch.guiSwing;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

/**
Hauptfenster mit einfachen Standard-GUI-Komponenten.
<p>
Anonyme Klassen: von 'ActionListener' <br />
                 und von 'MouseAdapter' <br />
Model-Klasse:    'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Im Paket 'at.beringer.oopBuch.guiSwingI18N' befindet sich eine vollständig 
internationalisierte - und parametrisierte (Fenstergröße) - Version dieser 
Fensterklasse.
</p>

<hr />

@since Jänner 2002
@version Juni 2006 (Refaktorierung)
@version <br />Juni 2012 (ohne ContentPane, ab JDK 6 möglich)

@see AutoM
@see ProgrammMusterfenster
@see at.beringer.oopBuch.guiSwingI18N.MusterfensterV
@see at.beringer.oopBuch.guiJavaFX.MusterfensterV

@author Beringer Alfred
*/
public class MusterfensterV extends JFrame
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    private static final String  NEUEZEILE = 
                                     System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private JLabel        label        = new JLabel("Label");
    private JButton       button1      = new JButton("Schaltfläche für Check");
    private JButton       button2      = new JButton("Schaltfläche für Liste");

    private JCheckBox     check1       = new JCheckBox("Check1", false);
    private JCheckBox     check2       = new JCheckBox("Check2", true);
    private JCheckBox     check3       = new JCheckBox("Check3", true);

    private String        randtext     = "Auto-Aktionen:";
    private ButtonGroup   buttongruppe = new ButtonGroup();
    private JRadioButton  radio1       = new JRadioButton("raufschalten", false);
    private JRadioButton  radio2       = new JRadioButton("bremsen     ", false);
    private JRadioButton  radio3       = new JRadioButton("check3 aus! ", true);

    private JLabel        labTxfeld    = new JLabel(" \"return\" ... beendet die Eingabe: ");
    private JTextField    textfeld     = new JTextField("Textfeld", 20);

    private JLabel        labTxbereich = new JLabel(" \"return\" ... nur Zeilenvorschub: ");
    private JTextArea     textbereich  = new JTextArea("Textbereich-Zeile1" + NEUEZEILE
                                                       + "Zeile2", 4, 30);
    private JScrollPane   textscroll   = new JScrollPane(textbereich);

    private JLabel        labListe1    = new JLabel("nur 1 einziges Element auswählbar: ");
    private DefaultListModel<String> liste1Model = new DefaultListModel<>();
    private JList<String> liste1       = new JList<>(liste1Model);
    private JScrollPane   liste1scroll = new JScrollPane(liste1);

    // Der Einfachheit halber fix codierte Daten für Listeninhalt
    private String[]      liste1Daten  = {
                                           "Liste1-Element1",
                                           "Liste1-Element2",
                                           "Liste1-Element3",
                                           "Liste1-Element4",
                                           "Liste1-Element5",
                                           "Liste1-Element6",
                                           "Liste1-Element7",
                                           "Liste1-Element8"
                                          };

    private JLabel        labListe2    = new JLabel("mehrere Elemente auswählbar: ");
    private DefaultListModel<String> liste2Model = new DefaultListModel<>();
    private JList<String> liste2       = new JList<>(liste2Model);

    // Der Einfachheit halber fix codierte Daten für Listeninhalt
    private String[]      liste2Daten  = {
                                           "Liste2-Element1",
                                           "Liste2-Element2",
                                           "Liste2-Element3",
                                           "Liste2-Element4"
                                         };

    private JLabel        labCombo     = new JLabel("Das ist eine Auswahl-Komponente: ");

    // Der Einfachheit halber fix initialisierte Daten für Listeninhalt
    private JComboBox<String> combo    = new JComboBox<>(new String[]  {
                                                              "Combo-Element1",
                                                              "Combo-Element2",
                                                              "Combo-Element3"
                                                                       });

    /** Aktionszähler                                                         */
    private int  zaehler = 0;

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Das Erzeugen der einzelnen Teilfenster (Panels) und das Anmelden bei den
    Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    public MusterfensterV()
    {
        this.setTitle("Musterfenster mit vordefinierten Swing-Komponenten");
        this.setSize(550, 700);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        initKomponenten();
        initFenster();
        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Eigenschaften der GUI-Komponenten definieren.
    */
    private void initKomponenten()
    {
        // Optionsfelder
        buttongruppe.add(radio1);
        buttongruppe.add(radio2);
        buttongruppe.add(radio3);

        // Textfeld (einzeilig)
        textfeld.setEditable(true);

        // Textbereich (mehrzeilig)
        textbereich.setEditable(true);

        // Listen

        liste1.setVisibleRowCount(3);
        liste1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // Liste1 füllen (Zeilen einzeln hinzufügen)
        for (String zeile : liste1Daten)
        {
            liste1Model.addElement(zeile);
        }

        // Liste2 füllen (Zeilen einzeln hinzufügen)
        for (String zeile : liste2Daten)
        {
            liste2Model.addElement(zeile);
        }
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        JPanel  poben1 = new JPanel(new FlowLayout());
        JPanel  poben2 = new JPanel(new FlowLayout());
        JPanel  poben3 = new JPanel(new FlowLayout());
        JPanel  poben  = new JPanel(new GridLayout(0,1));            // 1 Spalte
                                       // Anzahl 0 bedeutet: "so viel wie nötig"

        JPanel  pmitte4 = new JPanel(new FlowLayout());
        JPanel  pmitte5 = new JPanel(new FlowLayout());
        JPanel  pmitte6 = new JPanel(new FlowLayout());
        JPanel  pmitte7 = new JPanel(new FlowLayout());
        JPanel  pmitte8 = new JPanel(new FlowLayout());
        JPanel  pmitte  = new JPanel(new GridLayout(0,1));           // 1 Spalte
                                       // Anzahl 0 bedeutet: "so viel wie nötig"

        // Initialisieren 'poben1' für Schaltflächen
        poben1.add(label);
        poben1.add(button1);
        poben1.add(button2);

        // Initialisieren 'poben2' für Kontrollkästchen
        poben2.add(check1);
        poben2.add(check2);
        poben2.add(check3);

        // Initialisieren 'poben3' für Optionsfelder
        initPoben3(poben3);

        // Initialisieren 'poben'
        poben.add(poben1);
        poben.add(poben2);
        poben.add(poben3);

        // Initialisieren Panels für 'pmitte'
        initPanelMitBeschriftung(pmitte4, labTxfeld,    textfeld);
        initPanelMitBeschriftung(pmitte5, labTxbereich, textscroll);
        initPanelMitBeschriftung(pmitte6, labListe1,    liste1scroll);
        initPanelMitBeschriftung(pmitte7, labListe2,    liste2);
        initPanelMitBeschriftung(pmitte8, labCombo,     combo);

        // Initialisieren 'pmitte'
        pmitte.add(pmitte4);
        pmitte.add(pmitte5);
        pmitte.add(pmitte6);
        pmitte.add(pmitte7);
        pmitte.add(pmitte8);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new BorderLayout());

        this.add(poben,BorderLayout.NORTH);
        this.add(pmitte,BorderLayout.CENTER);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren 'poben3' für Optionsfelder.
    <p>
    Gezeigt wird beispielhaft die Verwendung eines Randes.
    </p>

    @param  p  'poben3' für Optionsfelder
    */
    private void initPoben3(JPanel p)
    {
        Border  rand      = BorderFactory.createLoweredBevelBorder();
        Border  randtitel = BorderFactory.createTitledBorder(rand, randtext);

        p.setBorder(randtitel);
        p.add(radio1);
        p.add(radio2);
        p.add(radio3);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren Panelzeile für eine GUI-Komponente mit Beschriftung. <br />
    Die Beschriftung steht links - die Komponente rechts daneben.

    @param  p             Panel für GUI-Komponente mit Beschriftung
    @param  beschriftung  Beschriftung (Label)
    @param  komponente    GUI-Komponente
    */
    private void initPanelMitBeschriftung(JPanel      p,
                                          JLabel      beschriftung,
                                          JComponent  komponente)
    {
        JPanel  plinks  = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JPanel  prechts = new JPanel(new FlowLayout(FlowLayout.LEFT));

        plinks.add(beschriftung);
        prechts.add(komponente);
        p.setLayout(new GridLayout(1,2));
        p.add(plinks);
        p.add(prechts);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button1event(evt);
                  }
              });    // button1 ist zuständig für Kontrollkästchen
        button2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button2event(evt);
                  }
              });    // button2 ist zuständig für Listen
        check1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      check1event(evt);
                  }
              });     // nicht notwendig, zuständig ist button1!
    
        // check2 und check3 nicht angemeldet (zuständig ist button1)
    
        radio1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio1event(evt);
                  }
              });
        radio2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio2event(evt);
                  }
              });
        radio3.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio3event(evt);
                  }
              });
        textfeld.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      textfeldevent(evt);
                  }
              });
        liste1.addMouseListener(new MouseAdapter()
              {
                  @Override
                  public void mouseClicked(MouseEvent evt)
                  {
                      liste1event(evt);
                  }
              });
        combo.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      comboevent(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Methode für Schaltfläche1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Schaltfläche1 gedrückt");

        if (check1.isSelected())
        {
            auto.ausgeben("     check1 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check1 leer");
        }

        if (check2.isSelected())
        {
            auto.ausgeben("     check2 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check2 leer");
        }

        if (check3.isSelected())
        {
            auto.ausgeben("     check3 hat Hakerl");
        }
        else
        {
            auto.ausgeben("     check3 leer");
        }
    }

    // *************************************************************************

    /**
    Methode für Schaltfläche2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button2event(ActionEvent evt)
    {
        java.util.List<String>  inhalte = liste2.getSelectedValuesList();
                                          // seit Java 7.0
        int[]                   indices = liste2.getSelectedIndices();

        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");

        auto.ausgeben("Schaltfläche2 gedrückt, ausgewählte Zeile von Liste1: ");
        auto.ausgeben("            " +  liste1.getSelectedValue()
                              + " (" + (liste1.getSelectedIndex()+1) 
                              + ". Zeile)");

        auto.ausgeben("                          ausgewählte Zeilen von Liste2: ");
        for (int i = 0; i < inhalte.size(); i++)
        {
            auto.ausgeben("            " + inhalte.get(i)
                                  + " (" + (indices[i]+1) 
                                  + ". Zeile)");
        }
    }

    // *************************************************************************

    /**
    Methode für Kontrollkästchen1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void check1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("check1 gedrückt");
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("radio1 gedrückt (raufschalten)");
        auto.raufschalten();
        auto.zeigeInhalt();
        textbereich.append(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio2event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("radio2 gedrückt (bremsen)");
        auto.bremsen();
        auto.zeigeInhalt();
        textbereich.append(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld3.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio3event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!"
                           + NEUEZEILE + auto);
        auto.ausgeben("radio3 gedrückt, check3 wird leer");
        check3.setSelected(false);
    }

    // *************************************************************************

    /**
    Methode für Textfeld.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void textfeldevent(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Textfeld geändert: " + textfeld.getText());
    }

    // *************************************************************************

    /**
    Methode für Listenfeld1.

    @param  evt  MouseEvent (von API-Methode 'mouseClicked' übernommen)
    */
    private void liste1event(MouseEvent evt)
    {
        auto.ausgeben("Mauswächter: Liste1, ausgewählte Zeile von Liste1: ");
        auto.ausgeben("            " +  liste1.getSelectedValue()
                              + " (" + (liste1.getSelectedIndex() + 1) 
                              + ". Zeile)");
        auto.ausgeben("    Mausklick in "
                      + (liste1.locationToIndex(evt.getPoint()) + 1) 
                      + ". Zeile");
    }

    // *************************************************************************

    /**
    Methode für Auswahlfeld.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void comboevent(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + ". Action-Ereignis!");
        auto.ausgeben("Auswahl Combo: " +  combo.getSelectedItem()
                                 + " (" + (combo.getSelectedIndex() + 1) 
                                 + ". Zeile)");
    }
}
