/*
 * DialogModalElternfensterV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

import static at.beringer.oopBuch.guiSwingI18N.DialogModalKonstante.*;

/**
Hauptfenster für einen modalen Dialog. <br />
Schaltfläche öffnet Dialogfenster, Textbereich speichert Dialogeingabe.
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Vollständig internationalisierte und parametrisierte Klasse.
</p>

<hr />

@since Juni 2012

@see DialogModalElternfensterV
@see DialogModalKonstante
@see DialogModalKindfensterV
@see AutoM
@see ProgrammDialogModal

@author Beringer Alfred
*/
public class DialogModalElternfensterV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
    
    private JLabel       label1     = new JLabel(LABEL1TEXT);
    private JButton      button     = new JButton(BUTTON);
    private JLabel       label2     = new JLabel(LABEL2TEXT);
    private JTextArea    text       = new JTextArea("",4,20);
    private JScrollPane  textscroll = new JScrollPane(text);

    private DialogModalKindfensterV  dlg;

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public DialogModalElternfensterV()
    {
        this.setTitle(FENSTERTITEL_E);

        // Methode 'pack' (statt 'setSize') erst nach Fertigstellung des 
        // Fensters durchführen!
        
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        this.setLayout(new GridLayout(2,1));

        // Eigenschaften der GUI-Komponenten definieren

        text.setEditable(false);

        // GUI-Komponenten mit Hilfe von Layout-Managern anordnen 

        JPanel p1  = new JPanel();
        JPanel p11 = new JPanel();
        JPanel p12 = new JPanel();
        JPanel p2  = new JPanel();
        JPanel p21 = new JPanel();
        JPanel p22 = new JPanel();

        p11.setLayout(new FlowLayout(FlowLayout.RIGHT));
        p11.add(label1);
        p12.setLayout(new FlowLayout(FlowLayout.LEFT));
        p12.add(button);

        p1.setLayout(new GridLayout(1,2));
        p1.add(p11);
        p1.add(p12);

        p21.setLayout(new FlowLayout(FlowLayout.RIGHT));
        p21.add(label2);
        p22.setLayout(new FlowLayout(FlowLayout.LEFT));
        p22.add(textscroll);

        p2.setLayout(new GridLayout(1,2));
        p2.add(p21);
        p2.add(p22);

        // GUI-Komponenten dem Fenster hinzufügen 

        this.add(p1);
        this.add(p2);

        this.pack();

        // GUI-Komponenten bei Ereigniswächtern anmelden

        button.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      buttonevent(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });

        initDialogfenster();
    }

    /**
    Dialogfenster erzeugen - eigene Referenz mitgeben, falls Daten vom 
                             Elternfenster benötigt werden <br />
    Fenster als Dialog darstellen, d.h. 
    Fenstergröße kann nicht verändert werden, nur Schließen möglich
    */
    private void initDialogfenster()
    {
        dlg = new DialogModalKindfensterV(this);
        dlg.setModal(true);                           // Dialog modal definieren
        dlg.setResizable(false);     // Fenstergröße kann nicht verändert werden
        dlg.setLocation(LOCATIONx, LOCATIONy);       // Dialogfenster platzieren
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
  
    /**
    Methode für Schaltfläche 'button'.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void buttonevent(ActionEvent evt)
    {
        dlg.setVisible(true);                          // Dialogfenster anzeigen

        /*
        Applikation stoppt hier (Dialog ist modal!)
        */
          
        if (dlg.getEnde().equals(DLG_BUTTON1))
        {
            auto.setFarbe(dlg.getRueckgabe());
            auto.zeigeInhalt();                                // nur zum Testen

            text.append(dlg.getRueckgabe());
            text.append(NEUEZEILE);
        }
    }
}
