/*
 * DialogModalKindfensterV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

import static at.beringer.oopBuch.guiSwingI18N.DialogModalKonstante.*;

/**
Modales Dialogfenster. <br />
Textfeld für Eingabe mit 2 Buttons ('OK' und 'Abbruch').
<p>
Anonyme Klassen: von 'ActionListener' <br />
                 und von 'WindowAdapter' <br />
Model-Klasse:    'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Vollständig internationalisierte und parametrisierte Klasse.
</p>

<hr />

@since Juni 2012

@see DialogModalKindfensterV
@see DialogModalKonstante
@see DialogModalElternfensterV
@see AutoM

@author Beringer Alfred
*/
public class DialogModalKindfensterV extends JDialog
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private JLabel      label   = new JLabel(LABELTEXT);
    private JTextField  text    = new JTextField(20);
    private JButton     button1 = new JButton(DLG_BUTTON1);
    private JButton     button2 = new JButton(DLG_BUTTON2);

    private JFrame      elternfenster;

    /** Inhalt des Textfelds, der ans Elternfenster übergeben werden soll.    */
    private String      rueckgabe;

    /** gedrückter Button ('OK' oder 'Abbruch' oder 'X').                     */
    private String      ende;

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM        auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Referenz auf Elternfenster wird übernommen, falls Daten vom Elternfenster 
    benötigt werden.

    @param   elternfenster   Elternfenster 
    */
    public DialogModalKindfensterV(JFrame elternfenster)
    {
        this.elternfenster = elternfenster;

        this.setTitle(FENSTERTITEL_K);
        this.setSize(FENSTERBREITE, FENSTERHOEHE);

        this.setLayout(null);

        // Eigenschaften der GUI-Komponenten definieren

        text.setEditable(true);

        // GUI-Komponenten im Fenster platzieren 

        label.setBounds(   10, 20, label.getPreferredSize().width,
                                   label.getPreferredSize().height);
        text.setBounds(   230, 20, text.getPreferredSize().width,
                                   text.getPreferredSize().height);

        button1.setBounds(100, 70, button1.getPreferredSize().width,
                                   button1.getPreferredSize().height);
        button2.setBounds(280, 70, button2.getPreferredSize().width,
                                   button2.getPreferredSize().height);

        // GUI-Komponenten dem Fenster hinzufügen 

        this.add(label);
        this.add(text);
        this.add(button1);
        this.add(button2);

        // GUI-Komponenten bei Ereigniswächtern anmelden

        this.addWindowListener(new WindowAdapter()
              {
                  @Override
                  public void windowClosing(WindowEvent evt)
                  {
                      fensterschliessen(evt);
                  }
              });

        button1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button1event(evt);
                  }
              });

        button2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button2event(evt);
                  }
              });
    }

  // ---------------------------------------------------------------------------
  // Zugriffsmethoden (Get- und Set-Methoden)
  // ---------------------------------------------------------------------------

    /**
    Liefert den Inhalt des Textfelds, der ans Elternfenster übergeben werden 
    soll.

    @return  Inhalt Textfeld
    */
    public String getRueckgabe()
    {
        return  rueckgabe;
    }

    /**
    Liefert den gedrückten Button ('OK' oder 'Abbruch' oder 'X').

    @return  gedrückter Button
    */
    public String getEnde()
    {
        return  ende;
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
  
    /**
    Methode für Schaltfläche 'OK'.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button1event(ActionEvent evt)
    {
        ende = DLG_BUTTON1;
        rueckgabe = text.getText();
        auto.ausgeben(ende + " " + MELDETEXT + " *" + rueckgabe + '*');
        fensterende();
    }
 
    /**
    Methode für Schaltfläche 'Abbruch'.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button2event(ActionEvent evt)
    {
        ende = DLG_BUTTON2;
        rueckgabe = "";
        auto.ausgeben(ende + " " + MELDETEXT);
        fensterende();
    }
   
    /**
    Methode für 'Fenster schließen'.

    @param  evt  WindowEvent (von API-Methode 'windowClosing' übernommen)
    */
    private void fensterschliessen(WindowEvent evt)
    {
        ende = "X";
        rueckgabe = "";
        auto.ausgeben(ende + " " + MELDETEXT);
        fensterende();
    }

    /**
    Dialogfenster beenden.
    */
    private void fensterende()
    {
        text.setText("");
        this.setVisible(false);
        this.dispose();
    }
}
