/*
 * DialogStandardV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;

import static at.beringer.oopBuch.guiSwingI18N.DialogModalKonstante.*;

/**
Hauptfenster für Standarddialog (JOptionPane). <br />
Entspricht der Klasse 'DialogModalElternFensterV',
die Klasse 'DialogModalKindFensterV' wird durch das Standarddialogfenster 
'JOptionPane.showInputDialog(...)' ersetzt.
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>

<hr />

@since Jänner 2002
@version Juni 2012 (ohne ContentPane, ab JDK 6 möglich)

@see DialogModalKonstante
@see ProgrammDialogStandard
@see DialogModalElternfensterV
@see DialogModalKindfensterV
@see AutoM

@author Beringer Alfred
*/
public class DialogStandardV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private JLabel       label1     = new JLabel(LABEL1TEXT);
    private JButton      button     = new JButton(BUTTON);
    private JLabel       label2     = new JLabel(LABEL2TEXT);
    private JTextArea    text       = new JTextArea("",4,20);
    private JScrollPane  textscroll = new JScrollPane(text);

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public DialogStandardV()
    {
        this.setTitle(FENSTERTITEL_E);

        // Methode 'pack' (statt 'setSize') erst nach Fertigstellung des 
        // Fensters durchführen!

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        this.setLayout(new GridLayout(2,1));

        // Eigenschaften der GUI-Komponenten definieren

        text.setEditable(false);

        // GUI-Komponenten mit Hilfe von Layout-Managern anordnen 

        JPanel  p1  = new JPanel();
        JPanel  p11 = new JPanel();
        JPanel  p12 = new JPanel();
        JPanel  p2  = new JPanel();
        JPanel  p21 = new JPanel();
        JPanel  p22 = new JPanel();

        p11.setLayout(new FlowLayout(FlowLayout.RIGHT));
        p11.add(label1);
        p12.setLayout(new FlowLayout(FlowLayout.LEFT));
        p12.add(button);

        p1.setLayout(new GridLayout(1,2));
        p1.add(p11);
        p1.add(p12);

        p21.setLayout(new FlowLayout(FlowLayout.RIGHT));
        p21.add(label2);
        p22.setLayout(new FlowLayout(FlowLayout.LEFT));
        p22.add(textscroll);

        p2.setLayout(new GridLayout(1,2));
        p2.add(p21);
        p2.add(p22);

        // GUI-Komponenten dem Fenster hinzufügen 

        this.add(p1);
        this.add(p2);

        this.pack();

        // GUI-Komponenten bei Ereigniswächtern anmelden

        button.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      buttonevent(evt);
                      // Event mitgeben, falls Daten vom Event benötigt werden
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
  
    /**
    Methode für Schaltfläche 'button'.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void buttonevent(ActionEvent evt)
    {
        String  rueckgabe = JOptionPane.showInputDialog(
                                                 null,
                                                 LABELTEXT,
                                                 FENSTERTITEL_K,
                                                 JOptionPane.PLAIN_MESSAGE);
                                                       // Dialogfenster anzeigen

        /*
        Applikation stoppt hier (Standarddialog ist modal!)
        */
      
        if (rueckgabe != null)
        {
            auto.setFarbe(rueckgabe);
            auto.zeigeInhalt();

            text.append(rueckgabe);
            text.append(NEUEZEILE);
        }
    }
}
