/*
 * LabelButtonFensterV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.Auto;
import at.beringer.oopBuch.einfacheBsp.Person;

import static at.beringer.oopBuch.guiSwingI18N.LabelButtonFensterKonstante.*;

/**
Hauptfenster mit Text ('Label') und 2 Schaltfächen.
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:  'Auto' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Diese Klasse ist die internationalisierte und parametrisierte Version der 
entsprechenden Klasse im Paket 'at.beringer.oopBuch.guiSwing'. <br />
Als frei wählbarer Parameter wurde die Fenstergröße gewählt.
</p>
<p>
Alle Parameter - sowohl internationalisierte (wie Texte und Beschriftungen) wie
auch nichtlokalisierte (wie die Fenstergröße) - wurden in die statische 
Klasse 'LabelButtonFensterKonstante' ausgelagert.
</p>
<p>
Die lokalisierten Ressource-Dateien befinden sich im Paket 
'at.beringer.oopBuch.resources'. <br />
Die lokalisierte Basisdatei heißt 'labelButtonFenster.properties'. <br />
Die Properties-Datei für die (nichtlokalisierten) Parameter befindet sich im 
Verzeichnis 'at/beringer/oopBuch/properties' und heißt ebenfalls 
'labelButtonFenster.properties'.
</p>

<hr />

@since Juli 2012

@see LabelButtonFensterKonstante
@see Auto
@see Person
@see ProgrammLabelButtonFenster
@see at.beringer.oopBuch.guiSwing.LabelButtonFensterV

@author Beringer Alfred
*/
public class LabelButtonFensterV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private JButton  schaltflaeche1 = new JButton(SCHALTFLAECHE1);
    private JButton  schaltflaeche2 = new JButton(SCHALTFLAECHE2);

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private Auto  auto;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public LabelButtonFensterV()
    {
        this.setTitle(FENSTERTITEL);
        this.setSize(FENSTERBREITE, FENSTERHOEHE);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // ToolTip (Quick-Help)
        schaltflaeche1.setToolTipText(TOOLTIP1);
        schaltflaeche2.setToolTipText(TOOLTIP2);

        // Tasten-Hotkey, wird unterstrichen angezeigt, 
        //    funktioniert nur mit ALT-Taste.
        schaltflaeche1.setMnemonic(MNEMONIC1);
        schaltflaeche2.setMnemonic(MNEMONIC2);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new FlowLayout());

        this.add(new JLabel(LABELTEXT));
        this.add(schaltflaeche1);
        this.add(schaltflaeche2);

        initListener();

        initFachobjekt();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden 
    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        schaltflaeche1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      schaltflaeche1event(evt);
                  }
              });
        schaltflaeche2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      schaltflaeche2event(evt);
                  }
              });
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren des Fachobjekts.
    <p>
    Zum Testen wurden der Einfachheit halber willkürliche Werte in der 
    Properties-Datei definiert.
    </p>
    */
    private void initFachobjekt()
    {
        char  geschlecht = BES_GESCHLECHT;

        /*
        Bem.:
        Die Klasse 'Person' ist nicht lokalisiert, daher sind die möglichen 
        Kennzeichen für das Geschlecht immer fix vorgegeben.
        Das korrekte Kennzeichen muss also DIREKT in der Properties-Datei 
        angegeben werden!
        Es ist aber möglich, auch dieses Kennzeichen zu internationalisieren.
        */
        if ((geschlecht != Person.MAENNLICH) && (geschlecht != Person.WEIBLICH))
        {
            geschlecht = Person.MAENNLICH;
        }

        auto = new Auto();
        auto.setBesitzer(new Person(BES_GEBURTSJAHR, geschlecht));
        auto.getBesitzer().aendernVorname(BES_VORNAME);
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Methode für Schaltfläche1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void schaltflaeche1event(ActionEvent evt)
    {
        auto.raufschalten();
        auto.zeigeInhalt();
    }

    // *************************************************************************

    /**
    Methode für Schaltfläche2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void schaltflaeche2event(ActionEvent evt)
    {
        auto.bremsen();
        auto.zeigeInhalt();
    }
}
