/*
 * MausbewegungenKonstante.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import java.util.Locale;
import java.util.ResourceBundle;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;

/**
Statische Klasse für Ressource- und Properties-Konstante.
<p>
Texte und Beschriftungen werden aus einer (lokalisierten) Ressource-Datei 
(d.h. einer lokalisierten Properties-Datei) gelesen. <br />
Die Dateinamenserweiterung von Ressource-Dateien muss 'properties' sein.
</p>
<p>
Bem.: Klassen-Dateien und Bild-Dateien sind ebenfalls Ressourcen.
</p>
<p>
Ressource-Dateien müssen INNERHALB des Klassenpfads liegen.
</p>
<p>
Verzeichnis für die Ressource-Dateien: <br />
<pre>
     <strong>at.beringer.oopBuch.resources</strong>
</pre>
</p>
<p>
Basisname der Ressource-Dateien: <br />
<pre>
     <strong>mausbewegungen</strong>
</pre>
</p>
<p>
--> vollständiger qualifizierter Name der Ressource-Dateien: <br />
<pre>
     at.beringer.oopBuch.resources.mausbewegungen.properties
     at.beringer.oopBuch.resources.mausbewegungen_de_AT.properties
     at.beringer.oopBuch.resources.mausbewegungen_en.properties
</pre> <br />
usw.
</p>
<p>
Die JVM sucht - analog zur Suche einer Klasse - automatisch nach der 
entsprechenden (lokalisierten) Ressource-Datei. <br />
Die JVM behandelt eine Ressource-Datei wie eine Klasse, u.a. wird eine 
Resource-Datei automatisch geladen. Wenn die Datei nicht existiert, wird eine 
'MissingResourceException' ausgelöst.
</p>
<p>
Der Benutzer kann (und darf) Ressource-Dateien <strong>nicht</strong> ändern.

</p><hr /><p>

Nicht-lokalisierte Konstante können in Properties-Dateien definiert werden. 
Im Allg. ist die Dateinamenserweiterung von Properties-Dateien 'properties', 
allerdings ist - im Gegensatz zu Ressource-Dateien - jede beliebige 
Dateinamenserweiterung erlaubt.
</p>
<p>
Properties-Dateien müssen AUSSERHALB des Klassenpfads liegen.
</p>
<p>
Name der Properties-Datei für nicht-lokalisierte Konstante: <br />
<pre>
     <strong>mausbewegungen.properties</strong>
</pre>
</p>
<p>
Diese Properties-Datei muss enthalten sein im Unterverzeichnis<br />
<pre>
     <strong>at/beringer/oopBuch/properties</strong>
</pre> <br />
des Benutzerverzeichnisses (d.h. des Projektverzeichnis).
</p>
<p>
--> vollständiger qualifizierter Name der Properties-Datei: <br />
<pre>
     <strong>at/beringer/oopBuch/properties/mausbewegungen.properties</strong>
</pre>
</p>

<dl>
<dt>Bem.:
  <dd>Ressource- und Properties-Dateien befinden sich immer in 
      unterschiedlichen Verzeichnissen (innerhalb bzw. außerhalb des 
      Klassenpfads). Daher können die Dateinamen von Ressource- und Properties-
      Dateien gleich sein (müssen aber nicht).
</dl>

<p>
Der Benutzer kann (und darf) Properties-Dateien ändern.
</p>

<hr />

@since Juni 2012

@see MausbewegungenV

@author Beringer Alfred
*/
public final class MausbewegungenKonstante
{
    // -------------------------------------------------------------------------
    // Private Klassenkonstante
    // -------------------------------------------------------------------------

    // -------------------------------------------------------------------------
    // Ressource-Datei

    /**
    Basisname der Ressource-Dateien für lokalisierte Konstante
    (Texte und Beschriftungen)
    */
    private static final String  RESS_DATEINAME = 
        "at.beringer.oopBuch.resources.mausbewegungen";

    /**
    Ressource-Datei für lokalisierte Konstante (Texte und Beschriftungen).
    <p>
    Die JVM sucht - analog zur Suche einer Klasse - automatisch nach der 
    entsprechenden Ressource-Datei.
    </p>
    */
    private static final ResourceBundle  RESS;

    // -------------------------------------------------------------------------
    // Projekt-Verzeichnis

    /** Name des Projekt-Verzeichnisses                                       */
    private static final String  PROJEKTVERZEICHNISNAME = 
        System.getProperty("user.dir") + System.getProperty("file.separator")
                        + "at"         + System.getProperty("file.separator")
                        + "beringer"   + System.getProperty("file.separator")
                        + "oopBuch"    + System.getProperty("file.separator");

    // -------------------------------------------------------------------------
    // Properties-Datei

    /** Name der Properties-Datei (für nichtlokalisierte Konstante)           */
    private static final String  PROPS_DATEINAME = 
        PROJEKTVERZEICHNISNAME
                        + "properties" + System.getProperty("file.separator")
                        + "mausbewegungen.properties";

    /** Properties-Datei für nichtlokalisierte Konstante                      */
    private static final Properties  PROPS;

    // -------------------------------------------------------------------------
    // Öffentliche Klassenkonstante
    // -------------------------------------------------------------------------

    // -------------------------------------------------------------------------
    // System Konstante:

    public static final String NEUEZEILE = System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Lokalisierte Konstante:

    // Texte und Beschriftungen für GUI-Komponenten:

    public static final String  FENSTERTITEL;

    public static final String  AUSGABE;

    // -------------------------------------------------------------------------
    // Nichtlokalisierte Konstante:

    // Fenstergröße (Hauptfenster)

    public static final int      FENSTERBREITE;
    private static final String  FENSTERBREITEstr;

    public static final int      FENSTERHOEHE;
    private static final String  FENSTERHOEHEstr;

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------
  
    /**
    Klassenkonstruktor. <br />
    Initialisieren der Klassenkonstanten.
    <p>
    Die Anfangswerte für die Klassenkonstanten werden von einer Ressource- und 
    einer Properties-Datei übernommen.
    </p>
    */
    static
    {
        // ---------------------------------------------------------------------
        // Ressources

        RESS = ResourceBundle.getBundle(RESS_DATEINAME, Locale.getDefault());

        FENSTERTITEL = RESS.getString("FENSTERTITEL");

        AUSGABE      = RESS.getString("AUSGABE");

        // ---------------------------------------------------------------------
        // Properties

        PROPS = new Properties();
        try
        {
            PROPS.load(new FileInputStream(PROPS_DATEINAME));
        }
        catch (IOException exc)
        {
            System.err.println("Properties-Datei nicht gefunden!!!!!!!!");
            
            // Default properties:
            PROPS.setProperty("FENSTERBREITE", "480");
            PROPS.setProperty("FENSTERHOEHE",  "260");
        }

        FENSTERBREITEstr = PROPS.getProperty("FENSTERBREITE");
        FENSTERBREITE    = Integer.parseInt(FENSTERBREITEstr);

        FENSTERHOEHEstr = PROPS.getProperty("FENSTERHOEHE");
        FENSTERHOEHE    = Integer.parseInt(FENSTERHOEHEstr);
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /** Privater Standardkonstruktor                                          */
    private MausbewegungenKonstante()
    {
    }
}