/*
 * MausbewegungenV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import static at.beringer.oopBuch.guiSwingI18N.MausbewegungenKonstante.*;

/**
Hauptfenster für Mausbewegungen. <br />
Die Mauskoordinaten werden in einem Textbereich gespeichert. <br />
<p>
Anonyme Klasse: von 'MouseMotionAdapter'
</p>
<p>
Vollständig internationalisierte und parametrisierte Klasse.
</p>

<hr />

@since Juni 2012

@see MausbewegungenKonstante
@see ProgrammMausbewegungen
@see at.beringer.oopBuch.guiSwing.MausbewegungenV

@author Beringer Alfred
*/
public class MausbewegungenV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private JTextArea    ausgabe       = new JTextArea(AUSGABE, 8, 20);
    private JScrollPane  ausgabeScroll = new JScrollPane(ausgabe);

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor
    */
    public MausbewegungenV()
    {
        this.setTitle(FENSTERTITEL);
        this.setSize(FENSTERBREITE, FENSTERHOEHE);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        this.setLayout(new FlowLayout());

        // Textbereich
        ausgabe.setEditable(false);

        // GUI-Komponenten dem Fenster hinzufügen 

        this.add(ausgabeScroll);

        // GUI-Komponenten bei Ereigniswächtern anmelden

        this.addMouseMotionListener(new MouseMotionAdapter()
              // ***************************************************************
              // anonyme (innere) Klasse
              // ***************************************************************
              {
                  // -----------------------------------------------------------
                  // API-Methoden überschreiben
                  // -----------------------------------------------------------

                  @Override
                  public void mouseMoved(MouseEvent evt)
                  {
                      mausEreignis(evt);
                      // Ereignisobjekt mitgeben, 
                      //    falls Daten vom Ereignis benötigt werden
                  }
              }
              // ********************************** Ende anonyme (innere) Klasse
                                   );
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden (private Methoden des Hauptfensters)
    // *************************************************************************
    
    /**
    Methode für Mausbewegung.

    @param  evt  MouseEvent (von API-Methode 'mouseMoved' übernommen)
    */
    private void mausEreignis(MouseEvent evt)
    {
        int  x = evt.getX();
        int  y = evt.getY();

        ausgabe.append(NEUEZEILE + "x=" + x + "  y=" + y);
    }
}
