/*
 * MusterfensterKonstante.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import java.util.Locale;
import java.util.ResourceBundle;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;

/**
Statische Klasse für Ressource- und Properties-Konstante.
<p>
Texte und Beschriftungen werden aus einer (lokalisierten) Ressource-Datei 
(d.h. einer lokalisierten Properties-Datei) gelesen. <br />
Die Dateinamenserweiterung von Ressource-Dateien muss 'properties' sein.
</p>
<p>
Bem.: Klassen-Dateien und Bild-Dateien sind ebenfalls Ressourcen.
</p>
<p>
Ressource-Dateien müssen INNERHALB des Klassenpfads liegen.
</p>
<p>
Verzeichnis für die Ressource-Dateien: <br />
<pre>
     <strong>at.beringer.oopBuch.resources</strong>
</pre>
</p>
<p>
Basisname der Ressource-Dateien: <br />
<pre>
     <strong>musterfenster</strong>
</pre>
</p>
<p>
--> vollständiger qualifizierter Name der Ressource-Dateien: <br />
<pre>
     at.beringer.oopBuch.resources.musterfenster.properties
     at.beringer.oopBuch.resources.musterfenster_de_AT.properties
     at.beringer.oopBuch.resources.musterfenster_en.properties
</pre> <br />
usw.
</p>
<p>
Die JVM sucht - analog zur Suche einer Klasse - automatisch nach der 
entsprechenden (lokalisierten) Ressource-Datei. <br />
Die JVM behandelt eine Ressource-Datei wie eine Klasse, u.a. wird eine 
Resource-Datei automatisch geladen. Wenn die Datei nicht existiert, wird eine 
'MissingResourceException' ausgelöst.
</p>
<p>
Der Benutzer kann (und darf) Ressource-Dateien <strong>nicht</strong> ändern.

</p><hr /><p>

Nicht-lokalisierte Konstante können in Properties-Dateien definiert werden. 
Im Allg. ist die Dateinamenserweiterung von Properties-Dateien 'properties', 
allerdings ist - im Gegensatz zu Ressource-Dateien - jede beliebige 
Dateinamenserweiterung erlaubt.
</p>
<p>
Properties-Dateien müssen AUSSERHALB des Klassenpfads liegen.
</p>
<p>
Name der Properties-Datei für nicht-lokalisierte Konstante: <br />
<pre>
     <strong>musterfenster.properties</strong>
</pre>
</p>
<p>
Diese Properties-Datei muss enthalten sein im Unterverzeichnis<br />
<pre>
     <strong>at/beringer/oopBuch/properties</strong>
</pre> <br />
des Benutzerverzeichnisses (d.h. des Projektverzeichnis).
</p>
<p>
--> vollständiger qualifizierter Name der Properties-Datei: <br />
<pre>
     <strong>at/beringer/oopBuch/properties/musterfenster.properties</strong>
</pre>
</p>

<dl>
<dt>Bem.:
  <dd>Ressource- und Properties-Dateien befinden sich immer in 
      unterschiedlichen Verzeichnissen (innerhalb bzw. außerhalb des 
      Klassenpfads). Daher können die Dateinamen von Ressource- und Properties-
      Dateien gleich sein (müssen aber nicht).
</dl>

<p>
Der Benutzer kann (und darf) Properties-Dateien ändern.
</p>

<hr />

@since Juni 2012

@see MusterfensterV

@author Beringer Alfred
*/
public final class MusterfensterKonstante
{
    // -------------------------------------------------------------------------
    // Private Klassenkonstante
    // -------------------------------------------------------------------------

    // -------------------------------------------------------------------------
    // Ressource-Datei

    /**
    Basisname der Ressource-Dateien für lokalisierte Konstante
    (Texte und Beschriftungen)
    */
    private static final String  RESS_DATEINAME = 
        "at.beringer.oopBuch.resources.musterfenster";

    /**
    Ressource-Datei für lokalisierte Konstante (Texte und Beschriftungen).
    <p>
    Die JVM sucht - analog zur Suche einer Klasse - automatisch nach der 
    entsprechenden Ressource-Datei.
    </p>
    */
    private static final ResourceBundle  RESS;

    // -------------------------------------------------------------------------
    // Projekt-Verzeichnis

    /** Name des Projekt-Verzeichnisses                                       */
    private static final String  PROJEKTVERZEICHNISNAME = 
        System.getProperty("user.dir") + System.getProperty("file.separator")
                        + "at"         + System.getProperty("file.separator")
                        + "beringer"   + System.getProperty("file.separator")
                        + "oopBuch"    + System.getProperty("file.separator");

    // -------------------------------------------------------------------------
    // Properties-Datei

    /** Name der Properties-Datei (für nichtlokalisierte Konstante)           */
    private static final String  PROPS_DATEINAME = 
        PROJEKTVERZEICHNISNAME
                        + "properties" + System.getProperty("file.separator")
                        + "musterfenster.properties";

    /** Properties-Datei für nichtlokalisierte Konstante                      */
    private static final Properties  PROPS;

    // -------------------------------------------------------------------------
    // Öffentliche Klassenkonstante
    // -------------------------------------------------------------------------

    // -------------------------------------------------------------------------
    // System Konstante:

    public static final String NEUEZEILE = System.getProperty("line.separator");

    // -------------------------------------------------------------------------
    // Lokalisierte Konstante:

    // Texte und Beschriftungen für GUI-Komponenten:

    public static final String  FENSTERTITEL;

    public static final String  LABEL;
    public static final String  BUTTON1;
    public static final String  BUTTON2;

    public static final String  CHECK1;
    public static final String  CHECK2;
    public static final String  CHECK3;

    public static final String  RANDTEXT;
    public static final String  RADIO1;
    public static final String  RADIO2;
    public static final String  RADIO3;

    public static final String  LABTXFELD;
    public static final String  TEXTFELD;

    public static final String  LABTXBEREICH;
    public static final String  TEXTBEREICH;

    public static final String  LABLISTE1;
    public static final String  LISTE1_1;
    public static final String  LISTE1_2;
    public static final String  LISTE1_3;
    public static final String  LISTE1_4;
    public static final String  LISTE1_5;
    public static final String  LISTE1_6;
    public static final String  LISTE1_7;
    public static final String  LISTE1_8;

    public static final String  LABLISTE2;
    public static final String  LISTE2_1;
    public static final String  LISTE2_2;
    public static final String  LISTE2_3;
    public static final String  LISTE2_4;

    public static final String  LABCOMBO;
    public static final String  COMBO_1;
    public static final String  COMBO_2;
    public static final String  COMBO_3;

    // Texte für die Meldungen (Ausgaben):

    public static final String  AUSGACTION;
    public static final String  AUSGZEILE;

    public static final String  AUSGBUTTON1;
    public static final String  AUSGBUTTON1_1;
    public static final String  AUSGBUTTON1_2;
    public static final String  AUSGBUTTON1_3;
    public static final String  AUSGBUTTON1_4;
    public static final String  AUSGBUTTON1_5;
    public static final String  AUSGBUTTON1_6;

    public static final String  AUSGBUTTON2_1;
    public static final String  AUSGBUTTON2_2;

    public static final String  AUSGCHECK1;

    public static final String  AUSGRADIO1;
    public static final String  AUSGRADIO2;
    public static final String  AUSGRADIO3;

    public static final String  AUSGTEXTFELD;

    public static final String  AUSGLISTE1_1;
    public static final String  AUSGLISTE1_2;

    public static final String  AUSGCOMBO;

    // -------------------------------------------------------------------------
    // Nichtlokalisierte Konstante:

    // Fenstergröße (Hauptfenster)

    public static final int      FENSTERBREITE;
    private static final String  FENSTERBREITEstr;

    public static final int      FENSTERHOEHE;
    private static final String  FENSTERHOEHEstr;

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------
  
    /**
    Klassenkonstruktor. <br />
    Initialisieren der Klassenkonstanten.
    <p>
    Die Anfangswerte für die Klassenkonstanten werden von einer Ressource- und 
    einer Properties-Datei übernommen.
    </p>
    */
    static
    {
        // ---------------------------------------------------------------------
        // Ressources

        RESS = ResourceBundle.getBundle(RESS_DATEINAME, Locale.getDefault());

        FENSTERTITEL = RESS.getString("FENSTERTITEL");

        LABEL     = RESS.getString("LABEL");
        BUTTON1   = RESS.getString("BUTTON1");
        BUTTON2   = RESS.getString("BUTTON2");

        CHECK1    = RESS.getString("CHECK1");
        CHECK2    = RESS.getString("CHECK2");
        CHECK3    = RESS.getString("CHECK3");

        RANDTEXT  = RESS.getString("RANDTEXT");
        RADIO1    = RESS.getString("RADIO1");
        RADIO2    = RESS.getString("RADIO2");
        RADIO3    = RESS.getString("RADIO3");

        LABTXFELD = RESS.getString("LABTXFELD");
        TEXTFELD  = RESS.getString("TEXTFELD");

        LABTXBEREICH = RESS.getString("LABTXBEREICH");
        TEXTBEREICH  = RESS.getString("TEXTBEREICH");

        LABLISTE1 = RESS.getString("LABLISTE1");
        LISTE1_1  = RESS.getString("LISTE1_1");
        LISTE1_2  = RESS.getString("LISTE1_2");
        LISTE1_3  = RESS.getString("LISTE1_3");
        LISTE1_4  = RESS.getString("LISTE1_4");
        LISTE1_5  = RESS.getString("LISTE1_5");
        LISTE1_6  = RESS.getString("LISTE1_6");
        LISTE1_7  = RESS.getString("LISTE1_7");
        LISTE1_8  = RESS.getString("LISTE1_8");

        LABLISTE2 = RESS.getString("LABLISTE2");
        LISTE2_1  = RESS.getString("LISTE2_1");
        LISTE2_2  = RESS.getString("LISTE2_2");
        LISTE2_3  = RESS.getString("LISTE2_3");
        LISTE2_4  = RESS.getString("LISTE2_4");

        LABCOMBO  = RESS.getString("LABCOMBO");
        COMBO_1   = RESS.getString("COMBO_1");
        COMBO_2   = RESS.getString("COMBO_2");
        COMBO_3   = RESS.getString("COMBO_3");

        AUSGACTION = RESS.getString("AUSGACTION");
        AUSGZEILE  = RESS.getString("AUSGZEILE");

        AUSGBUTTON1   = RESS.getString("AUSGBUTTON1");
        AUSGBUTTON1_1 = RESS.getString("AUSGBUTTON1_1");
        AUSGBUTTON1_2 = RESS.getString("AUSGBUTTON1_2");
        AUSGBUTTON1_3 = RESS.getString("AUSGBUTTON1_3");
        AUSGBUTTON1_4 = RESS.getString("AUSGBUTTON1_4");
        AUSGBUTTON1_5 = RESS.getString("AUSGBUTTON1_5");
        AUSGBUTTON1_6 = RESS.getString("AUSGBUTTON1_6");

        AUSGBUTTON2_1 = RESS.getString("AUSGBUTTON2_1");
        AUSGBUTTON2_2 = RESS.getString("AUSGBUTTON2_2");

        AUSGCHECK1 = RESS.getString("AUSGCHECK1");

        AUSGRADIO1 = RESS.getString("AUSGRADIO1");
        AUSGRADIO2 = RESS.getString("AUSGRADIO2");
        AUSGRADIO3 = RESS.getString("AUSGRADIO3");

        AUSGTEXTFELD = RESS.getString("AUSGTEXTFELD");

        AUSGLISTE1_1 = RESS.getString("AUSGLISTE1_1");
        AUSGLISTE1_2 = RESS.getString("AUSGLISTE1_2");

        AUSGCOMBO = RESS.getString("AUSGCOMBO");

        // ---------------------------------------------------------------------
        // Properties

        PROPS = new Properties();
        try
        {
            PROPS.load(new FileInputStream(PROPS_DATEINAME));
        }
        catch (IOException exc)
        {
            System.err.println("Properties-Datei nicht gefunden!!!!!!!!");
            
            // Default properties:
            PROPS.setProperty("FENSTERBREITE", "550");
            PROPS.setProperty("FENSTERHOEHE",  "700");
        }

        FENSTERBREITEstr = PROPS.getProperty("FENSTERBREITE");
        FENSTERBREITE    = Integer.parseInt(FENSTERBREITEstr);

        FENSTERHOEHEstr = PROPS.getProperty("FENSTERHOEHE");
        FENSTERHOEHE    = Integer.parseInt(FENSTERHOEHEstr);
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /** Privater Standardkonstruktor                                          */
    private MusterfensterKonstante()
    {
    }
}