/*
 * MusterfensterV.java
 */
package at.beringer.oopBuch.guiSwingI18N;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.einfacheBsp.AutoM;
import static at.beringer.oopBuch.guiSwingI18N.MusterfensterKonstante.*;

/**
Hauptfenster mit einfachen Standard-GUI-Komponenten.
<p>
Anonyme Klassen: von 'ActionListener' <br />
                 und von 'MouseAdapter' <br />
Model-Klasse:    'AutoM' (dient als Fachobjekt zum Testen dieser Fensterklasse)
</p>
<p>
Vollständig internationalisierte und parametrisierte Klasse.
</p>

<hr />

@since Juni 2012

@see AutoM
@see ProgrammMusterfenster
@see at.beringer.oopBuch.guiSwingI18N.MusterfensterKonstante
@see at.beringer.oopBuch.guiSwing.MusterfensterV

@author Beringer Alfred
*/
public class MusterfensterV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------
      
    private JLabel        label        = new JLabel(LABEL);
    private JButton       button1      = new JButton(BUTTON1);
    private JButton       button2      = new JButton(BUTTON2);

    private JCheckBox     check1       = new JCheckBox(CHECK1, false);
    private JCheckBox     check2       = new JCheckBox(CHECK2, true);
    private JCheckBox     check3       = new JCheckBox(CHECK3, true);

    private String        randtext     = RANDTEXT;
    private ButtonGroup   buttongruppe = new ButtonGroup();
    private JRadioButton  radio1       = new JRadioButton(RADIO1, false);
    private JRadioButton  radio2       = new JRadioButton(RADIO2, false);
    private JRadioButton  radio3       = new JRadioButton(RADIO3, true);

    private JLabel        labTxfeld    = new JLabel(LABTXFELD);
    private JTextField    textfeld     = new JTextField(TEXTFELD, 20);

    private JLabel        labTxbereich = new JLabel(LABTXBEREICH);
    private JTextArea     textbereich  = new JTextArea(TEXTBEREICH, 4, 30);
    private JScrollPane   textscroll   = new JScrollPane(textbereich);

    private JLabel        labListe1    = new JLabel(LABLISTE1);
    private DefaultListModel<String> liste1Model = new DefaultListModel<>();
    private JList<String> liste1       = new JList<>(liste1Model);
    private JScrollPane   liste1scroll = new JScrollPane(liste1);

    private String[]      liste1Daten  = {
                                           LISTE1_1,
                                           LISTE1_2,
                                           LISTE1_3,
                                           LISTE1_4,
                                           LISTE1_5,
                                           LISTE1_6,
                                           LISTE1_7,
                                           LISTE1_8
                                         };

    private JLabel        labListe2    = new JLabel(LABLISTE2);
    private DefaultListModel<String> liste2Model = new DefaultListModel<>();
    private JList<String> liste2       = new JList<>(liste2Model);

    private String[]      liste2Daten  = {
                                           LISTE2_1,
                                           LISTE2_2,
                                           LISTE2_3,
                                           LISTE2_4
                                         };

    private JLabel        labCombo     = new JLabel(LABCOMBO);
    private JComboBox<String> combo    = new JComboBox<String>(new String[]  {
                                                                        COMBO_1,
                                                                        COMBO_2,
                                                                        COMBO_3
                                                                             });

    /** Aktionszähler                                                         */
    private int  zaehler = 0;

    /** Model-Klasse: Fachobjekt (zum Testen)                                 */
    private AutoM  auto = new AutoM();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Das Erzeugen der einzelnen Teilfenster (Panels) und das Anmelden bei den
    Ereigniswächtern wird in private Methoden ausgelagert.
    </p>
    */
    public MusterfensterV()
    {
        this.setTitle(FENSTERTITEL);
        this.setSize(FENSTERBREITE, FENSTERHOEHE);

        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        initKomponenten();
        initFenster();
        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Eigenschaften der GUI-Komponenten definieren.
    */
    private void initKomponenten()
    {
        // Optionsfelder
        buttongruppe.add(radio1);
        buttongruppe.add(radio2);
        buttongruppe.add(radio3);

        // Textfeld (einzeilig)
        textfeld.setEditable(true);

        // Textbereich (mehrzeilig)
        textbereich.setEditable(true);

        // Listen

        liste1.setVisibleRowCount(3);
        liste1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // Liste1 füllen (Zeilen einzeln hinzufügen)
        for (String zeile : liste1Daten)
        {
            liste1Model.addElement(zeile);
        }

        // Liste2 füllen (Zeilen einzeln hinzufügen)
        for (String zeile : liste2Daten)
        {
            liste2Model.addElement(zeile);
        }
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        JPanel  poben1 = new JPanel(new FlowLayout());
        JPanel  poben2 = new JPanel(new FlowLayout());
        JPanel  poben3 = new JPanel(new FlowLayout());
        JPanel  poben  = new JPanel(new GridLayout(0,1));            // 1 Spalte
                                       // Anzahl 0 bedeutet: "so viel wie nötig"

        JPanel  pmitte4 = new JPanel(new FlowLayout());
        JPanel  pmitte5 = new JPanel(new FlowLayout());
        JPanel  pmitte6 = new JPanel(new FlowLayout());
        JPanel  pmitte7 = new JPanel(new FlowLayout());
        JPanel  pmitte8 = new JPanel(new FlowLayout());
        JPanel  pmitte  = new JPanel(new GridLayout(0,1));           // 1 Spalte
                                       // Anzahl 0 bedeutet: "so viel wie nötig"

        // Initialisieren 'poben1' für Schaltflächen
        poben1.add(label);
        poben1.add(button1);
        poben1.add(button2);

        // Initialisieren 'poben2' für Kontrollkästchen
        poben2.add(check1);
        poben2.add(check2);
        poben2.add(check3);

        // Initialisieren 'poben3' für Optionsfelder
        initPoben3(poben3);

        // Initialisieren 'poben'
        poben.add(poben1);
        poben.add(poben2);
        poben.add(poben3);

        // Initialisieren Panels für 'pmitte'
        initPanelMitBeschriftung(pmitte4, labTxfeld,    textfeld);
        initPanelMitBeschriftung(pmitte5, labTxbereich, textscroll);
        initPanelMitBeschriftung(pmitte6, labListe1,    liste1scroll);
        initPanelMitBeschriftung(pmitte7, labListe2,    liste2);
        initPanelMitBeschriftung(pmitte8, labCombo,     combo);

        // Initialisieren 'pmitte'
        pmitte.add(pmitte4);
        pmitte.add(pmitte5);
        pmitte.add(pmitte6);
        pmitte.add(pmitte7);
        pmitte.add(pmitte8);

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        this.setLayout(new BorderLayout());

        this.add(poben,BorderLayout.NORTH);
        this.add(pmitte,BorderLayout.CENTER);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren 'poben3' für Optionsfelder.
    <p>
    Gezeigt wird beispielhaft die Verwendung eines Randes.
    </p>

    @param  p  'poben3' für Optionsfelder
    */
    private void initPoben3(JPanel p)
    {
        Border  rand      = BorderFactory.createLoweredBevelBorder();
        Border  randtitel = BorderFactory.createTitledBorder(rand, randtext);

        p.setBorder(randtitel);
        p.add(radio1);
        p.add(radio2);
        p.add(radio3);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren Panelzeile für eine GUI-Komponente mit Beschriftung. <br />
    Die Beschriftung steht links - die Komponente rechts daneben.

    @param  p             Panel für GUI-Komponente mit Beschriftung
    @param  beschriftung  Beschriftung (Label)
    @param  komponente    GUI-Komponente
    */
    private void initPanelMitBeschriftung(JPanel      p,
                                          JLabel      beschriftung,
                                          JComponent  komponente)
    {
        JPanel  plinks  = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JPanel  prechts = new JPanel(new FlowLayout(FlowLayout.LEFT));

        plinks.add(beschriftung);
        prechts.add(komponente);
        p.setLayout(new GridLayout(1,2));
        p.add(plinks);
        p.add(prechts);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        button1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button1event(evt);
                  }
              });    // button1 ist zuständig für Kontrollkästchen
        button2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      button2event(evt);
                  }
              });    // button2 ist zuständig für Listen
        check1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      check1event(evt);
                  }
              });     // nicht notwendig, zuständig ist button1!
    
        // check2 und check3 nicht angemeldet (zuständig ist button1)
    
        radio1.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio1event(evt);
                  }
              });
        radio2.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio2event(evt);
                  }
              });
        radio3.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      radio3event(evt);
                  }
              });
        textfeld.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      textfeldevent(evt);
                  }
              });
        liste1.addMouseListener(new MouseAdapter()
              {
                  @Override
                  public void mouseClicked(MouseEvent evt)
                  {
                      liste1event(evt);
                  }
              });
        combo.addActionListener(new ActionListener()
              {
                  @Override
                  public void actionPerformed(ActionEvent evt)
                  {
                      comboevent(evt);
                  }
              });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************
    
    /**
    Methode für Schaltfläche1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGBUTTON1);

        if (check1.isSelected())
        {
            auto.ausgeben(AUSGBUTTON1_1);
        }
        else
        {
            auto.ausgeben(AUSGBUTTON1_2);
        }

        if (check2.isSelected())
        {
            auto.ausgeben(AUSGBUTTON1_3);
        }
        else
        {
            auto.ausgeben(AUSGBUTTON1_4);
        }

        if (check3.isSelected())
        {
            auto.ausgeben(AUSGBUTTON1_5);
        }
        else
        {
            auto.ausgeben(AUSGBUTTON1_6);
        }
    }

    // *************************************************************************

    /**
    Methode für Schaltfläche2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void button2event(ActionEvent evt)
    {
        java.util.List<String>  inhalte = liste2.getSelectedValuesList();   // seit Java 7.0
                                          // seit Java 7.0
        int[]                   indices = liste2.getSelectedIndices();

        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);

        auto.ausgeben(AUSGBUTTON2_1);
        auto.ausgeben("            " +  liste1.getSelectedValue()
                              + " (" + (liste1.getSelectedIndex()+1) 
                              + AUSGZEILE);

        auto.ausgeben(AUSGBUTTON2_2);
        for (int i = 0; i < inhalte.size(); i++)
        {
            auto.ausgeben("            " + inhalte.get(i)
                                  + " (" + (indices[i]+1) 
                                  + AUSGZEILE);
        }
    }

    // *************************************************************************

    /**
    Methode für Kontrollkästchen1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void check1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGCHECK1);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld1.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio1event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGRADIO1);
        auto.raufschalten();
        auto.zeigeInhalt();
        textbereich.append(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld2.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio2event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGRADIO2);
        auto.bremsen();
        auto.zeigeInhalt();
        textbereich.append(NEUEZEILE + auto);
    }

    // *************************************************************************

    /**
    Methode für Optionsfeld3.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void radio3event(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION
                           + NEUEZEILE + auto);
        auto.ausgeben(AUSGRADIO3);
        check3.setSelected(false);
    }

    // *************************************************************************

    /**
    Methode für Textfeld.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void textfeldevent(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGTEXTFELD + textfeld.getText());
    }

    // *************************************************************************

    /**
    Methode für Listenfeld1.

    @param  evt  MouseEvent (von API-Methode 'mouseClicked' übernommen)
    */
    private void liste1event(MouseEvent evt)
    {
        auto.ausgeben(AUSGLISTE1_1);
        auto.ausgeben("            " +  liste1.getSelectedValue()
                              + " (" + (liste1.getSelectedIndex() + 1) 
                              + AUSGZEILE);
        auto.ausgeben(AUSGLISTE1_2
                      + (liste1.locationToIndex(evt.getPoint()) + 1) 
                      + AUSGZEILE);
    }

    // *************************************************************************

    /**
    Methode für Auswahlfeld.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void comboevent(ActionEvent evt)
    {
        textbereich.append(NEUEZEILE + (++zaehler) + AUSGACTION);
        auto.ausgeben(AUSGCOMBO + combo.getSelectedItem()
                         + " (" + (combo.getSelectedIndex() + 1) 
                         + AUSGZEILE);
    }
}
