/*
 * DBconnectionKonstante.java   
 */
package at.beringer.oopBuch.jdbc;

import java.util.Properties;
import java.io.FileInputStream;
import java.io.IOException;

/**
Statische Klasse für Properties-Konstante (nicht-lokalisierte Konstante).
<p>
Name der Properties-Datei (für die URL der DB-Verbindung):
</p>
     'datenbank.properties'
<p>
Diese Properties-Datei befindet sich im Unterverzeichnis 'properties' des 
Benutzerverzeichnisses (= Projekt-Verzeichnis). <br />
--> vollständiger qualifizierter Name von Properties-Dateien:
</p>
     'at/beringer/oopBuch/properties/datenbank.properties'
<p>
Bemerkungen:
<ul>
  <li>Die Konstante 'DBMS' dient nur zum Testen bei den Konsolausgaben.</li>
  <li>Die Treiberklasse wird ab Java Version 6.0 nicht mehr benötigt, weil sie 
      automatisch geladen wird.</li>
</ul>
</p>

<hr />

@since Juli 2012

@see DBconnection

@author Beringer Alfred
*/
public final class DBconnectionKonstante
{
    // -------------------------------------------------------------------------
    // Private Klassenkonstante
    // -------------------------------------------------------------------------

    // -------------------------------------------------------------------------
    // Projekt-Verzeichnis

    /** Name des Projekt-Verzeichnisses                                       */
    private static final String  PROJEKT_VERZEICHNISNAME = 
        System.getProperty("user.dir") + System.getProperty("file.separator")
                        + "at"         + System.getProperty("file.separator")
                        + "beringer"   + System.getProperty("file.separator")
                        + "oopBuch"    + System.getProperty("file.separator");

    // -------------------------------------------------------------------------
    // Properties-Datei

    /** Name der Properties-Datei (für nicht-lokalisierte Konstante)          */
    private static final String  PROPS_DATEINAME = 
        PROJEKT_VERZEICHNISNAME
                        + "properties" + System.getProperty("file.separator")
                        + "datenbank.properties";

    /**
    Properties-Datei für nicht-lokalisierte Konstante:
    <ul>
      <li>URL der DB-Verbindung</li>
    </ul>
    */
    private static final Properties  PROPS;

    // -------------------------------------------------------------------------
    // Öffentliche Klassenkonstante
    // -------------------------------------------------------------------------

    // ----- Nicht-lokalisierte Konstante

    /** Klassenkonstante für die URL der DB-Verbindung                        */
    public static final String  CONNECTIONSTRING;

    /** Klassenkonstante für den DBMS-Namen (nur zum Testen!)                 */
    public static final String  DBMS;

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------
    
    /**
    Klassenkonstruktor. <br />
    Initialisieren der Klassenkonstanten.
    <p>
    Die Anfangswerte für die Klassenkonstanten werden von einer 
    Properties-Datei übernommen.
    </p>
    */
    static
    {
        // ---------------------------------------------------------------------
        // Properties

        PROPS = new Properties();
        try
        {
            PROPS.load(new FileInputStream(PROPS_DATEINAME));
        }
        catch (IOException fehler)
        {
            System.err.println("Fehler beim Laden der Properties-Datei: "
                               + fehler.getMessage());
            
            // Default properties:
            PROPS.setProperty("CONNECTIONSTRING",
                        "jdbc:derby:E:\\Alfred1\\Datenbanken-JavaDB\\Schueler");
            PROPS.setProperty("DBMS", "JavaDB");
        }

        CONNECTIONSTRING = PROPS.getProperty("CONNECTIONSTRING");
        DBMS             = PROPS.getProperty("DBMS");
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /** Privater Standardkonstruktor                                          */
    private DBconnectionKonstante()
    {
    }
}
