/*
 * LesenDBAusgeben.java
 */
package at.beringer.oopBuch.jdbc;

import java.util.*;

import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler;
import at.beringer.oopBuch.daoKlassen.DAOausgebenSchueler;

/**
Einfache Konsolenanwendung für Lesen von Daten aus einer Datenbank und 
Ausgeben (Schreiben) der Daten auf die Standardausgabe (= Bildschirm).<br />
Die Ausgabe erfolgt in mehreren Varianten.<br />
<p>
Eine Variante listet nur die Schüler und Schülerinnen einer einzigen Klasse 
(Klassenliste). Die Klassenbezeichnung für diese Klassenliste) wird beim Aufruf 
als Parameter übergeben.
</p>

<hr />

@since Juli 2012
@version Jänner 2014 (neue Ausgabe-Instanz für neuerliche Ausgabe)

@see Schueler
@see DAOlesenDBSchueler
@see DAOausgebenSchueler

@see ProgrammLesenDBAusgeben

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben
@see ErfassenSchreibenDB

@author Beringer Alfred
*/
public class LesenDBAusgeben
{
    // -------------------------------------------------------------------------
    // Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Öffentliche Methode zum Starten der Applikation.
    <p>
    SQL-SELECT-Befehle ausführen und ausgeben (mehrere Varianten)
    </p>

    @param  klasse  Klassenbezeichnung für Klassenliste
    */
    public void start(String klasse)
    {
        DAOlesenDBSchueler   eingabedaten = new DAOlesenDBSchueler();
        DAOausgebenSchueler  ausgabedaten;

        List<String>         schuelerlisteStr;
        List<Schueler>       schuelerliste;

        // Öffnen der Datenbank erfolgt in der DAO-Klasse

        // ---------------------------------------------------------------------
        // Alle Schüler (Schülerdaten als Zeichenkette)
        // Ausgabe direkt ohne DAO-Klasse (1 Zeile pro Schüler-Zeichenkette)

        System.out.println();
        System.out.println("*************************************************");
        System.out.println("Sehr einfache Schülerliste");
        System.out.println();

        schuelerlisteStr = eingabedaten.selectAlleSchuelerStr();
        if (schuelerlisteStr == null || schuelerlisteStr.size() == 0)
        {
            System.out.println();
            System.out.println("Keine Zeilen gefunden!");
        }
        else
        {
            for (String schueler : schuelerlisteStr)
            {
                System.out.println(schueler);               // 1 Zeile schreiben
            }
        }

        // ---------------------------------------------------------------------
        // Alle Schüler (Schülerdaten als 'Schueler'-Objekte)
        // Ausgabe mittels DAO-Klasse 'DAOausgebenSchueler' 
        //    (1 Zeile pro 'Schueler'-Objekt)

        System.out.println();
        System.out.println("*************************************************");
        System.out.println("Schülerliste mit aufbereiteten Schüler-Daten");

        ausgabedaten = new DAOausgebenSchueler();

        schuelerliste = eingabedaten.selectAlleSchueler();
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            System.out.println();
            System.out.println("Keine Zeilen gefunden!");
        }
        else
        {
            for (Schueler schueler : schuelerliste)
            {
                // Daten übernehmen
                ausgabedaten.setSchueler( schueler );        // Daten übernehmen
                ausgabedaten.schreibenZeile();              // 1 Zeile schreiben
            }
        }

        // ---------------------------------------------------------------------
        // Klassenliste - nur Schüler einer Klasse 
        //    (Schülerdaten als 'Schueler'-Objekte)
        // Ausgabe mittels DAO-Klasse 'DAOausgebenSchueler' 
        //    (1 Zeile pro 'Schueler'-Objekt)

        System.out.println();
        System.out.println("*************************************************");
        System.out.println("Schülerliste der Klasse " + klasse);

        ausgabedaten = new DAOausgebenSchueler();
        // Hinweis: Neue Ausgabe-Instanz notwendig!
        //          Instanzattribute (z.B. Zeilenzähler) müssen wieder 
        //          initialisiert werden!

        schuelerliste = eingabedaten.selectSchuelerKlasse(klasse);
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            System.out.println();
            System.out.println("Keine Zeilen gefunden!");
        }
        else
        {
            for (Schueler schueler : schuelerliste)
            {
                ausgabedaten.setSchueler( schueler );        // Daten übernehmen
                ausgabedaten.schreibenZeile();              // 1 Zeile schreiben
            }
        }

        // ---------------------------------------------------------------------
        System.out.println();
        System.out.println("*************************************************");

        // Am Ende der Applikation Verbindung zur Datenbank schließen
        eingabedaten.schliessenVerbindung();
    }
}
