/*
 * ErfassenSchreibenDBV.java
 */
package at.beringer.oopBuch.jdbcSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOschreibenDBSchueler;

/**
Einfache Swing-Anwendung für Erfassen von Daten und 
Schreiben der Daten in eine Datenbank.
<p>
Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt)
</p>
<p>
Bemerkungen:
<ul>
  <li>Es werden der Reihe nach Daten eines 'Schueler'-Objekts erfasst und in 
      eine Datenbank geschrieben, d.h. diese Fensterklasse entspricht der 
      DAO-Klasse 'DAOerfassenSchueler' und der Applikationsklasse 
      'ErfassenSchreibenDB'.</li>
  <li>Definition eines 'Default-Buttons'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see Schueler
@see DAOschreibenDBSchueler

@see ProgrammErfassenSchreibenDB

@see at.beringer.oopBuch.daoKlassen.DAOerfassenSchueler
@see at.beringer.oopBuch.jdbc.ErfassenSchreibenDB

@author Beringer Alfred
*/
public class ErfassenSchreibenDBV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private JLabel      labName     = new JLabel("Name: ");
    private JTextField  txfName     = new JTextField(15);

    private JLabel      labNote     = new JLabel("Notenschnitt: ");
    private JTextField  txfNote     = new JTextField(10);

    private JLabel      labKlasse   = new JLabel("Klasse: ");
    private JTextField  txfKlasse   = new JTextField(10);

    private JButton     butErfassen = new JButton("Erfassen");
    private JButton     butEnde     = new JButton("  Ende  ");

    // ----------

    private DefaultListModel<String>  listeDaten = new DefaultListModel<>();
    private JList<String>             liste      = new JList<>(listeDaten);
    private JScrollPane               scrListe   = new JScrollPane(liste);

    // ----------

    /**
    Datenobjekt für die zu erfassenden Daten
    */
    private Schueler  schueler;

    private DAOschreibenDBSchueler  ausgabedaten = new DAOschreibenDBSchueler();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public ErfassenSchreibenDBV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenerfassung (Datenbank)");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butErfassen);

        initFenster();
        initListener();

        // Öffnen der Datenbank erfolgt in der DAO-Klasse
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());
        
        // oben ---------------------------------
        JPanel pOben   = new JPanel();

        JPanel pName   = new JPanel();
        JPanel pNote   = new JPanel();
        JPanel pKlasse = new JPanel();

        // Initialisieren Panels für 'pOben'
        initPanelMitBeschriftung(pName,   labName,   txfName);
        initPanelMitBeschriftung(pNote,   labNote,   txfNote);
        initPanelMitBeschriftung(pKlasse, labKlasse, txfKlasse);

        pOben.setLayout(new GridLayout(3,2));

        pOben.add(pName);
        pOben.add(pNote);
        pOben.add(pKlasse);

        this.add(pOben, BorderLayout.NORTH);

        // Mitte --------------------------------
        this.add(scrListe);

        // unten --------------------------------
        JPanel pUnten = new JPanel();

        pUnten.setLayout(new FlowLayout());
        pUnten.add(butErfassen);
        pUnten.add(butEnde);
        
        this.add(pUnten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren Panelzeile für eine GUI-Komponente mit Beschriftung. <br />
    Die Beschriftung steht links - die Komponente rechts daneben.

    @param  p             Panel für GUI-Komponente mit Beschriftung
    @param  beschriftung  Beschriftung (Label)
    @param  komponente    GUI-Komponente
    */
    private void initPanelMitBeschriftung(JPanel      p,
                                          JLabel      beschriftung,
                                          JComponent  komponente)
    {
        JPanel  plinks  = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JPanel  prechts = new JPanel(new FlowLayout(FlowLayout.LEFT));

        plinks.add(beschriftung);
        prechts.add(komponente);
        p.setLayout(new GridLayout(1,2));
        p.add(plinks);
        p.add(prechts);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });
        butErfassen.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     butErfassenEreignis(evt);
                 }
             });
        butEnde.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     fensterSchliessen(evt);
                 }
             });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Daten erfassen, eingegebene Daten kontrollieren, korrekte Daten schreiben

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void butErfassenEreignis(ActionEvent evt)
    {
        String  name;
        String  notenschnittStr;
        float   notenschnitt;
        String  klasse;

        // --------------------
        // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)

        name = txfName.getText();

        // --------------------
        // 2. Datenfeld (Notendurchschnitt - Zahl)
        
        notenschnittStr = txfNote.getText();
        try
        {
            notenschnitt = Float.parseFloat(notenschnittStr);
        }
        catch (NumberFormatException fehler)
        {
            listeDaten.addElement("   ***** Falscher Notendurchschnitt: " 
                                  + notenschnittStr + " *****");
            return;
        }

        // --------------------
        // 3. Datenfeld (Klassenbezeichnung - Zeichenkette mit fixer Länge)

        klasse = txfKlasse.getText();

        // --------------------
        // Erfasste Daten sind korrekt -> Objekt für Datensatz instanzieren

        schueler = new Schueler();
        schueler.setName(name);
        schueler.setNotenschnitt(notenschnitt);
        schueler.setKlasse(klasse);

        // Daten übernehmen und Datensatz schreiben
        ausgabedaten.setSchueler(schueler);
        ausgabedaten.schreiben();

        // Erfasste Daten zur Kontrolle in die Liste übertragen
        listeDaten.addElement(name + " "
                              + notenschnitt + " "
                              + klasse);

        // Textfelder leeren
        txfName.setText("");
        txfNote.setText("");
        txfKlasse.setText("");
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    private void fensterSchliessen(AWTEvent evt)
    {
        ausgabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
