/*
 * LesenDBAusgebenJTableV.java
 */
package at.beringer.oopBuch.jdbcSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import java.util.*;
import java.util.List;

import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler;

/**
Einfache Swing-Anwendung für Lesen von Daten aus einer Datenbank und 
Ausgeben (Schreiben) der Daten in eine Tabelle (JTable).<br />
Die Datensätze werden einzeln der Reihe nach in eine Tabelle geschrieben.
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts. Sie werden 
aufbereitet und formatiert in einer Tabelle ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt)
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der DAO-Klasse 'DAOausgebenSchueler' und 
      der Applikationsklasse 'LesenDBAusgeben'.</li>
  <li>Definition eines 'Default-Buttons'.</li>
  <li>Definition eines 'Escape-Buttons'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see Schueler
@see DAOlesenDBSchueler

@see ProgrammLesenDBAusgebenJTable

@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@author Beringer Alfred
*/
public class LesenDBAusgebenJTableV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DefaultTableModel  tabelleDTM           = new DefaultTableModel();
    private JTable             table                = new JTable(tabelleDTM);
    private JScrollPane        tablescroll          = new JScrollPane(table);
    private Vector<String>     tabellenueberschrift = new Vector<String>();

    private Vector<Vector<String>>  alleZeilen = new Vector<Vector<String>>();

    private JButton  butNext = new JButton("  Nächster Satz  ");
    private JButton  butEnde = new JButton("  Ende  ");

    // ----------

    private DAOlesenDBSchueler  eingabedaten = new DAOlesenDBSchueler();
    private List<Schueler>      schuelerliste;
    private Iterator<Schueler>  schueleriterator;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public LesenDBAusgebenJTableV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenausgabe in einer JTable (Datenbank)");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butNext);

        initTabelle();
        initFenster();
        initListener();

        // Öffnen der Datenbank erfolgt in der DAO-Klasse
        lesenDaten();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    SQL-SELECT-Befehl ausführen. Iterator für Ergebnisliste bereitstellen.
    */
    private void lesenDaten()
    {
        schuelerliste = eingabedaten.selectAlleSchueler();
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            JOptionPane.showMessageDialog(this,
                                          "***** Keine Zeilen gefunden! *****");
            schueleriterator = null;
            return;
        }

        schueleriterator = schuelerliste.iterator();
    }

    // -------------------------------------------------------------------------

    /**
    JTable erstellen.
    */
    private void initTabelle()
    {
        tabellenueberschrift.add("Name (variable Länge");
        tabellenueberschrift.add("Notendurchschnitt");
        tabellenueberschrift.add("Klasse (fixe Länge");

        tabelleDTM.setDataVector(alleZeilen, tabellenueberschrift);
        
        // RowSorter für Tabelle setzen (neu seit JDK 6.0).
        // Damit können durch Klick auf eine Spalte die Zeilen sortiert werden,
        // nochmaliges Klicken dreht die Sortierfolge um.
        // Achtung:
        //    Sortierung erfolgt für Zeichenketten (von links nach rechts)!
        RowSorter<TableModel>  sorter = 
                                     new TableRowSorter<TableModel>(tabelleDTM);
        table.setRowSorter(sorter);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        // oben --------------------------------
        JPanel oben = new JPanel();

        oben.setLayout(new FlowLayout());
        oben.add(butNext);

        this.add(oben, BorderLayout.NORTH);

        // Mitte --------------------------------
        this.add(tablescroll);

        // unten --------------------------------
        JPanel unten = new JPanel();

        unten.setLayout(new FlowLayout());
        unten.add(butEnde);

        this.add(unten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });

        butNext.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent evt)
                {
                    lesen(evt);
                }
            });

        butEnde.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });
    }

    // *************************************************************************

    /**
    Definiert 'Escape'-Taste für 'Ende'-Schaltfläche. <br />

    'Escape'-Taste ist gültig im gesamten Fenster (unabh. vom Focus).
    */
    @Override
    protected JRootPane createRootPane()
    {
        JRootPane  rootpane = new JRootPane();

        rootpane.registerKeyboardAction(
                new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent evt)
                    {
                        fensterSchliessen(evt);
                    }
                },
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW);

        return rootpane;
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Lesen eines Datensatzes (aus der Resultatliste des SELECT-Ergebnisses).

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    private void lesen(ActionEvent evt)
    {
        if (schueleriterator.hasNext())
        {
            // nächsten Datensatz übernehmen
            addTabellenzeile(schueleriterator.next());
        }
        else
        {
            JOptionPane.showMessageDialog(this, "EOF");
        }
    }

    // -------------------------------------------------------------------------

    private void addTabellenzeile(Schueler daten)
    {
        Vector<String>  zeile = new Vector<String>();

        zeile.add(daten.getName());
        zeile.add("" + daten.getNotenschnitt());
        zeile.add(daten.getKlasse());
        
        alleZeilen.add(zeile);
        tabelleDTM.setDataVector(alleZeilen, tabellenueberschrift);
//        tabelleDTM.addRow(zeile);
      
        testdruck();   // Nur zum Testen
    }

    // -------------------------------------------------------------------------

    /**
    Diese Methode dient NUR zum Testen der Ausgabe.
    <p>
    Achtung:<br />
    Nach jedem Lesen werden immer wieder ALLE bisher gelesenen Datensätze 
    ausgedruckt!
    */
    private void testdruck()
    {
        Vector<String>  zeile;
        
        for (int i = 0; i < alleZeilen.size(); i++)
        {
            zeile = alleZeilen.get(i);
            System.out.println(zeile.get(0) + " " +
                               zeile.get(1) + " " +
                               zeile.get(2));
        }
        System.out.println();
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    private void fensterSchliessen(AWTEvent evt)
    {
        eingabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
