/*
 * LesenDBAusgebenStringV.java
 */
package at.beringer.oopBuch.jdbcSwing;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.util.List;

import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler;
import at.beringer.oopBuch.daoKlassen.DAOausgebenSchuelerString;

/**
Sehr Einfache Swing-Anwendung für Lesen von Daten aus einer Datenbank und 
Ausgeben (Schreiben) der Daten in eine Liste (JList).<br />
Am Beginn werden alle Datensätze gelesen und in eine Liste geschrieben.
<p>
Die Schülerdaten werden in einer Zeichenkette aufbereitet, diese Zeichenkette 
wird in einer Liste ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der 
      Applikationsklasse 'LesenDBAusgeben'.</li>
  <li>Definition eines 'Default-Buttons'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see DAOlesenDBSchueler
@see DAOausgebenSchuelerString

@see ProgrammLesenDBAusgebenString

@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@author Beringer Alfred
*/
public class LesenDBAusgebenStringV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DefaultListModel<String>  listeDaten = new DefaultListModel<>();
    private JList<String>             liste      = new JList<>(listeDaten);
    private JScrollPane               scrListe   = new JScrollPane(liste);

    private JButton  butEnde = new JButton("  Ende  ");

    // ----------

    private DAOlesenDBSchueler  eingabedaten;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public LesenDBAusgebenStringV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenausgabe in einer JList (Datenbank)");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butEnde);

        initListe();
        initFenster();
        initListener();

        lesenDaten();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    SQL-SELECT-Befehl ausführen, ausgeben aller Datensätze (Zeilen)
    */
    private void lesenDaten()
    {
        DAOausgebenSchuelerString  ausgabedaten;
        List<Schueler>             schuelerliste;

        eingabedaten = new DAOlesenDBSchueler();

        // Öffnen der Datenbank erfolgt in der DAO-Klasse

        schuelerliste = eingabedaten.selectAlleSchueler();
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            listeDaten.addElement("***** Keine Zeilen gefunden! *****" 
                                  + "   --->   Ende");
            return;
        }

        // ---------------------------------------------------------------------
        // Alle Datensätze (Zeilen) lesen und in die Liste schreiben

        ausgabedaten = new DAOausgebenSchuelerString();

        // Überschrift mit nachfolgender Leerzeile
        listeDaten.addElement(DAOausgebenSchuelerString.getUeberschrift());
        listeDaten.addElement(" ");
        
        for (Schueler schueler : schuelerliste)
        {
            // Daten übernehmen
            ausgabedaten.setSchueler(schueler);
            listeDaten.addElement(ausgabedaten.schreibenZeile());
        }
    }

    // -------------------------------------------------------------------------

    /**
    JList initialisieren.
    */
    private void initListe()
    {
        // Nichtproportionale Schrift festlegen (für spaltenweise Aufbereitung)
        liste.setFont( new Font(Font.MONOSPACED,
                                liste.getFont().getStyle(),
                                liste.getFont().getSize()) );
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        // Mitte --------------------------------
        this.add(scrListe);

        // unten --------------------------------
        JPanel unten = new JPanel();

        unten.setLayout(new FlowLayout());
        unten.add(butEnde);

        this.add(unten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });

        butEnde.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     fensterSchliessen(evt);
                 }
             });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    private void fensterSchliessen(AWTEvent evt)
    {
        eingabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
