/*
 * ErfassenSchreibenDBV.java
 */
package at.beringer.oopBuch.jdbcSwingView;

import java.awt.*;
import java.awt.event.*;

import at.beringer.oopBuch.viewKlassen.ErfassenV;
import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOschreibenDBSchueler;

/**
Einfache Swing-Anwendung für Erfassen von Daten und 
Schreiben der Daten in eine Datenbank.
<p>
Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt) <br />
View-Klasse:    'ErfassenV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Es werden der Reihe nach Daten eines 'Schueler'-Objekts erfasst und in 
      eine Datenbank geschrieben, d.h. diese Fensterklasse entspricht der 
      DAO-Klasse 'DAOerfassenSchueler' und der Applikationsklasse 
      'ErfassenSchreibenDB'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see ErfassenV
@see Schueler
@see DAOschreibenDBSchueler

@see ProgrammErfassenSchreibenDB

@see at.beringer.oopBuch.daoKlassen.DAOerfassenSchueler
@see at.beringer.oopBuch.jdbc.ErfassenSchreibenDB

@author Beringer Alfred
*/
public class ErfassenSchreibenDBV extends ErfassenV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu erfassenden Daten
    */
    private Schueler  schueler;

    private DAOschreibenDBSchueler  ausgabedaten = new DAOschreibenDBSchueler();

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public ErfassenSchreibenDBV()
    {
        setTitle("Fenster für Datenerfassung (Datenbank)");

        // Öffnen der Datenbank erfolgt in der DAO-Klasse
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Daten erfassen, eingegebene Daten kontrollieren, korrekte Daten schreiben

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    @Override
    protected void butErfassenEreignis(ActionEvent evt)
    {
        String  name;
        String  notenschnittStr;
        float   notenschnitt;
        String  klasse;

        // --------------------
        // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)

        name = txfName.getText();

        // --------------------
        // 2. Datenfeld (Notendurchschnitt - Zahl)
        
        notenschnittStr = txfNote.getText();
        try
        {
            notenschnitt = Float.parseFloat(notenschnittStr);
        }
        catch (NumberFormatException fehler)
        {
            listeDaten.addElement("   ***** Falscher Notendurchschnitt: " 
                                  + notenschnittStr + " *****");
            return;
        }

        // --------------------
        // 3. Datenfeld (Klassenbezeichnung - Zeichenkette mit fixer Länge)

        klasse = txfKlasse.getText();

        // --------------------
        // Erfasste Daten sind korrekt -> Objekt für Datensatz instanzieren

        schueler = new Schueler();
        schueler.setName(name);
        schueler.setNotenschnitt(notenschnitt);
        schueler.setKlasse(klasse);

        // Daten übernehmen und Datensatz schreiben
        ausgabedaten.setSchueler(schueler);
        ausgabedaten.schreiben();

        // Erfasste Daten zur Kontrolle in die Liste übertragen
        listeDaten.addElement(name + " "
                              + notenschnitt + " "
                              + klasse);

        // Textfelder leeren
        txfName.setText("");
        txfNote.setText("");
        txfKlasse.setText("");
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        ausgabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
