/*
 * LesenDBAusgebenJTableV.java
 */
package at.beringer.oopBuch.jdbcSwingView;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import java.util.*;
import java.util.List;

import at.beringer.oopBuch.viewKlassen.AusgebenJTableV;
import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler;

/**
Einfache Swing-Anwendung für Lesen von Daten aus einer Datenbank und 
Ausgeben (Schreiben) der Daten in eine Tabelle (JTable).<br />
Die Datensätze werden einzeln der Reihe nach in eine Tabelle geschrieben.
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts. Sie werden 
aufbereitet und formatiert in einer Tabelle ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
Model-Klasse:   'Schueler' (Fachobjekt) <br />
View-Klasse:    'AusgebenJTableV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der DAO-Klasse 'DAOausgebenSchueler' und 
      der Applikationsklasse 'LesenDBAusgeben'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see AusgebenJTableV
@see Schueler
@see DAOlesenDBSchueler

@see ProgrammLesenDBAusgebenJTable

@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@author Beringer Alfred
*/
public class LesenDBAusgebenJTableV extends AusgebenJTableV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DAOlesenDBSchueler  eingabedaten = new DAOlesenDBSchueler();
    private List<Schueler>      schuelerliste;
    private Iterator<Schueler>  schueleriterator;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public LesenDBAusgebenJTableV()
    {
        setTitle("Fenster für Datenausgabe in einer JTable (Datenbank)");

        // Öffnen der Datenbank erfolgt in der DAO-Klasse
        lesenDaten();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    SQL-SELECT-Befehl ausführen. Iterator für Ergebnisliste bereitstellen.
    */
    private void lesenDaten()
    {
        schuelerliste = eingabedaten.selectAlleSchueler();
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            JOptionPane.showMessageDialog(this,
                                          "***** Keine Zeilen gefunden! *****");
            schueleriterator = null;
            return;
        }

        schueleriterator = schuelerliste.iterator();
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Lesen eines Datensatzes (aus der Resultatliste des SELECT-Ergebnisses).

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    @Override
    protected void lesen(ActionEvent evt)
    {
        if (schueleriterator.hasNext())
        {
            // nächsten Datensatz übernehmen
            addTabellenzeile(schueleriterator.next());
        }
        else
        {
            JOptionPane.showMessageDialog(this, "EOF");
        }
    }

    // -------------------------------------------------------------------------

    private void addTabellenzeile(Schueler daten)
    {
        Vector<String>  zeile = new Vector<String>();

        zeile.add(daten.getName());
        zeile.add("" + daten.getNotenschnitt());
        zeile.add(daten.getKlasse());
        
        alleZeilen.add(zeile);
        tabelleDTM.setDataVector(alleZeilen, tabellenueberschrift);
//        tabelleDTM.addRow(zeile);
      
        testdruck();   // Nur zum Testen
    }

    // -------------------------------------------------------------------------

    /**
    Diese Methode dient NUR zum Testen der Ausgabe.
    <p>
    Achtung:<br />
    Nach jedem Lesen werden immer wieder ALLE bisher gelesenen Datensätze 
    ausgedruckt!
    */
    private void testdruck()
    {
        Vector<String>  zeile;
        
        for (int i = 0; i < alleZeilen.size(); i++)
        {
            zeile = alleZeilen.get(i);
            System.out.println(zeile.get(0) + " " +
                               zeile.get(1) + " " +
                               zeile.get(2));
        }
        System.out.println();
    }

    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        eingabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
