/*
 * LesenDBAusgebenStringV.java
 */
package at.beringer.oopBuch.jdbcSwingView;

import java.awt.*;

import java.util.List;

import at.beringer.oopBuch.viewKlassen.AusgebenStringV;
import at.beringer.oopBuch.modelKlassen.Schueler;
import at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler;
import at.beringer.oopBuch.daoKlassen.DAOausgebenSchuelerString;

/**
Sehr Einfache Swing-Anwendung für Lesen von Daten aus einer Datenbank und 
Ausgeben (Schreiben) der Daten in eine Liste (JList).<br />
Am Beginn werden alle Datensätze gelesen und in eine Liste geschrieben.
<p>
Die Schülerdaten werden in einer Zeichenkette aufbereitet, diese Zeichenkette 
wird in einer Liste ausgegeben.
</p>
<p>
Anonyme Klasse: von 'ActionListener' <br />
View-Klasse:    'AusgebenStringV'
</p>
<p>
Bemerkungen:
<ul>
  <li>Diese Fensterklasse entspricht der 
      Applikationsklasse 'LesenDBAusgeben'.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
</ul>
</p>

<hr />

@since Juli 2014

@see AusgebenStringV
@see DAOlesenDBSchueler
@see DAOausgebenSchuelerString

@see ProgrammLesenDBAusgebenString

@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@author Beringer Alfred
*/
public class LesenDBAusgebenStringV extends AusgebenStringV
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private DAOlesenDBSchueler  eingabedaten;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public LesenDBAusgebenStringV()
    {
        setTitle("Fenster für Datenausgabe in einer JList (Datenbank)");

        lesenDaten();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    SQL-SELECT-Befehl ausführen
    */
    private void lesenDaten()
    {
        DAOausgebenSchuelerString  ausgabedaten;
        List<Schueler>             schuelerliste;

        eingabedaten = new DAOlesenDBSchueler();

        // Öffnen der Datenbank erfolgt in der DAO-Klasse

        schuelerliste = eingabedaten.selectAlleSchueler();
        if (schuelerliste == null || schuelerliste.size() == 0)
        {
            listeDaten.addElement("***** Keine Zeilen gefunden! *****" 
                                  + "   --->   Ende");
            return;
        }

        // ---------------------------------------------------------------------
        // Alle Datensätze lesen und in die Liste schreiben

        ausgabedaten = new DAOausgebenSchuelerString();

        // Überschrift mit nachfolgender Leerzeile
        listeDaten.addElement(DAOausgebenSchuelerString.getUeberschrift());
        listeDaten.addElement(" ");
        
        for (Schueler schueler : schuelerliste)
        {
            // Daten übernehmen
            ausgabedaten.setSchueler(schueler);
            listeDaten.addElement(ausgabedaten.schreibenZeile());
        }
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Beenden der Applikation - Verbindung zur Datenbank schließen

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    @Override
    protected void fensterSchliessen(AWTEvent evt)
    {
        eingabedaten.schliessenVerbindung();
        System.exit(0);
    }
}
