/*
 * Schueler.java
 */
package at.beringer.oopBuch.modelKlassen;

/**
Datenklasse ('Model'-Klasse) für eine Datenstruktur mit 3 Datenfeldern.
<p>
Diese Klasse dient als 'Model'-Klasse für die Demo-Programme zur Verarbeitung 
von Dateien und Datenbanken.
Objekte dieser Klasse (genauer: deren Attribute) werden als Datensätze in 
einer Datei bzw. als Zeilen in einer Datenbank-Tabelle gespeichert.
</p>
<p>
Jede Instanz dieser Klasse repräsentiert einen Schüler mit
<ul>
   <li>Name</li>
   <li>Notendurchschnitt</li>
   <li>Klassenbezeichnung</li>
</ul>
</p>
<p>
Das 1. Datenfeld (Name) beinhaltet eine Zeichenkette, die beliebig lang sein 
kann (d.h. die Zeichenkette hat eine variable Länge). <br />
Das 2. Datenfeld (Notendurchschnitt) beinhaltet einen float-Wert. <br />
Das 3. Datenfeld (Klassenbezeichnung) beinhaltet eine Zeichenkette mit einer 
fixen Anzahl von Zeichen - in diesem Fall die Klassenbezeichnung mit immer 
genau 7 Zeichen (d.h. die Zeichenkette hat eine fixe Länge).
</p>
<p>
Der Einfachheit halber ist die 'toString'-Methode nicht internationalisiert.
</p>

<hr />

@since August 2000
@version Juli 2002 (Zeichenkette mit variabler Länge)
@version <br />Juni 2006 (Methode 'toString' überschreiben)
@version <br />Juli 2014 (Verschieben in Paket 'modelKlassen')

@see at.beringer.oopBuch.daoKlassen.DAOerfassenSchueler
@see at.beringer.oopBuch.daoKlassen.DAOschreibenSchueler
@see at.beringer.oopBuch.daoKlassen.DAOschreibenDBSchueler
@see at.beringer.oopBuch.daoKlassen.DAOlesenSchueler
@see at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler
@see at.beringer.oopBuch.daoKlassen.DAOausgebenSchueler
@see at.beringer.oopBuch.daoKlassen.DAOausgebenSchuelerString

@author Beringer Alfred
*/
public class Schueler
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /**
    Zeichenkette mit einer fixen Anzahl von Leerzeichen. Sie dient zum Auffüllen
    einer zu kurzen Zeichenkette mit Leerzeichen.
    <p>
    Die Initialisierung erfolgt im Klassenkonstruktor.
    </p>
    */
    public static final String  KLASSELEER;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String  name;               // Zeichenkette mit variabler Länge
    private float   notenschnitt;       // float-Zahl
    private String  klasse;             // Zeichenkette mit fixer Länge

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenkonstante könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    </p>
    <p>
    Der Einfachheit halber werden die Werte fix vorgegeben - in diesem Fall eine
    Zeichenkette mit genau 7 Leerzeichen.
    </p>
    */
    static
    {
        KLASSELEER = "       ";         // Zeichenkette mit genau 7 Leerzeichen
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert das erste Datenfeld (Name) - eine Zeichenkette mit variabler Länge.

    @return  Name (1. Datenfeld)
    */
    public String getName()
    {
        return  name;
    }

    // ----------

    /**
    Ändert das erste Datenfeld (Name).

    @param  name  neuer Name (Zeichenkette mit variabler Länge)
    */
    public void setName(String name)
    {
        this.name = name;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das zweite Datenfeld (Notendurchschnitt) - ein float-Feld.

    @return  Notendurchschnitt (2. Datenfeld)
    */
    public float getNotenschnitt()
    {
        return  notenschnitt;
    }

    // ----------

    /**
    Ändert das zweite Datenfeld (Notendurchschnitt).

    @param  notenschnitt  neuer Notendurchschnitt (float-Wert)
    */
    public void setNotenschnitt(float notenschnitt)
    {
        this.notenschnitt = notenschnitt;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das dritte Datenfeld (Klassenbezeichnung) - eine Zeichenkette mit 
    fixer Länge.

    @return  Klassenbezeichnung (3. Datenfeld)
    */
    public String getKlasse()
    {
        return  klasse;
    }

    // ----------

    /**
    Ändert das dritte Datenfeld (Klassenbezeichnung) und kontrolliert auch 
    gleichzeitig die Länge dieses Feldes.
    <p>
    Eine zu kurze Zeichenkette wird mit Leerzeichen aufgefüllt, eine zu lange 
    wird einfach abgeschnitten.
    </p>

    @param  klasse  neue Klassenbezeichnung (Zeichenkette mit fixer Länge)
    */
    public void setKlasse(String klasse)
    {
        // mit Leerzeichen erweitern:
        this.klasse = klasse + KLASSELEER;                       
        // kürzen auf fixe Länge:
        this.klasse = this.klasse.substring(0,KLASSELEER.length());
    }

    // -------------------------------------------------------------------------
    // Methode 'toString'
    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'String Object.toString()'.
    <p>
    Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
    <pre>
      Schueler[ Name = ... (Länge = ...),
                Notendurchschnitt = ...,
                Klassenbezeichnung = *...* ]
    </pre>
    </p>
    <p>
    Die Texte sind der Einfachheit halber Literale.
    </p>

    @see java.lang.Object#toString()

    @return  Zeichenkette, die eine Instanz beschreibt
    */
    @Override
    public String toString()
    {
        StringBuilder text = new StringBuilder(this.getClass().getSimpleName());

        text.append("[ Name = "           ).append(name         ).append(
                    " (Länge = "          ).append(name.length()).append(
                "), Notendurchschnitt = " ).append(notenschnitt ).append(
                ", Klassenbezeichnung = *").append(klasse       ).append("* ]");

        return  new String(text);
    }
}
