/*
 * Kunde.java
 */
package at.beringer.oopBuch.organisation;

import java.time.LocalDate;

/**
Instanzen dieser Klasse repräsentieren Kunden.

<hr />

@since August 1999
@version April 2014 (aktualisieren Java 8 - 'GregorianCalendar' ersetzen)

@see Person

@author Beringer Alfred
*/
public class Kunde extends Person
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private float  jahresumsatz = 0.0f;
    private float  kreditlimit  = 0.0f;
    
    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    @see Person#Person(String, LocalDate, char, String)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    @param  adresse       Adresse
    */
    public Kunde(String     name,
                 LocalDate  geburtsdatum,
                 char       geschlecht,
                 String     adresse)
    {
        super(name, geburtsdatum, geschlecht, adresse);
    }

    // ----------

    /**
    @see Person#Person(String, LocalDate, char)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Kunde(String name, LocalDate geburtsdatum, char geschlecht)
    {
        this(name, geburtsdatum, geschlecht, "");
    }

    // ----------

    /**
    @see Person#Person(LocalDate, char)

    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Kunde(LocalDate geburtsdatum, char geschlecht)
    {
        this("", geburtsdatum, geschlecht, "");
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den Jahresumsatz des Kunden.

    @return  Jahresumsatz
    */
    public float getJahresumsatz()
    {
        return  jahresumsatz;
    }

    // ----------

    /**
    Ändert den Jahresumsatz des Kunden.

    @param  jahresumsatz  neuer Jahresumsatz
    */
    public void setJahresumsatz(float jahresumsatz)
    {
        this.jahresumsatz = jahresumsatz;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Kreditlimit des Kunden.

    @return  Kreditlimit
    */
    public float getKreditlimit()
    {
        return  kreditlimit;
    }

    // ----------

    /**
    Ändert das Kreditlimit des Kunden.

    @param  kreditlimit  neues Kreditlimit
    */
    public void setKreditlimit(float kreditlimit)
    {
        this.kreditlimit = kreditlimit;
    }
}
