/*
 * Manager.java
 */
package at.beringer.oopBuch.organisation;

import java.time.LocalDate;

/**
Instanzen dieser Klasse repräsentieren Manager einer Organisation.

<hr />

@since August 1999
@version April 2014 (aktualisieren Java 8 - 'GregorianCalendar' ersetzen)

@see Person
@see Mitarbeiter
@see Organisation
@see OrganisationTesten

@author Beringer Alfred
*/
public class Manager extends Mitarbeiter
{
    // -------------------------------------------------------------------------
    // Klassenattribute
    // -------------------------------------------------------------------------

    /**
    Klasseninstanzattribut für Höchstgehalt
    (Klassenattribut aus der Klasse 'Mitarbeiter').

    @see Mitarbeiter#MaxGehalt
    */
    private static float  MaxGehalt;

    /**
    Klassenattribut für Gewinnanteil.
    */
    private static float  Gewinnanteil;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private Organisation  geleiteteOrganisation;

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenattribute könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    */
    static
    {
        setMaxGehalt(15000.0f);
        setGewinnanteil(0.05f);
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    @see Person#Person(String, LocalDate, char, String)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    @param  adresse       Adresse
    */
    public Manager(String     name,
                   LocalDate  geburtsdatum,
                   char       geschlecht,
                   String     adresse)
    {
        super(name, geburtsdatum, geschlecht, adresse);
        setStellung("Leiter");
        setGrundgehalt(5000.0f);
    }

    // ----------

    /**
    @see Person#Person(String, LocalDate, char)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Manager(String name, LocalDate geburtsdatum, char geschlecht)
    {
        this(name, geburtsdatum, geschlecht, "");
    }

    // ----------

    /**
    @see Person#Person(LocalDate, char)

    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Manager(LocalDate geburtsdatum, char geschlecht)
    {
        this("", geburtsdatum, geschlecht, "");
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die vom Manager geleitete Organisation.

    @return  geleitete Organisation
    */
    public Organisation getGeleiteteOrganisation()
    {
        return  geleiteteOrganisation;
    }

    // ----------

    /**
    Ändert die vom Manager geleitete Organisation, ändert gegebenenfalls seinen
    Status in der Organisation.

    @param  geleiteteOrganisation  neue geleitete Organisation
    */
    public void setGeleiteteOrganisation(Organisation geleiteteOrganisation)
    {
        if ( getOrganisation() != null )
        {
            getOrganisation().removeMitarbeiter(this);
        }
        setOrganisation(geleiteteOrganisation);
        this.geleiteteOrganisation = geleiteteOrganisation;
        if ( geleiteteOrganisation != null )
        {
            geleiteteOrganisation.setLeiter(this);
        }
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert den Verdienst des Managers (= Grundgehalt + Prämie).
    <p>
    Überschreibt die Methode 'verdient' der Klasse 'Mitarbeiter'.
    </p>

    @see Mitarbeiter#verdient()

    @return  Verdienst
    */
    @Override
    public float verdient()
    {
        return  getGrundgehalt()
            + ( geleiteteOrganisation.gewinnOhnePraemie() * getGewinnanteil() );
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert das Höchstgehalt eines Managers.
    <p>
    Da die Klassen 'Manager' und 'Mitarbeiter' jeweils eigene Klassenvariablen
    'MaxGehalt' haben, müssen auch in jeder Klasse eigene Zugriffsmethoden
    definiert werden.
    </p>

    @see Mitarbeiter#getMaxGehalt()

    @return  Höchstgehalt
    */
    public static float getMaxGehalt()
    {
        return  MaxGehalt;
    }

    // ----------

    /**
    Ändert die Höchstgrenze für das Grundgehalt eines Managers.

    @param  maxGehalt  neues Höchstgehalt
    */
    public static void setMaxGehalt(float maxGehalt)
    {
        MaxGehalt = maxGehalt;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den Gewinnanteil eines Managers.

    @return  Gewinnanteil
    */
    public static float getGewinnanteil()
    {
        return  Gewinnanteil;
    }

    // ----------

    /**
    Ändert den Gewinnanteil eines Managers.

    @param  gewinnanteil  neuer Gewinnanteil
    */
    public static void setGewinnanteil(float gewinnanteil)
    {
        Gewinnanteil = gewinnanteil;
    }
}
