/*
 * Person7.java
 */
package at.beringer.oopBuch.organisation;

import java.util.*;

/**
Jede Instanz dieser Klasse repräsentiert eine natürliche Person 
(alte Version bis JDK 7).

<p>
Diese Klasse verwendet die JDK-Klasse 'GregorianCalendar' und wurde nicht mit 
Java 8 aktualisiert. Abgesehen davon ist diese Klasse daher identisch mit der 
Klasse 'Person'.
</p>

<p>
Zusätzlich wurde die 'toString'-Methode implementiert.
</p>

<hr />

@since August 1999
@version April 2002 (überschreiben 'toString')
@version <br />Juli 2003 (öffentliche Klassenkonstante mit 
                          Initialisierungsmethode)

@see Person

@author Beringer Alfred
*/
public class Person7
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /**
    Kennzeichen für das Geschlecht "männlich"
    */
    public static final char    MAENNLICH;

    /**
    Kennzeichen für das Geschlecht "weiblich"
    */
    public static final char    WEIBLICH;

    /**
    Anrede für Personen männlichen Geschlechts
    */
    public static final String  ANREDEMAENNLICH;

    /**
    Anrede für Personen weiblichen Geschlechts
    */
    public static final String  ANREDEWEIBLICH;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String             name;
    private GregorianCalendar  geburtsdatum;
    private char               geschlecht;
    private String             adresse;
    
    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenkonstante könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    <dl>
      <dt>Hinweis:</dt>
        <dd>Nur in dieser Methode sind Zuweisungen für Klassenkonstante 
            erlaubt!</dd>
    </dl>
    */
    static
    {
        MAENNLICH = 'M';
        WEIBLICH  = 'W';
        ANREDEMAENNLICH = "Herr";
        ANREDEWEIBLICH  = "Frau";
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht haben.
    Zusätzlich können der Name und die Adresse angegeben werden. <br />
    Standardkonstruktor nicht erlaubt!

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    @param  adresse       Adresse
    */
    public Person7(String             name,
                   GregorianCalendar  geburtsdatum,
                   char               geschlecht,
                   String             adresse)
    {
        setName(name);
        setGeburtsdatum(geburtsdatum);
        setGeschlecht(geschlecht);
        setAdresse(adresse);
    }

    // ----------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht haben.
    Zusätzlich kann der Name angegeben werden. <br />
    Die Adresse wird mit einer leeren Zeichenkette initialisiert. <br />
    Standardkonstruktor nicht erlaubt!

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Person7(String name, GregorianCalendar geburtsdatum, char geschlecht)
    {
        this(name, geburtsdatum, geschlecht, "");
    }

    // ----------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht 
    haben. <br />
    Name und Adresse werden mit einer leeren Zeichenkette initialisiert. <br />
    Standardkonstruktor nicht erlaubt!

    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Person7(GregorianCalendar geburtsdatum, char geschlecht)
    {
        this("", geburtsdatum, geschlecht, "");
    }

    // -------------------------------------------------------------------------
    // Öffentliche Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den Namen der Person.

    @return  Name
    */
    public String getName()
    {
        return  name;
    }

    // ----------

    /**
    Ändert den Namen der Person.

    @param  name  neuer Name
    */
    public void setName(String name)
    {
        this.name = name;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geburtsdatum der Person.

    @return  Kopie des Geburtsdatums
    */
    public GregorianCalendar getGeburtsdatum()
    {
        // Achtung: Nur eine Kopie des Originalobjekts wird zurückgegeben!
        return  (GregorianCalendar) geburtsdatum.clone();
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geschlecht der Person.

    @return  Geschlecht
    */
    public char getGeschlecht()
    {
        return  geschlecht;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Adresse der Person.

    @return  Adresse
    */
    public String getAdresse()
    {
        return  adresse;
    }

    // ----------

    /**
    Ändert die Adresse der Person.

    @param  adresse  neue Adresse
    */
    public void setAdresse(String adresse)
    {
        this.adresse = adresse;
    }

    // -------------------------------------------------------------------------
    // Sonstige öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert die Anrede für die Person.

    @return  Anrede entsprechend dem Geschlecht bzw. leere Zeichenkette, falls
                                                     das Geschlecht falsch ist
    */
    public String anrede()
    {
        if (geschlecht == MAENNLICH)
        {
            return ANREDEMAENNLICH;
        }

        if (geschlecht == WEIBLICH)
        {
            return ANREDEWEIBLICH;
        }

        return "";
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet das Alter der Person.

    @return  Alter in Jahren
    */
    public int alterBerechnen()
    {
        GregorianCalendar  heute = new GregorianCalendar();
      
        return  heute.get(Calendar.YEAR) - geburtsdatum.get(Calendar.YEAR);
    }

    // -------------------------------------------------------------------------
    // Methode 'toString'
    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'String Object.toString()'.
    <p>
    Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
    <pre>
         Person [ Name = ..., 
                  Geschlecht = ..., 
                  Geburtsdatum = tt. mm. jj,
                  Adresse = ... ]
    </pre>
    </p>

    @see java.lang.Object#toString()

    @return  Zeichenkette, die eine Instanz beschreibt
    */
    @Override
    public String toString()
    {
        StringBuilder      text  = new StringBuilder();
        GregorianCalendar  datum = getGeburtsdatum();

        // Zugriff auf Attribute nur über Zugriffsmethoden (Datenkapselung)!
        text.append(getClass().getName()).append(
                     " [ Name = "       ).append(getName()      ).append(
                     ", Geschlecht = "  ).append(getGeschlecht()).append(
                     ", Geburtsdatum = ").append(
                          datum.get(Calendar.DAY_OF_MONTH)).append(". ").append(
                         (datum.get(Calendar.MONTH) + 1)  ).append(". ").append(
                          datum.get(Calendar.YEAR)        ).append(
                     ", Adresse = "     ).append(getAdresse()   ).append(
                     " ]" );

        return  new String(text);
    }

    // -------------------------------------------------------------------------
    // Private Zugriffsmethoden
    // -------------------------------------------------------------------------

    /**
    Ändert das Geburtsdatum der Person.
    <p>
    Das neue Geburtsdatum muss vor dem aktuellen Tagesdatum liegen.
    </p>

    @param  geburtsdatum  neues Geburtsdatum

    @return  true, wenn 'geburtsdatum' ein gültiges Geburtsdatum ist
    */
    private boolean setGeburtsdatum(GregorianCalendar geburtsdatum)
    {
        GregorianCalendar  heute = new GregorianCalendar();

        if (geburtsdatum.before(heute))
        {
            // Achtung: Nur Kopie des Parameterobjekts verwenden!
            this.geburtsdatum = (GregorianCalendar) geburtsdatum.clone();
            return  true;
        }

        // ungültiges Geburtsdatum
        this.geburtsdatum = heute;
        return  false;
    }

    // -------------------------------------------------------------------------

    /**
    Ändert das Geschlecht der Person.
    <p>
    Prüft, ob das Geschlecht gültig ist (Kennzeichen muss ein gültiges 
    Geschlechtskennzeichen - männlich bzw. weiblich - sein).
    </p>

    @param  geschlecht  neues Geschlecht (männlich oder weiblich)

    @return  true, wenn 'geschlecht' ein gültiges Geschlechtskennzeichen ist
    */
    private boolean setGeschlecht(char geschlecht)
    {
        this.geschlecht = geschlecht;
        return  ((geschlecht == MAENNLICH) || (geschlecht == WEIBLICH)) ? true
                                                                        : false;
    }
}
