/*
 * Projekt.java
 */
package at.beringer.oopBuch.organisation;

/**
Instanzen dieser Klasse repräsentieren Projekte einer Organisation.

<hr />

@since August 1999

@see Organisation
@see OrganisationTesten

@author Beringer Alfred
*/
public class Projekt extends Organisation
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private int    planZeit         = 1;
    private float  planJahresgewinn = 0.0f;
    private int    planLebensdauer  = 0;

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die geplante Zeit des Projekts in Jahren.

    @return  Planzeit in Jahren
    */
    public int getPlanZeit()
    {
        return  planZeit;
    }

    // ----------

    /**
    Ändert die geplante Zeit des Projekts.

    @param  planZeit  neue Planzeit in Jahren
    */
    public void setPlanZeit(int planZeit)
    {
        this.planZeit = planZeit;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den geplanten Jahresgewinn des Projekts.

    @return  geplanter Jahresgewinn
    */
    public float getPlanJahresgewinn()
    {
        return  planJahresgewinn;
    }

    // ----------

    /**
    Ändert den geplanten Jahresgewinn des Projekts.

    @param  planJahresgewinn  neuer geplanter Jahresgewinn des Projekts
    */
    public void setPlanJahresgewinn(float planJahresgewinn)
    {
        this.planJahresgewinn = planJahresgewinn;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die geplante Lebensdauer des Projekts in Jahren.

    @return  geplante Lebensdauer in Jahren
    */
    public int getPlanLebensdauer()
    {
        return  planLebensdauer;
    }

    // ----------

    /**
    Ändert die geplante Lebensdauer des Projekts.

    @param  planLebensdauer  neue Lebensdauer in Jahren
    */
    public void setPlanLebensdauer(int planLebensdauer)
    {
        this.planLebensdauer = planLebensdauer;
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Berechnet den Gewinn des Projekts.

    @see Organisation#gewinnOhnePraemie()
    @see Abteilung#gewinnOhnePraemie()

    @return  Projektgewinn - Gesamtkosten ohne Prämie
    */
    @Override
    public float gewinnOhnePraemie()
    {
        return  (planJahresgewinn * planLebensdauer)
                        / (planZeit * 12) - gesamtkostenOhnePraemie() ;
    }
}
