/*
 * LesenAusgebenJTableV.java
 */
package at.beringer.oopBuch.viewKlassen;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import java.util.*;

/**
Einfaches Swing-Fenster für Ausgeben (Schreiben) von Daten in eine 
Tabelle (JTable).<br />
Die Datensätze werden einzeln der Reihe nach in eine Tabelle geschrieben.
<p>
Bemerkungen:
<ul>
  <li>Definition eines 'Default-Buttons'.</li>
  <li>Definition eines 'Escape-Buttons'.</li>
</ul>
</p>

<hr />

@since Juli 2014

@see at.beringer.oopBuch.einAusgabeSwingView.LesenAusgebenJTableV
@see at.beringer.oopBuch.jdbcSwingView.LesenDBAusgebenJTableV

@author Beringer Alfred
*/
public abstract class AusgebenJTableV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    protected DefaultTableModel  tabelleDTM           = new DefaultTableModel();
    protected JTable             table                = new JTable(tabelleDTM);
    protected JScrollPane        tablescroll          = new JScrollPane(table);
    protected Vector<String>     tabellenueberschrift = new Vector<String>();

    protected Vector<Vector<String>>  alleZeilen = new Vector<Vector<String>>();

    protected JButton  butNext = new JButton("  Nächster Satz  ");
    protected JButton  butEnde = new JButton("  Ende  ");

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public AusgebenJTableV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenausgabe in einer JTable");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butNext);

        initTabelle();
        initFenster();
        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    JTable erstellen.
    */
    private void initTabelle()
    {
        tabellenueberschrift.add("Name (variable Länge");
        tabellenueberschrift.add("Notendurchschnitt");
        tabellenueberschrift.add("Klasse (fixe Länge");

        tabelleDTM.setDataVector(alleZeilen, tabellenueberschrift);
        
        // RowSorter für Tabelle setzen (neu seit JDK 6.0).
        // Damit können durch Klick auf eine Spalte die Zeilen sortiert werden,
        // nochmaliges Klicken dreht die Sortierfolge um.
        // Achtung:
        //    Sortierung erfolgt für Zeichenketten (von links nach rechts)!
        RowSorter<TableModel>  sorter = 
                                     new TableRowSorter<TableModel>(tabelleDTM);
        table.setRowSorter(sorter);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        // oben --------------------------------
        JPanel oben = new JPanel();

        oben.setLayout(new FlowLayout());
        oben.add(butNext);

        this.add(oben, BorderLayout.NORTH);

        // Mitte --------------------------------
        this.add(tablescroll);

        // unten --------------------------------
        JPanel unten = new JPanel();

        unten.setLayout(new FlowLayout());
        unten.add(butEnde);

        this.add(unten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });

        butNext.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent evt)
                {
                    lesen(evt);
                }
            });

        butEnde.addActionListener(new ActionListener()
            {
                @Override
                public void actionPerformed(ActionEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });
    }

    // *************************************************************************

    /**
    Definiert 'Escape'-Taste für 'Ende'-Schaltfläche. <br />

    'Escape'-Taste ist gültig im gesamten Fenster (unabh. vom Focus).
    */
    @Override
    protected JRootPane createRootPane()
    {
        JRootPane  rootpane = new JRootPane();

        rootpane.registerKeyboardAction(
                new ActionListener()
                {
                    @Override
                    public void actionPerformed(ActionEvent evt)
                    {
                        fensterSchliessen(evt);
                    }
                },
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW);

        return rootpane;
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Lesen eines Datensatzes.

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    protected abstract void lesen(ActionEvent evt);

    // *************************************************************************

    /**
    Beenden der Applikation.

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    protected abstract void fensterSchliessen(AWTEvent evt);
}
