/*
 * AusgebenStringV.java
 */
package at.beringer.oopBuch.viewKlassen;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
Sehr Einfaches Swing-Fenster für Anzeigen von Daten in einer Liste (JList) - 
alle auf einmal.
<p>
Bemerkungen:
<ul>
  <li>Definition eines 'Default-Buttons'.</li>
</ul>
</p>

<hr />

@since Juli 2014

@see at.beringer.oopBuch.einAusgabeSwingView.LesenAusgebenStringV
@see at.beringer.oopBuch.jdbcSwingView.LesenDBAusgebenStringV

@author Beringer Alfred
*/
public abstract class AusgebenStringV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    protected DefaultListModel<String>  listeDaten = new DefaultListModel<>();
    protected JList<String>             liste      = new JList<>(listeDaten);
    protected JScrollPane               scrListe   = new JScrollPane(liste);

    protected JButton  butEnde = new JButton("  Ende  ");

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters, sowie das Anmelden bei den Ereigniswächtern 
    wird in private Methoden ausgelagert.
    */
    public AusgebenStringV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenausgabe in einer JList");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butEnde);

        initListe();
        initFenster();
        initListener();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    JList initialisieren.
    */
    private void initListe()
    {
        // Nichtproportionale Schrift festlegen (für spaltenweise Aufbereitung)
        liste.setFont( new Font(Font.MONOSPACED,
                                liste.getFont().getStyle(),
                                liste.getFont().getSize()) );
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());

        // Layout des Fensters definieren und Komponenten dem Fenster hinzufügen

        // Mitte --------------------------------
        this.add(scrListe);

        // unten --------------------------------
        JPanel unten = new JPanel();

        unten.setLayout(new FlowLayout());
        unten.add(butEnde);

        this.add(unten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });

        butEnde.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     fensterSchliessen(evt);
                 }
             });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Beenden der Applikation.

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    protected abstract void fensterSchliessen(AWTEvent evt);
}
