/*
 * ErfassenV.java
 */
package at.beringer.oopBuch.viewKlassen;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
Einfaches Swing-Fenster für Erfassen von Schülerdaten.
<p>
Bemerkungen:
<ul>
  <li>Definition eines 'Default-Buttons'.</li>
</ul>
</p>

<hr />

@since Juli 2014

@see at.beringer.oopBuch.einAusgabeSwingView.ErfassenSchreibenV
@see at.beringer.oopBuch.jdbcSwingView.ErfassenSchreibenDBV

@author Beringer Alfred
*/
public abstract class ErfassenV extends JFrame
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    protected JLabel      labName     = new JLabel("Name: ");
    protected JTextField  txfName     = new JTextField(15);

    protected JLabel      labNote     = new JLabel("Notenschnitt: ");
    protected JTextField  txfNote     = new JTextField(10);

    protected JLabel      labKlasse   = new JLabel("Klasse: ");
    protected JTextField  txfKlasse   = new JTextField(10);

    protected JButton     butErfassen = new JButton("Erfassen");
    protected JButton     butEnde     = new JButton("  Ende  ");

    // ----------

    protected DefaultListModel<String>  listeDaten = new DefaultListModel<>();
    protected JList<String>             liste      = new JList<>(listeDaten);
    protected JScrollPane               scrListe   = new JScrollPane(liste);

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Das Erstellen des Fensters wird in private Methoden ausgelagert.
    */
    public ErfassenV()
    {
        setSize(600, 500);
        setTitle("Fenster für Datenerfassung");

        // Keine 'DefaultCloseOperation'!

        // Standardschaltfläche definieren (Eingabe mit RETURN-Taste möglich)
        this.getRootPane().setDefaultButton(butErfassen);

        initListener();
        initFenster();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten zusammenbauen.
    */
    private void initFenster()
    {
        this.setLayout(new BorderLayout());
        
        // oben ---------------------------------
        JPanel pOben   = new JPanel();

        JPanel pName   = new JPanel();
        JPanel pNote   = new JPanel();
        JPanel pKlasse = new JPanel();

        // Initialisieren Panels für 'pOben'
        initPanelMitBeschriftung(pName,   labName,   txfName);
        initPanelMitBeschriftung(pNote,   labNote,   txfNote);
        initPanelMitBeschriftung(pKlasse, labKlasse, txfKlasse);

        pOben.setLayout(new GridLayout(3,2));

        pOben.add(pName);
        pOben.add(pNote);
        pOben.add(pKlasse);

        this.add(pOben, BorderLayout.NORTH);

        // Mitte --------------------------------
        this.add(scrListe);

        // unten --------------------------------
        JPanel pUnten = new JPanel();

        pUnten.setLayout(new FlowLayout());
        pUnten.add(butErfassen);
        pUnten.add(butEnde);
        
        this.add(pUnten, BorderLayout.SOUTH);
    }

    // -------------------------------------------------------------------------

    /**
    Initialisieren Panelzeile für eine GUI-Komponente mit Beschriftung. <br />
    Die Beschriftung steht links - die Komponente rechts daneben.

    @param  p             Panel für GUI-Komponente mit Beschriftung
    @param  beschriftung  Beschriftung (Label)
    @param  komponente    GUI-Komponente
    */
    private void initPanelMitBeschriftung(JPanel      p,
                                          JLabel      beschriftung,
                                          JComponent  komponente)
    {
        JPanel  plinks  = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        JPanel  prechts = new JPanel(new FlowLayout(FlowLayout.LEFT));

        plinks.add(beschriftung);
        prechts.add(komponente);
        p.setLayout(new GridLayout(1,2));
        p.add(plinks);
        p.add(prechts);
    }

    // -------------------------------------------------------------------------

    /**
    GUI-Komponenten bei Ereigniswächtern anmelden.
    <p>
    Alle Ereigniswächter werden durch anonyme Klassen definiert.
    </p>
    <p>
    Die Tätigkeiten in den überschriebenen API-Methoden werden in private
    Methoden ausgelagert, wobei die 'Event'-Objekte der API-Methoden an diese
    privaten Methoden immer weitergegeben werden, sodass Daten vom Ereignis
    zur Verfügung stehen, falls diese benötigt werden.
    </p>
    */
    private void initListener()
    {
        this.addWindowListener(new WindowAdapter()
            {
                @Override
                public void windowClosing(WindowEvent evt)
                {
                    fensterSchliessen(evt);
                }
            });
        butErfassen.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     butErfassenEreignis(evt);
                 }
             });
        butEnde.addActionListener(new ActionListener()
             {
                 @Override
                 public void actionPerformed(ActionEvent evt)
                 {
                     fensterSchliessen(evt);
                 }
             });
    }

    // *************************************************************************
    // Ereignisbehandlungsmethoden
    // *************************************************************************

    /**
    Daten erfassen, eingegebene Daten kontrollieren, korrekte Daten schreiben

    @param  evt  ActionEvent (von API-Methode 'actionPerformed' übernommen)
    */
    protected abstract void butErfassenEreignis(ActionEvent evt);

    // *************************************************************************

    /**
    Beenden der Applikation.

    @param  evt  AWTEvent (von API-Methode 'actionPerformed' bzw. 
                                           'windowClosing' übernommen)
    */
    protected abstract void fensterSchliessen(AWTEvent evt);
}
