<?xml version="1.0"?>

<st-source>
<time-stamp>From VisualWorks® Personal Use Edition, 7.10.1 of 06. Februar 2014 on 31. Mai 2014 at 11:31:39</time-stamp>


<class>
<name>Person</name>
<environment>BeringerEinfacheBsp</environment>
<super>Core.Object</super>
<private>false</private>
<indexed-type>none</indexed-type>
<inst-vars>vorname geburtsjahr geschlecht </inst-vars>
<class-inst-vars></class-inst-vars>
<imports></imports>
<category>Einfache Beispiele</category>
<attributes>
<package>einfacheBsp</package>
</attributes>
</class>

<shared-variable>
<name>WEIBLICH</name>
<environment>BeringerEinfacheBsp.Person</environment>
<private>false</private>
<constant>true</constant>
<category>Klassenvariable</category>
<initializer>$W</initializer>
<attributes>
<package>einfacheBsp</package>
</attributes>
</shared-variable>

<shared-variable>
<name>MAENNLICH</name>
<environment>BeringerEinfacheBsp.Person</environment>
<private>false</private>
<constant>true</constant>
<category>Klassenvariable</category>
<initializer>$M</initializer>
<attributes>
<package>einfacheBsp</package>
</attributes>
</shared-variable>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<methods>
<class-id>BeringerEinfacheBsp.Person class</class-id> <category>instance creation</category>

<body package="einfacheBsp" selector="new">new
	"Eine Personeninstanz kann nur mit Geburtsdatum und Geschlecht erzeugt werden."

	self
		error: 'Keine Daten (mindestens Geburtsdatum und Geschlecht sind anzugeben)'</body>

<body package="einfacheBsp" selector="newGeburtsjahr:geschlecht:">newGeburtsjahr: einJahr geschlecht: kennzeichen
	"Jede Person muss ein Geburtsjahr und ein Geschlecht haben.
	 Standardkonstruktor ist nicht erlaubt."

	^ (super new)
		aendernGeburtsjahr: einJahr;
		aendernGeschlecht: kennzeichen;
		initialize</body>
</methods>

<!-- -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   -->


<comment>
<class-id>BeringerEinfacheBsp.Person</class-id>
<body>Eine einfache Personen-Klasse.
Jede Instanz dieser Klasse repräsentiert eine natürliche Person.
Klassenkonstante für gültige Geschlechtskennzeichen.
- Jede Person hat immer ein Geburtsjahr und ein Geschlecht.
- Geburtsjahr und Geschlecht können nicht geändert werden.
- Geburtsjahr und Geschlecht werden geprüft:
- Geburtsjahr darf nicht größer sein als das laufende Jahr, das Geschlecht darf nur 'M' oder 'W' sein.
Es wurde zusätzlich noch die Methode 'printOn:' überschrieben.

Hinweis: Die Namen der Standardzugriffsmethoden entsprechen NICHT den üblichen 
         Smalltalk- bzw. UML-Konventionen ('attribut' bzw. 'attribut:'). 

Class Variables
	MAENNLICH	&lt;Character&gt;	Kennzeichen für männliches Geschlecht
	WEIBLICH		&lt;Character&gt;	Kennzeichen für weibliches Geschlecht

Instance Variables
	vorname		&lt;String&gt;		Vorname
	geburtsjahr		&lt;Integer&gt;		Geburtsjahr
	geschlecht		&lt;Character&gt;	Geschlecht
</body>
</comment>

<methods>
<class-id>BeringerEinfacheBsp.Person</class-id> <category>accessing</category>

<body package="einfacheBsp" selector="aendernVorname:">aendernVorname: neuerVorname

	vorname := neuerVorname</body>

<body package="einfacheBsp" selector="geboren">geboren

	^ geburtsjahr</body>

<body package="einfacheBsp" selector="hatGeschlecht">hatGeschlecht

	^ geschlecht</body>

<body package="einfacheBsp" selector="heisst">heisst

	^ vorname</body>
</methods>

<methods>
<class-id>BeringerEinfacheBsp.Person</class-id> <category>initialize-release</category>

<body package="einfacheBsp" selector="initialize">initialize
	"Initialisiert ein neu erzeugtes Objekt."

	super initialize.
	^ self</body>
</methods>

<methods>
<class-id>BeringerEinfacheBsp.Person</class-id> <category>printing</category>

<body package="einfacheBsp" selector="printOn:">printOn: aStream
	"Dient zur textuellen Darstellung einer Personeninstanz.
	 Ermöglicht der Methode 'printString' eine sinnvolle Ausgabe."

	aStream
		nextPutAll: self class name , ': ' , self heisst printString , ' (' , self geboren printString , ') '</body>
</methods>

<methods>
<class-id>BeringerEinfacheBsp.Person</class-id> <category>berechnungen</category>

<body package="einfacheBsp" selector="alterBerechnen">alterBerechnen
	"Berechnet das Alter der Person. Date ist eine Standardklasse von VisualWorks."

	^ (Date today year - self geboren)</body>

<body package="einfacheBsp" selector="anrede">anrede
	"Liefert die Anrede für die Person (abhängig vom Geschlecht).
	 Die Anrede ist fix codiert und nicht variabel!"

	(self hatGeschlecht = MAENNLICH)
		ifTrue: [
			^ 'Sehr geehrter Herr'
		].

	^ 'Sehr geehrte Frau'</body>
</methods>

<methods>
<class-id>BeringerEinfacheBsp.Person</class-id> <category>private</category>

<body package="einfacheBsp" selector="aendernGeburtsjahr:">aendernGeburtsjahr: einJahr
	"Setzt das Geburtsjahr der Person.
	 Das Geburtsjahr wird auf Gültigkeit geprüft:   (Geburtsjahr &lt;= laufendes Jahr).
	 Bei einem Fehler wird das Geburtsjahr nicht gesetzt!"

	( einJahr &lt;= Date today year )
		ifTrue: [
			geburtsjahr := einJahr.
			^ true
		].

	"ungültiges Geburtsjahr"
	^ false</body>

<body package="einfacheBsp" selector="aendernGeschlecht:">aendernGeschlecht: kennzeichen
	"Setzt das Geschlecht der Person.
	 Das Geschlecht wird auf Gültigkeit geprüft.
	 Bei einem Fehler wird das Geschlecht nicht gesetzt!"

	( (kennzeichen = MAENNLICH) or: [kennzeichen = WEIBLICH] )
		ifTrue: [
			geschlecht := kennzeichen.
			^ true
		].

	"ungültiges Kennzeichen für Geschlecht"
	^ false</body>
</methods>

<initialize>
<variable-id>BeringerEinfacheBsp.Person.WEIBLICH</variable-id>
</initialize>

<initialize>
<variable-id>BeringerEinfacheBsp.Person.MAENNLICH</variable-id>
</initialize>

</st-source>
